with TEXT_IO; use TEXT_IO;
with G_PAR_OP;

procedure test_op is

  num_pr : constant positive := 4;        -- num of processors
  l_vect : constant positive := 1000;     -- length of vectors
  l_vect_int : constant positive := 100;  -- length of vect-int items

  procedure null_int (the_item: out INTEGER);

  procedure add_int (suma_item: in out INTEGER;
                     the_item: in INTEGER);

  package PAR_INT_SUM is new G_PAR_OP (INTEGER, num_pr, null_int,
                            add_int);
    -- this package enable to work with integer vectors

  type VECT_INT is array (INTEGER range 1..l_vect_int) of INTEGER;

  procedure null_vect_int (the_item: out VECT_INT);

  procedure add_vect_int (suma_item: in out VECT_INT;
                          the_item: in VECT_INT);

  package PAR_INT_VECT_SUM is new G_PAR_OP (VECT_INT, num_pr,
                          null_vect_int, add_vect_int);

    -- this package enable to work with vectors of integer vectors 

  package INT_IO is new INTEGER_IO (INTEGER);

  use INT_IO;

  vect_int_a: PAR_INT_SUM.REF_VECTOR;
  rslt_int_sum: INTEGER;
  vect_vect_int: PAR_INT_VECT_SUM.REF_VECTOR;
  rslt_vect_int_sum: VECT_INT;
  vect_int_1: VECT_INT;

  procedure null_int (the_item: out INTEGER) is
  begin
    the_item := 0;
  end;

  procedure add_int (suma_item: in out INTEGER;
                  the_item: in INTEGER) is
  begin
    suma_item := suma_item + the_item;
  end;

  procedure null_vect_int (the_item: out VECT_INT) is
  begin
    for i in the_item'FIRST..the_item'LAST loop
      the_item(i) := 0;
    end loop;
  end;

  procedure add_vect_int (suma_item: in out VECT_INT;
                          the_item: in VECT_INT) is 
  begin
    for i in the_item'FIRST..the_item'LAST loop 
      suma_item(i) := suma_item(i) + the_item(i);
    end loop;
  end;



begin

    -- sum of vector of integer items

  vect_int_a := new PAR_INT_SUM.VECTOR(1..l_vect);

  for i in 1..l_vect loop
    vect_int_a(i) := 1;
  end loop;

  PAR_INT_SUM.set_part (30);
  PAR_INT_SUM.red_op(vect_int_a, rslt_int_sum);

  put ("sum_vect = ");
  put (rslt_int_sum);
  new_line;

    -- sum of vector whose items are integer vectors 
    -- (i.e integer matrix l_vect_int x l_vect)
    -- the resuls is vector of l_vec_in integer items  

  vect_vect_int := new PAR_INT_VECT_SUM.VECTOR(1..l_vect);

  for i in 1..l_vect_int loop
    vect_int_1(i) := 1;
  end loop;

  for i in 1..l_vect loop
    vect_vect_int(i) := vect_int_1;
  end loop;

  PAR_INT_VECT_SUM.set_part (30);
  PAR_INT_VECT_SUM.red_op (vect_vect_int, rslt_vect_int_sum);

  put ("sum_vect_vect(first) = ");
  put (rslt_vect_int_sum(1));
  new_line;
  put ("sum_vect_vect(last) = ");
  put (rslt_vect_int_sum(l_vect_int));
  new_line;

end test_op;


