// Module of monitor which can repeatedly make sum of
//   a given vector which contains items of the double type   
      
// Internal private class describing "piece of work"
class Cl_part_work {
  public double[] data;      // reference to data vector
  public int beg_of_part;    // initial index of vector part
  public int end_of_part;    // first index behind the part
  public Cl_part_work (double[] par_data, int par_beg,
      int par_end) {
    data = par_data;
    beg_of_part = par_beg;
    end_of_part = par_end;
  }
  public boolean empty () {  // returns true for empty work 
    if (beg_of_part == end_of_part)
      return true;
    else
      return false;
  } 
}


// public class - monitor
public class Cl_parop {

  // inner static class for worker threads
  private static class Cl_worker extends Thread {

    private Cl_parop monitor_ref;    // reference to monitor
                                     //    instance
    private int my_id;               // num. of worker
    private double loc_result;       // local result

    public Cl_worker (Cl_parop par_monitor_ref, int id) {
      monitor_ref = par_monitor_ref;
      my_id = id;
    }

    public void run () {             // worker activity
      while (!monitor_ref.stop_workers) {
        /* worker requests a piece of work */
        Cl_part_work part = monitor_ref.get_work(); // blocking

        /* sum operation */
        loc_result = 0;
        for (int i = part.beg_of_part; i < part.end_of_part; i++)
          loc_result += part.data[i];
        yield();

        /* return of result */
        monitor_ref.inc(loc_result);    // blocking
        System.out.println(my_id +" " + loc_result);
      }
      System.out.println("worker " + my_id + " terminated");
    }
  }


  //  monitor's data members
  private double[] data;       // ref. to data vector
  private int n;               // num of workers
  private int barrier_cnt;     // counter of sync. barrier
  private int step;            // vector part length
  private double op_result;    // result of operation
  private int next_index;      // first index of first free part
  private boolean op_running;  // flag of running op.
  private boolean free_work;   // flag - there is still
                               //   a free part of vector
  private boolean op_command;  // flag - command, set when at least
                               // one worker is working
  private boolean result_ready;  // flag - set up by the worker
                               //   who last leaved inc() operation
  private boolean stop_workers;  // flag to finish workers

  private Cl_worker[] worker;  // references to worker threads


  // monitor's public methods - called from clients

  public Cl_parop (int n_proc, int par_step) { // constructor
    n = n_proc;
    step = par_step;
    free_work = false;
    op_command = false;
    result_ready = false;
    op_running = false;
    stop_workers = false;
    barrier_cnt = 0;
    worker = new Cl_worker[n];
    for (int i = 0; i < n; i++) {
      worker[i] = new Cl_worker (this, i);
      worker[i].start();
    }
  }

  // operation
  public synchronized double op (double[] v) {

    double result_instance;

    while (op_running) try {    // it blocks other clients
      wait ();
    } catch (InterruptedException e) {};

    data = v;                   // data setup for operation
    op_result = 0;
    next_index = 0;
    op_running = true;
    free_work = true;
    op_command = true;
    result_ready = false;

    notifyAll();                // it awakes waiting workers

    while (!result_ready) try {    // it blocks the calling client
      wait ();
    } catch (InterruptedException e) {};

    result_instance = op_result;
    op_running = false;
    notify_All();               // it awakes waiting clients
    return result_instance;
  }

  // method to kill workers - use it only at the end of program !!!
  public double stop () {

    double[] tmp_data = new double[1];  // data for fictive operation
    tmp_data[0] = 1.0;
    stop_workers = true;
    return op(tmp_data);

  }

  public int get_n () {return n;} // returns num of workers

  public int get_step () {return step;}  // returns size of part

  // method to set size of part
  public synchronized void set_step (int par_step) {
    step = par_step;
  }


  // private methods - called from workers

  // method returns a descriptor of vector part to be processed
  private synchronized Cl_part_work get_work () {

    while (!op_command) try {   // it blocks workers before
      wait ();                 //   start of operation
    } catch (InterruptedException e) {};

    Cl_part_work part;

    int end_of_part = next_index + step;

    if (end_of_part >= data.length) {
      end_of_part = data.length;
      free_work = false;
    }

    part = new Cl_part_work (data, next_index, end_of_part);
    next_index = end_of_part;

    return part;
  }

  // method updates global result by adding a local result,
  //   moreover it synchronizes workers at the end of operation
  private synchronized void inc (double par_result) {
    op_result += par_result;
    if (!free_work) {
      barrier_cnt++;
      if (barrier_cnt == n) {  // last worker entering inc()
        op_command = false;
        notifyAll();
      }
      while (op_command) try {  // sync. of workers before
        wait ();              //   the end of operation
      } catch (InterruptedException e) {};
      barrier_cnt--;
      if (barrier_cnt == 0) {  // last worker leaving inc()
        result_ready = true;
        notifyAll();
      }
    }
  }

}
