#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <pvm3.h>

/* PPR - Jaroslav Rychna							          */
/* Zadani cislo 9 - vypocet intenzity elektrickeho pole		  */
/*															  */
/* Farmer zadava workerum postupne sloupecky matice           */
/* a delnici pocitaji intenzitu a pro kazdy prvek matice      */
/* posilaji zpet vysledek - intenzitu i s jejimi souradnicemi */
/* Pouzite kody zprav:                                        */
/*   4 ... broadcast (id sefa  -> delnici)                    */
/*   5 ... odpoved   (id delnika -> sef)                      */
/*   6 ... zadani ukolu (sef -> delnik),                      */
/*   7 ... vysledek     (delnik -> sef),                      */

/* konstanta pi */
#define PI 3.141592654

/**
 * Funkce pocitajici konstantu pro vypocet.
 * Vypoctena hodnota se pouziva v kazdem cyklu.
 */
double pripravKonstantu(double e0, double eR) {
	double vysledek;
	vysledek = 4*PI*e0*eR;

	return(vysledek);
}

int  main(int argc, char *argv[]) {
  int mytid;      		/*  identifikator procesu delnik */
  int chief_tid;  		/*  identifikator procesu sef  */

  double Q;				/* velikost naboje */
  double e0;			/* permitivita vakua */
  double eR;			/* relativni permitivita */
  int odX;				/* zacatek pocitane oblasti v matici */
  int doX;				/* konec pocitane oblasti v matici */
  int rozmerY;			/* rozmerY matice */
  double konstanta;		/* konstanta pouzivana ve vypoctu */
  double vzdalenost;    /* pomocna promenna vypoctu */
  double intenzita;		/* vysledek vypoctu pro jedno policko matice */
  int souradniceZ;		/* souradnice naboje na ose Z */
  int i,j;

  /* Inicializace */
  mytid = pvm_mytid();  /* prihlaseni do PVM */

  /* cekani na zpravu od sefa */
  pvm_recv(-1, 4);               	/* 4 je kod zpravy */
  pvm_upkint(&chief_tid, 1, 1);  	/* zjisti cislo sefa */

  /* kontrola zda je sef ten pravy - totozny s otcem */
  if (chief_tid == pvm_parent()) {
    /* odpoved sefovi */
    pvm_initsend(PvmDataDefault);	/* inic. bufferu */
    pvm_pkint(&mytid, 1, 1);    	/* ulozeni identifikace */
    pvm_send(chief_tid, 5);     	/* kod odpovedi 5 */
  }

  /* delnik pracuje */
  while (1) {                  /* !! zadne odbory */
    pvm_recv (chief_tid, 6);   	/* cteni  prikazu */
    pvm_upkdouble(&Q,1,1);		/* cteni nastaveni vypoctu */
    pvm_upkdouble(&e0,1,1);
    pvm_upkdouble(&eR,1,1);
    pvm_upkint(&odX,1,1);
    pvm_upkint(&doX,1,1);
    pvm_upkint(&rozmerY,1,1);
    pvm_upkint(&souradniceZ,1,1);

	/* priprava konstanty pro vypocet */
	konstanta = pripravKonstantu(e0,eR);

    for(i=odX; i < doX; i++) {
		for(j=0; j < rozmerY; j++) {
			/* vlastni vypocet intenzity */
			vzdalenost = sqrt((i*i)+(j*j))+(souradniceZ*souradniceZ);
			if(vzdalenost > 0) {
				intenzita = Q/(vzdalenost*konstanta);
			} else {
			    intenzita = -1;
			}

			/* odeslani vysledku - jedno policko matice */
			pvm_initsend(PvmDataDefault);
			pvm_pkint(&mytid,1,1);			/* ulozeni identifikace */
			pvm_pkint(&i,1,1);				/* i pozice */
			pvm_pkint(&j,1,1);				/* j pozice */
			pvm_pkdouble(&intenzita,1,1);	/* ulozeni vysledku - intenzity */
			pvm_send(chief_tid, 7);			/* odeslani */
		}
	}
    /* delnik skoncil svoji cinnost, da o tom zpravu */
        i = -1; j = 0; intenzita = 0;
		pvm_initsend(PvmDataDefault);
		pvm_pkint(&mytid,1,1);			/* ulozeni identifikace */
		pvm_pkint(&i,1,1);				/* i pozice */
		pvm_pkint(&j,1,1);				/* j pozice */
		pvm_pkdouble(&intenzita,1,1);	/* ulozeni vysledku - intenzity */
		pvm_send(chief_tid, 7);			/* odeslani */
  }

  /* ukonceni cinnosti */
  pvm_exit();
  return(0);
}

