#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <pvm3.h>

#define ALLOC 												\
	boundary = (double *)malloc(3 * dimension * sizeof(double));	\
	bound = (double *)malloc(3 * dimension * sizeof(double));		\
	coefficients = (double *)malloc(dimension * sizeof(double));	\

/* PVM - ukazka vypocetniho modelu Farmer - Workers           */
/* Sef zadava delnikum postupne cela licha cisla do hodnoty m */
/*   a delnik zkouma, zda je to prvocislo. Vysledkem je  pole */
/*   vsech prvocisel az do hodnoty m.                         */
/* Pouzivane kody zprav:                                      */
/*   4 ... broadcast (id sefa  -> delnici)                    */  
/*   5 ... odpoved   (id delnika -> sef)                      */ 
/*   6 ... zadani ukolu (sef -> delnik),                      */
/*         zaporna hodnota cisla znamena konec cinnosti       */
/*   7 ... vysledek     (delnik -> sef),                      */
/*         zaporna hodnota cisla znamena "neni to prvocislo"  */

int dimension;
int workers;
double *boundary;
double *bound;
double *coefficients;

int userInputData()
{
	workers = 10;
	dimension = 3;
	ALLOC;
	coefficients[0] = 1;
	coefficients[1] = 2;
	coefficients[2] = 3;
	boundary[0] = 0.0; boundary[1] = 1.0; boundary[2] = 0.1; 
	boundary[3] = 0.0; boundary[4] = 1.0; boundary[5] = 0.1; 
	boundary[6] = 0.0; boundary[7] = 1.0; boundary[8] = 0.1; 

	return 0;
}

bool inc(int level) {
	if(bound[level*3 + 1] > (boundary[level*3 + 1] + boundary[level*3 + 2]/2)) {
		if(level+1 == dimension) return true;
		bound[level*3 + 0] = boundary[level*3 + 0];
		bound[level*3 + 1] = boundary[level*3 + 0] + boundary[level*3 + 2];
		bound[level*3 + 2] = boundary[level*3 + 2];
		
		bound[(level+1)*3 + 0] = bound[(level+1)*3 + 1];
		bound[(level+1)*3 + 1] += bound[(level+1)*3 + 2];
		return inc(level+1);
	}
	return false;
}

bool getStepBoundary(double *bound)
{
	bound[0] = bound[1];
	bound[1] += bound[2];
	if(inc(0)) return false;
	return true;
}

bool testData()
{
  int x = 1;
  int i;

  for(i = 0; i < dimension; i++)
	  x *= (int)((boundary[i*dimension + 1] - boundary[i*dimension + 0]) / boundary[i*dimension + 2]);
  if(x < workers) {
	  printf("Zvolte prosim mensi krok integrace nebo snizte pocet delniku.\n");
	  return false;
  }

  // data OK
  memcpy(bound, boundary, 3 * dimension * sizeof(double));
  for(i = 0; i < 3; i++)
	bound[i*dimension+1] = boundary[i*dimension] + boundary[i*dimension+2];
  return true;
}

main() {
  int mytid;      /*  identifikator procesu sef  */
  int tids[256];  /*  identifikatory procesu delnici */
  int tid;
  int i;
  double loc_res;
  double result;      /*  ukazatel na pole vysledku */ 

  userInputData();
  if(!testData()) return;

  mytid = pvm_mytid();  /* prihlaseni do PVM */

  /* vytvoreni delniku */
  for(i = 0; i < workers; i++) {
   if (pvm_spawn("worker", (char**)0, 0, "", 1, &tids[i]))
      printf("Vytvoren delnik %d.\n", tids[i]);
   else
      printf("Chyba pri vytvareni delnika. Cislo chyby %d\n", tids[i]);

  /* uvodni komunikace sefa s delniky (registrace prac. sil) */  
  pvm_initsend(PvmDataDefault);   /* inicializace bufferu */
  pvm_pkint(&mytid, 1, 1);    /* ulozeni identifikace */
  pvm_mcast(tids, workers, 100);  /* broadcast zpravy s kodem 4 */
                              /*              vsem delnikum */
  /* cekani na odezvu od delniku */
  for(i=0; i<workers; i++) { 
    pvm_recv(-1, 101);         /* 5 je kod zpravy */
    pvm_upkint(&tid, 1, 1);  /* cislo delnika */  
    printf("Delnik cislo %d k praci pripraven! \n ", tid);
  }

  // predani koeficientu funkce delnikum
  for(i=0; i<workers; i++) { 
    pvm_initsend(PvmDataDefault);// inic. bufferu 
    pvm_pkint(dimension, 1, 1);
    pvm_send(tids[i], 102);
    printf("Delnik cislo %d brevzal dimenzi! \n ", tid);
  }

  for(i=0; i<workers; i++) { 
	pvm_initsend(PvmDataDefault);// inic. bufferu 
    pvm_pkdouble(coefficients, dimension, 1);
    pvm_send(tids[i], 103);
    printf("Delnik cislo %d brevzal koeficienty! \n ", tid);
  }

  /* pocatecni ukolovani delniku */
  for(i=0; i<workers; i++) { 
    pvm_initsend(PvmDataDefault);   /* inicializace bufferu */
	pvm_pkdouble(&bound, dimension*3, 1);            /* ulozeni hodnoty k */
    pvm_send(tids[i], 104);           /* zaslani zpravy s kodem 6 */
	// neni treba testovat spravnost prideleni - proveden test fci "testData()"
	getStepBoundary(bound);
  }

  /* kdo dokoncil, dostane dalsi ukol */
  while (1) {
    pvm_recv (-1, 105);           /* cekani na vysledek */
    pvm_upkint (&tid, 1, 1);
    pvm_upkdouble (&loc_res, 1, 1);    /* kopirovani do pole vysledku */
	result += loc_res;

    /* zadani dalsiho ukolu */
	if(!getStepBoundary(bound)) break;
    pvm_initsend(PvmDataDefault);   /* inicializace bufferu */
	pvm_pkdouble(&bound, dimension*3, 1);            /* ulozeni hodnoty k */
    pvm_send(tid, 104);               /* zaslani zpravy s kodem 6 */
  }

  /* dobeh rozdelanych ukolu */
  for(i=0; i<workers; i++) { 
    pvm_recv (-1, 105);           /* cekani na vysledek */
    pvm_upkint (&tid, 1, 1);
    pvm_upkint (&loc_res, 1, 1);    /* kopirovani do pole vysledku */
	result += loc_res;
	
	bound[2] = -1.0;
    pvm_initsend(PvmDataDefault);   /* inicializace bufferu */
	pvm_pkdouble(&bound, dimension*3, 1);            /* ulozeni hodnoty k */
    printf("Delnik cislo %d praci ukoncil! \n ", tid);   
  }

  /* tisk vysledku */
  printf("Vysledek integrace: %f\n ", result);

  /* ukonceni cinnosti v PVM */
  pvm_exit();

}

