import java.util.Date;

// Trida pro spravu mezipameti (zasobniku)
// neni potreba fronty, nebot data jsou nezavisla na pozadi
class Stack
{
	private Request[] buffer;		// reference na data zasobniku
	private int index;				// ukazatel do bufferu na dalsi volnou polozku
	private int size;				// maximalni velikost zasobniku
	private long lqi;				// statistika - stresni delka zasobniku
	private long nCount;			// pocet zpracovanych pozadavku
	private long nSendOK;			// pocet uspesne vlozenych pozadavku do zasobniku
	private long nSendNO;			// pocet opakovanych vlozeni do zasobniku
	
	// konstruktor
	// int size - max. velikost zasobniku
	public Stack(int size) {
		this.size = size;
		buffer = new Request[size];		
		index = 0;
		nSendOK = nSendNO = lqi = nCount = 0;
	}
	
	// vraci pocet pozadavku
	public long getCountRequest() {
		return nCount+1;
	}
	
	// vraci vypoctene lq
	public double getLq() {
		return lqi/(double)nCount;
	}
	
	// vraci index jednorazove vlozenych ku opakovane vlkladanym
	public double getIndexNOOK() {
		return (nSendNO / (double)nSendOK) * 100;
	}
	
	// true=zasobnik je prazdny, jinak false
	public boolean isEmpty() {
		return (index == 0) ?  true :  false;
	}
	
	// vlozi pozadavek do zasobniku
	// pokud vlozeni bylo uspesne tak true, jinak false
	synchronized public boolean send(Request req) {
		if(index == size) { nSendNO++; return false; }
		nSendOK++;
		buffer[index++] = req;
		lqi += index;
		nCount++;
		return true;
	}
	
	// odebere pozadavek ze zasobniku
	// pokud je zasobnik prazdny tak je vraceno null
	synchronized public Request recv() {
		if(index == 0) return null;
		return buffer[--index];
	}
}

// Trida obsahujici data funkce (koeficienty, meze, kroky integrace)
abstract class Function
{
	public int dim;					// dimenze funkce -> dimense integrace
	public double[] koef;			// pole koeficientu funkce
	public FromTo[] boundary;		// pole mezi a kroku integrace pro dimenze
	public Stack stackRequest;		// reference na zasobnik
	public double result;			// vysledek integrace
	
	// nacte data ze vstupnich poli, ktera zada uzivatel, do vnitrnich promennych
	// double[] func - pole koeficientu funkce
	// double[] boundary - pole mezi a kroku integrace
	protected void loadData(double[] func, double[] boundary) {
		System.out.println("Nacitam data funkce.");
		dim = func.length;
		koef = new double[dim];
		for(int i=0; i<func.length; i++) koef[i] = func[i];
		
		// prepocet mezi a kroku z jednodimensizonalniho pole do 
		// vnitrnich struktur
		this.boundary = new FromTo[dim];
		int j = 0;
		for(int i=0; i<dim; i++) {
			this.boundary[i] = new FromTo(boundary[j], boundary[j+1], boundary[j+2]);
			j += 3;
		}
	}
	
	// rekonstrukce vysledku z mezivysledku
	// interResult - mezivysledek jednoho klienta
	synchronized public void modifyResult(double interResult) {
		result += interResult;
	}
}

// trida zahrnujici jeden pozadavek servru na zpracovani jednim klientem
// u 3D fce bude dim = 3
class Request
{
	private FromTo[] req;		// pole mezi pro jednotlive dimenze
	private int dim;			// dimenze funkce
	private int index;			// index do pole mezi, ukazuje na prvni volnou polozku
	
	// konstruktor
	// dim - dimense funkce
	public Request(int dim) {
		this.dim = dim;
		req = new FromTo[dim];
	}
	
	// pridani meze pro jednu dimensy
	// vraci true pokud pridani probehlo uspesne, jinak false (neni jiz misto v pozadavku)
	public boolean add(FromTo ft) {
		if(index == dim) return false;
		req[index++] = new FromTo(ft);
		return true;
	}

	// vraci meze jedne dimense, podle indexu ve vstupnich parametrech
	public FromTo get(int index) {
		return req[index];
	}
}

// Trida obsahujici meze odkud kam = interval integrace + krok zpracovani
class FromTo
{
	public double from;			// dolni mez integrace
	public double to;			// horni mez integrace
	public double step;			// krok integrace
	
	// konstruktor1
	public FromTo(double from, double to, double step) {
		this.from = from;
		this.to = to;
		this.step = step;
	}

	// konstruktor2
	public FromTo(FromTo ft) {
		this.from = ft.from;
		this.to = ft.from + ft.step;
		this.step = ft.step;
	}
}

// Trida pro zpracovani-integraci funkce, obsahuje Server a Clienty
class Integral
{
	public Function function;			// reference na danou funkci
	public static int countThread;		// pocet vlaken - zadava uzivatel
	public static int sizeBuffer;		// velikost zasobniku - zadava uzivatel
	public Server server;				// reference na Server
	public Client[] clients;			// pole referenci na Clienty
	public long diff;					// delka trvani vypoctu
	
	// konstruktor
	public Integral(Function function) {
		this.function = function;
	}

	// monitor pro rizeni Server->Client
	// boolean bStatus - true=aplikace wait(), false=aplikace notifyAll()
	synchronized public void monitorCS(boolean bStatus) {
		if(bStatus) {
			try { 
				wait(); 
			} catch(InterruptedException ex) {
				ex.printStackTrace(); 
			}
		} else {
			notifyAll();
		}
	}
	
	// metoda spoustejici integraci (vytvori a spusti Server i Clienty)
	public void integration() {
		// test poctu nastavenych vlaken
		System.out.println("Testuji spravnost zadani.");
		if((countThread < 1) || (countThread > 255)) {
			System.out.println("Pocet vlaken musi byt v rozmezi 1 az 255.");
			return;
		}
		// test velikosti zasobniku
		if(sizeBuffer < 1) {
			System.out.println("Velikost mezipameti musi byt alespon 1.");
			return;
		}
		
		System.out.println("Vytvarim mezipamet.");
		// vytvoreni spojovani mezipameti - zasobniku
		function.stackRequest = new Stack(sizeBuffer);

		System.out.println("Spoustim klienty.");
		// vytvoreni klientu + spusteni
		// po spusteni se clienti uspi a pockaji az je server vzbudi
		clients = new Client[countThread];
		for(int i=0; i<countThread; i++) {
			clients[i] = new Client(this);
			clients[i].start();
		}

		System.out.println("Spoustim server.");
		// vytvoreni servru + spusteni
		server = new Server(this);
		System.out.println("Zahajeni vypoctu.");
		server.start();
		
		Date date = new Date();
		// cekame na dokonceni prace servru
		try {
			server.join();
		} catch(InterruptedException ex) {
			ex.printStackTrace();
		}
		Date date2 = new Date();
		// vypocet doby trvani
		diff = date2.getTime() - date.getTime();
	}
}

// vlakno plnici zasobnik pozadavky pro clienty
class Server extends Thread
{
	private Integral integral;		// reference na integral
	private Function function;		// reference na danou funkci
	private FromTo[] relativ;		// lokalni promena pro vybrane meze vsech dimenzi integrace

	// konstruktor
	public Server(Integral integral) {
		this.integral = integral;
		this.function = integral.function;
		function.result = 0;
		relativ = new FromTo[function.dim];
		for(int i=0; i<function.dim; i++)
			relativ[i] = new FromTo(function.boundary[i]);
	}
	
	// projde intervaly vsech dimenzi, pokud nejaka vybrana dimenze
	// je na konci vstupniho intervalu pak ji nastavi na pocatek vstupniho intervalu
	// a interval v dalsi dimenzi posune o jeden krok vyse, toto je testovano pro
	// vsechny dimenze a pokud dosahne posledni (nejvyssi) dimenze konce vstupniho
	// intervalu pak vraci true -> konec vypoctu
	private boolean inc(int level) {
		if(relativ[level].to > (function.boundary[level].to + function.boundary[level].step/2)) {
			if(level+1 == function.dim) return true;
			relativ[level] = new FromTo(function.boundary[level]);
			
			relativ[level+1].from = relativ[level+1].to;
			relativ[level+1].to += relativ[level+1].step;
			return inc(level+1);
		}
		return false;
	}
	
	// parcovni metoda servru
	public void run() {
		int iCnt = 0;
		do {
			Request req = new Request(function.dim);
			// vlozeni intervalu do pozadavku
			for(int z=0; z<function.dim; z++) req.add(relativ[z]);
			// zvyseni intervalu prvni (nejnizsi) dimenze o krok (step)
			relativ[0].from = relativ[0].to;
			relativ[0].to += relativ[0].step;
			if(inc(0)) {
				// pokud jsou probrany vsechny dimenze do nejvyssich mezi
				// test, zda clienti jiz zpracovali vsechny pozadavky
				while(!function.stackRequest.isEmpty())
					try {
						// buzeni clientu
						integral.monitorCS(false);
						Thread.sleep(100);
					} catch(InterruptedException ex) {
						ex.printStackTrace();
					}
				System.out.println("Vypocet dokoncen.");
				break;
			}
			while(!function.stackRequest.send(req)) {
				// pokud se mi nepodarilo pridat pozadavek do zasobniku
				integral.monitorCS(false);
				yield();
			}
			// az napoprve dostatecne naplnime zasobnik, tak spustime clienty
			if(iCnt < 5*Integral.countThread) {
				integral.monitorCS(false);
				iCnt = 5*Integral.countThread;
			}
		} while(true);
		System.out.println("Uknoncuji klienty.");
		// ukoncovani clientu
		for(int i=0; i<integral.clients.length; i++) {
			integral.clients[i].bEnding = true;
		}
		integral.monitorCS(false);
		// cekani na ukonceni vsech clientu
		try { 
			for(int i=0; i<integral.clients.length; i++) 
				integral.clients[i].join();
		} catch(InterruptedException ex) {
			ex.printStackTrace();
		}
		System.out.println("Uknoncuji server.");
	}
}

// Trida zpracovavajici pozadavky
class Client extends Thread
{
	private Integral integral;		// reference na integral
	private Function function;		// reference na funkci
	private Stack stack;			// reference na zasobnik
	private boolean bEnd;			// ukoncovaci testovaci promenna
	public boolean bEnding;			// ukoncovaci testovaci promenna
	
	// konstruktor
	public Client(Integral integral) {
		this.integral = integral;
		this.function = integral.function;
		this.stack = integral.function.stackRequest;
		bEnding = false;
		bEnd = false;
	}
	
	// metoda vracejici hodnotu funkce f((ax+bx)/2,(ay+by)/2,(az+bz)/2)
	private double func(Request req) {
		double returnFunc = 0.0;
		for(int i=0; i<function.dim; i++) {
			returnFunc += function.koef[i] * ((req.get(i).from + req.get(i).to) / (double)2);
		}
		return returnFunc;
	}
	
	// pracovni metoda Clienta
	public void run() {
		Request req;
		double res;
		
		do {
			req = null;
			while((req == null) && !bEnding && !bEnd) {
				// pokud je zasobnik prazdny, vraci recv null
				req = stack.recv();
				if(req == null) { // client musi zpracovat pozadavky v zasobniku
					if(!bEnding) // pokud je zasobnik prazdny a clienti jeste nekonci->uspani
						integral.monitorCS(true);
					if(bEnding) bEnd = true;  // server nastavi bEnding na true -> klient konci
				}
			}
			if(!bEnd) {	// pokud neni client ukoncen
				res = func(req);
				for(int i=0; i<function.dim; i++) {
					res *= (req.get(i).to - req.get(i).from);
				}
				function.modifyResult(res);
			}
		} while(!bEnd);
	}
}

// funkce 3D
class Fce1 extends Function
{
	public Fce1() {
		// nastaveni poctu pracovnich vlaken
		Integral.countThread = 100;
		// nastaveni velikost mezipameti
		Integral.sizeBuffer = 1000;
		
		// nastaveni integracnich mezi
		// pole brano po trojicich, kde 1 cislo je dolni mez integrace,
		// 2 je horni mez a 3 je krok integrace
		double[] boundary = {0, 1.0, 0.01,
							 0, 1.0, 0.01,
							 0, 1.0, 0.01};
		// nastaveni koeficientu funkce ax + by + cz
		double[] fce = {1,1,1}; //1*x + 1*y + 1*z
		// nacteni vstupnich dat do vnitrnich promennych
		loadData(fce, boundary);
	}
}

// funkce 2D
class Fce2 extends Function
{
	public Fce2() {
		// nastaveni poctu pracovnich vlaken
		Integral.countThread = 10;
		// nastaveni velikost mezipameti
		Integral.sizeBuffer = 100;
		
		// nastaveni integracnich mezi
		// pole brano po trojicich, kde 1 cislo je dolni mez integrace,
		// 2 je horni mez a 3 je krok integrace
		double[] boundary = {0, 1.0, 0.1,
							 0, 1.0, 0.1};
		// nastaveni koeficientu funkce ax + by
		double[] fce = {1,1}; //1*x + 1*y
		// nacteni vstupnich dat do vnitrnich promennych
		loadData(fce, boundary);
	}
}

// funkce 1D
class Fce3 extends Function
{
	public Fce3() {
		// nastaveni poctu pracovnich vlaken
		Integral.countThread = 10;
		// nastaveni velikost mezipameti
		Integral.sizeBuffer = 100;
		
		// nastaveni integracnich mezi
		// pole brano po trojicich, kde 1 cislo je dolni mez integrace,
		// 2 je horni mez a 3 je krok integrace
		double[] boundary = {0, 1.0, 0.001};
		// nastaveni koeficientu funkce ax + by
		double[] fce = {1}; //1*x
		// nacteni vstupnich dat do vnitrnich promennych
		loadData(fce, boundary);
	}
}

public class Integrals
{
	public static void main (String[] args) {
		// vytvoreni/alokace funkce ve specialni tride zdedene od tridy Function
		Function function = new Fce1();
		// vlozeni dat do tridy Integral
		Integral integral = new Integral(function);
		// spusteni integrace
		integral.integration();
		
		System.out.println("Uloha byla rozdelena na "+function.stackRequest.getCountRequest()+" casti.");
		System.out.println("Pocet pracovnich vlaken = "+Integral.countThread);
		System.out.println("Velikost mezipameti (zasobnik) = "+Integral.sizeBuffer);
		System.out.println("lq mezipameti (zasobniku) = "+function.stackRequest.getLq());
		System.out.println("Zasobnik byl plny z "+function.stackRequest.getIndexNOOK()+"% celkoveho casu.");
		System.out.println("Delka vypoctu = "+integral.diff+"ms\n");
		// vypis vysledku integrace
		System.out.println("Vysledek integrovani="+function.result);
		
	}
}