package cz.zcu.fav.kiv.ppr;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * PPR1 
 * cz.zcu.fav.kiv.ppr 
 * Farmer.java
 * 
 * @author kablazka
 */
public class Farmer {
	private static final Logger logger = Logger.getLogger(Farmer.class);

	int numberThreads;
	int numberLines;
	List<String> data;
	String findString;

	Worker[] workers;

	boolean isWork;
	boolean allWorksDone;

	int continueFromLine = 0;
	int countOfPiecesWork;

	ArrayList<Occurence> arrayResults = new ArrayList<Occurence>();

	/**
	 * Constructor of the class Farmer.java
	 * 
	 * @param p_threads
	 * @param p_lines
	 */
	public Farmer(int p_threads, int p_lines) {
		numberThreads = p_threads;
		numberLines = p_lines;

		isWork = false;
		allWorksDone = false;

		/*
		 * Create the array of Workers Start the Workers
		 */
		workers = new Worker[numberThreads];
		for (int i = 0; i < numberThreads; i++) {
			workers[i] = new Worker(this, i);
			workers[i].start();
		}
	}

	/**
	 * waitForWork If is not work, thread waits and if all work is done, return
	 * false
	 * 
	 * @param threadNumber
	 * @return
	 */
	public synchronized boolean waitForWork(int threadNumber) {
		while ((!isWork) && (!allWorksDone)) {
			try {
				logger.trace("Thread No.: " + threadNumber + " is sleeping...");
				wait();
				logger.trace("Thread No.: " + threadNumber + " is wake up.");
			} catch (InterruptedException e) {
				logger.error(e);
			}
		}

		if ((allWorksDone)) {
			/*
			 * All work is done, Workers have to stop.
			 */
			logger.trace("Worker No.: " + threadNumber
					+ " gets info that: ALL WORKS IS DONE!");
			return false;
		} else {
			/*
			 * Work is not done, Workers have to continue in work.
			 */
			logger.trace("Worker No.: " + threadNumber
					+ " gets info that: WORKS IS IN PROGRESS!");
			return true;
		}
	}

	/**
	 * getWork Create new piece of work
	 * 
	 * @return IPieceOfWork
	 */
	public synchronized IPieceOfWork getWork() {
		if (!isWork) {
			return null;
		}

		/**
		 * Working parts is avaialable.
		 */
		int fromLine = continueFromLine;
		int toLine = fromLine + (numberLines - 1);

		/**
		 * Set bound of datas, if are out
		 */
		if (toLine >= data.size() - 1) {
			isWork = false;
			toLine = data.size() - 1;
			// System.out.println("posledni meze - isWork => false");
		}
		continueFromLine = toLine + 1;

		/**
		 * Create new instance of PieceOfWork
		 */
		IPieceOfWork pieceOfWork = new PieceOfWork(data, findString, fromLine,
				toLine);

		return pieceOfWork;
	}

	/**
	 * submitWork This method is called from Worker, when work is done
	 * 
	 * @param results
	 * @param threadId -
	 *            ID of thread
	 */
	public synchronized void submitWork(IPieceOfWork results, int threadId) {

		logger.info(getThreadPrefix(threadId) + "submited works on lines: "
				+ (results.getFromLine()+1) + "-" + (results.getToLine()+1) + ".");

		/**
		 * count of returned work
		 */
		countOfPiecesWork--;

		if (results != null) {
			arrayResults.addAll(results.getArray());
		}	

		/**
		 * All work is returned.
		 */
		if (countOfPiecesWork <= 0) {
			allWorksDone = true;
			logger.trace(getThreadPrefix(threadId)
					+ " After this thread set allWorksDone => true");
			notifyAll();
		}
	}

	/**
	 * getThreadPrefix Writes ID of thread
	 * 
	 * @param threadId
	 * @return
	 */
	private String getThreadPrefix(int threadId) {
		return "Worker No.: " + threadId + " ";
	}

	/**
	 * getStatistics Method wait until all works done, then return results.
	 * 
	 * @param data
	 * @param findString
	 * @return results
	 */
	public synchronized ArrayList<Occurence> getStatistics(List<String> data,
			String findString) {
		this.data = data;
		this.findString = findString;

		/**
		 * Couns number of all works.
		 */
		countOfPiecesWork = data.size() / numberLines;
		if (data.size() % numberLines > 0) {
			countOfPiecesWork++;
		}

		// logger.trace("countOfPiecesWork: " + countOfPiecesWork);

		allWorksDone = false;
		isWork = true;
		notifyAll();

		/**
		 * Wait until all works done
		 */
		while (!allWorksDone) {
			try {
				wait();
			} catch (InterruptedException e) {
				logger.error(e);
			}
		}		
		
		/**
		 * Sort result arrayList.
		 */
		Collections.sort(this.arrayResults);
		

		/**
		 * All is done, returns result.
		 */
		return arrayResults;
	}

}
