package cz.zcu.fav.kiv.ppr;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * PPR1 Inner class for holding piece of work. 
 * cz.zcu.fav.kiv.ppr
 * PieceOfWork.java
 * 
 * @author kablazka
 */
public class PieceOfWork implements IPieceOfWork {
	private static final Logger logger = Logger.getLogger(PieceOfWork.class);
	private static final String QUOTE = "\"";
	List<String> data;
	String findString;
	int fromLine;
	int toLine;
	ArrayList<Occurence> array;

	/**
	 * Constructor of the class PieceOfWork.java
	 * 
	 * @param data
	 * @param findString
	 * @param fromLine
	 * @param toLine
	 */
	public PieceOfWork(List<String> data, String findString, int fromLine,
			int toLine) {
		this.data = data;
		this.findString = findString;
		this.fromLine = fromLine;
		this.toLine = toLine;
		array = new ArrayList<Occurence>();
	}

	/**
	 * processData Method finds strings and save occurences to ArrayList of
	 * objects.
	 */
	public void processData() {
		int found;
		String line;
		int nextPosition = 0;

		for (int i = fromLine; i <= toLine; i++) {
			line = data.get(i);
			found = findIndex(line);

			while (found != -1) {
				/**
				 * Save occurence to the list
				 */
				array.add(new Occurence((i + 1), (found + nextPosition + 1)));
				logger.trace(QUOTE + findString + QUOTE + " was found on line: " + (i + 1)
						+ "  position: " + (found + nextPosition + 1));
				nextPosition = found + nextPosition + findString.length();
				if (nextPosition >= line.length()) {
					found = -1;
				} else {
					found = findIndex(line.substring(nextPosition));
				}

			}
			nextPosition = 0;
		}

	}

	/**
	 * findIndex Return position of find string.
	 * 
	 * @param source
	 * @return -1 if string not found, position of first occurence if string is
	 *         found
	 */
	private int findIndex(String source) {
		return source.indexOf(findString);
	}

	/**
	 * Getters
	 */

	public List<String> getData() {
		return data;
	}

	public String getFindString() {
		return findString;
	}

	public int getFromLine() {
		return fromLine;
	}

	public int getToLine() {
		return toLine;
	}

	public ArrayList<Occurence> getArray() {
		return array;
	}
}
