package cz.zcu.fav.kiv.ppr;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * PPR1 
 * cz.zcu.fav.kiv.ppr 
 * StringFinder.java
 * 
 * @author kablazka
 */
public final class StringFinder {

	private static final Logger logger = Logger.getLogger(StringFinder.class);
	private static final String TAB = "\t";
	private static final String NEWLINE = "\n";
	private static final String QUOTE = "\"";

	private StringFinder() {
		// EMPTY
	}

	/**
	 * main Handle input parameters
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		int numberThreads = 0;
		int numberLines = 0;

		logger.info("StringFinder was started.");

		if (args.length != 4) {
			if (args.length == 0) {
				writeInfo();
				return;
			} else {
				logger.warn("Count of parameters is wrong!");
				writeInfo();
				return;
			}
		}

		try {
			numberThreads = Integer.valueOf(args[0]);
			numberLines = Integer.valueOf(args[1]);
			if ((numberLines < 1) || (numberThreads < 1)) {
				logger.warn("Count of threads or count of work is wrong!");
				writeInfo();
				return;
			}
		} catch (java.lang.NumberFormatException e) {
			logger.warn("Format of parametrs is wrong!");
			writeInfo();
			return;
		}

		/**
		 * Read dataFile to String.
		 */
		List<String> data = readFile(args[3]);
		if (data == null) {
			return;
		}
		logger.info("File " + QUOTE + args[3] + QUOTE
				+ " was readed to the memory.");

		/**
		 * Create new instance of Farmer
		 */
		Farmer farmer = new Farmer(numberThreads, numberLines);

		/**
		 * Method getStatistics returns results of finding
		 */
		ArrayList<Occurence> result = farmer.getStatistics(data, args[2]);

		StringBuffer tmp = new StringBuffer();
		if (result.size() > 0) {
			tmp.append("Final results: " + NEWLINE + TAB + QUOTE + args[2]
					+ QUOTE + " was found in file " + QUOTE + args[3] + QUOTE
					+ " on this line and position:" + NEWLINE);
			for (int i = 0; i < result.size(); i++) {
				tmp.append(TAB + "line:  "
						+ ((IOccurence) result.get(i)).getLine()
						+ "  position:  "
						+ ((IOccurence) result.get(i)).getPosition() + NEWLINE);
			}
			logger.info(tmp);
		} else
			logger.info("Final results: " + NEWLINE + QUOTE + args[2] + QUOTE
					+ " was not found in the file " + QUOTE + args[3] + QUOTE
					+ ".");

		logger.info("StringFinder was ended.");

	}
	/**
	 * writeInfo Method writes important informations about parameters this
	 * program
	 */
	private static void writeInfo() {
		logger
				.info(NEWLINE
						+ "Enter 4 parameters:"
						+ NEWLINE
						+ " 1st parameter: count of threads - integer(minimal is 1) "
						+ NEWLINE
						+ " 2nd parameter: count of piece of work = count of lines - integer(minimal is 1)"
						+ NEWLINE + " 3rd parameter: find string" + NEWLINE
						+ " 4th parameter: absoluth filepath" + NEWLINE
						+ NEWLINE + " StringFinder will exit.");
	}

	/**
	 * readFile Method reads file to the List of Strings
	 * 
	 * @param fileName -
	 *            absoluth path to the file
	 * @return List of Strings
	 */
	private static List<String> readFile(String fileName) {
		List<String> data = null;
		BufferedReader in = null;

		try {
			in = new BufferedReader(new FileReader(fileName));
			String str;
			data = new ArrayList<String>();
			while ((str = in.readLine()) != null) {
				data.add(str);
			}
		} catch (IOException e) {
			logger.error("File " + QUOTE + fileName + QUOTE
					+ " cannot be read!" + NEWLINE + NEWLINE
					+ "StringFinder will exit.");
			return null;
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {
					logger.error("Problem with close of file " + QUOTE
							+ fileName + QUOTE + "!" + NEWLINE + NEWLINE
							+ "StringFinder will exit.");
					return null;
				}
			}
		}
		return data;
	}

}
