/**
 * Semestralni prace z KIV/PPR
 * @autor: Katerina Blazkova
 * @datum: 10.01.2007
 * @email: kablazka@students.zcu.cz
 */

#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <sys/time.h>
#include <string.h>

typedef struct pieceOfWork {
  char *findString;
  int fromLine;
  int toLine;
  char **data;
  char **results;
} PIECEOFWORK;

struct timeval start, end;

int mpi_self;			/* process number */
int mpi_npes;			/* number of process */
PIECEOFWORK pieceOfWork;
int maxLineLength;		/* length of the max. line */
int lineCount;			/* line count in file */
int workLineCount;



/**
 * readFile
 * @param filename 
 * @return 1 if is all OK
 * 0 If err
 */
int readFile(char* fileName) {
  FILE* fr;
  int c;
  int tempCount;
  int i = 0;
  char *line;

  printf("Thread No.: %d: is reading data file: %s\n", mpi_self, fileName);
  fflush(stdout);
  
  if ((fr = fopen(fileName, "r")) == NULL) 
  {
    printf("Thread No.: %d: File %s not found!\n",mpi_self, fileName);
    fflush(stdout);
    return 0;
  }

  // Test reading of file because we need to know file row count
  // and max. length of line also.
  tempCount = 0;
  while ((c = getc(fr)) != EOF) 
  {
    if (c != '\n')
    {
	tempCount++;
    }
    else 
    {
	if (maxLineLength < tempCount)
	{
	  maxLineLength = tempCount;
	}
	tempCount = 0;
	lineCount++;
    }
  }
  
  printf("Thread No.: %d: finished reading file, results are:\n", mpi_self);
  printf("Thread No.: %d: file have: %d line count and %d max. length of line.\n", mpi_self, lineCount, maxLineLength);
  
  if (fclose(fr) == EOF)
  {
    printf("Thread No.: %d: File %s can't be closed!\n", mpi_self, fileName);
    return 0;
  }
    
  if ((fr = fopen(fileName, "r")) == NULL)
  {
    printf("Thread No.: %d: File %s not found!\n", mpi_self, fileName);
    return 0;
  }

  //allocation
  pieceOfWork.data = (char **) malloc(lineCount*sizeof(char*));
  line = (char*) malloc((maxLineLength+3)*sizeof(char));
  
  //printf("Thread No.: %d: File reading again.\n", mpi_self);
  // Reading file into memory.
  for(i = 0; i < lineCount; i++)
  {
    fgets(line, maxLineLength+2, fr);
    pieceOfWork.data[i] = strdup(line);
  }
  
  //kontroll dump
  printf("Thread No.: %d: in memory is:\n", mpi_self);  
  for(i = 0; i < lineCount; i++)
  {
    printf("[%d]%s", i, pieceOfWork.data[i]);
  }  
  
  if (fclose(fr) == EOF)
  {
    printf("Thread No.: %d: File %s can't be closed!\n", mpi_self, fileName);
  }
  
  return 1;
}

/**
 * findString
 * @stringToFind
 */
void farmer(char* stringToFind) {
  int i, tmpInt, lineSend, lineReceive, resultsLength;
  int dataLength, findStringLength;
  MPI_Status status, statusNew;
  char *results;
  
  printf("Thread No.: %d I'm the horrible farmer!\n", mpi_self );
  fflush(stdout);
  
  // Allocating memory for results
  pieceOfWork.results = (char**) malloc(lineCount*sizeof(char*));    
  for (i=0;i<lineCount;i++)
  {
    pieceOfWork.results[i] = NULL;
  }
    
  // Send firsts data
  for(i=1;i<mpi_npes && i <=lineCount;i++)
  {
      dataLength = strlen(pieceOfWork.data[i-1]);
      
      // Send info about data vector length
      MPI_Send(&dataLength, 1, MPI_INT, i, 1, MPI_COMM_WORLD);
      
      // Send data vector to workers
      MPI_Send(pieceOfWork.data[i-1], dataLength, MPI_CHAR, i, 1, MPI_COMM_WORLD);
      
      // Send info about search string.
      findStringLength = strlen(stringToFind);
      MPI_Send(&findStringLength, 1, MPI_INT, i, 1, MPI_COMM_WORLD);
      MPI_Send(stringToFind, findStringLength, MPI_CHAR, i, 1, MPI_COMM_WORLD);
      
      // Send info about line number
      tmpInt = i-1;
      MPI_Send(&tmpInt, 1, MPI_INT, i, 1, MPI_COMM_WORLD);
      //printf("Thread No.: %d Send line=%d with length=%d\n", mpi_self, tmpInt, dataLength);
      fflush(stdout);
  }
  
  // HACK: lineSend & lineReceive are indexed from 0, it means indexed same way as a lines in file.
  lineSend = --i;
  lineReceive = -1;
  
  // Send all data and receive results.
  while (lineSend < lineCount)
  {
    //printf("LineSend:%d\n",lineSend);  
    fflush(stdout);
    // *** Receive part *** //
    // Receive resutls string
    MPI_Recv(&resultsLength, 1, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);

    results = (char*) malloc((resultsLength+1)*sizeof(char));
    MPI_Recv(results, resultsLength, MPI_CHAR, status.MPI_SOURCE, status.MPI_TAG, MPI_COMM_WORLD, &statusNew);

    results[resultsLength] = '\0';
    pieceOfWork.results[status.MPI_TAG] = results;
    lineReceive++;
    
    //printf("Thread No.: %d Gets data from line=%d and resultsLength=%d.\n", mpi_self, status.MPI_TAG, resultsLength);
    fflush(stdout);
    
    // *** Send part *** //
    // Send info about data vector length
    dataLength = strlen(pieceOfWork.data[lineSend]);
    MPI_Send(&dataLength, 1, MPI_INT, status.MPI_SOURCE, 1, MPI_COMM_WORLD);
    
    // Send data vector to workers
    MPI_Send(pieceOfWork.data[lineSend], dataLength, MPI_CHAR, status.MPI_SOURCE, 1, MPI_COMM_WORLD);
    
    //Send info about search string
    findStringLength = strlen(stringToFind);
    MPI_Send(&findStringLength, 1, MPI_INT, status.MPI_SOURCE, 1, MPI_COMM_WORLD);
    MPI_Send(stringToFind, findStringLength, MPI_CHAR, status.MPI_SOURCE, 1, MPI_COMM_WORLD);
    
    //Send info about line number
    MPI_Send(&lineSend, 1, MPI_INT, status.MPI_SOURCE, 1, MPI_COMM_WORLD);
    //printf("Thread No.: %d Send line=%d with length=%d\n", mpi_self, lineSend, dataLength);
    fflush(stdout);
    
    lineSend++;
  }
  
  // receive rest of the results
  while (lineReceive < (lineSend-1))
  {
    //printf("Thread No.: %d LineReceive=%d & lineSend=%d\n",mpi_self, lineReceive, lineSend);
    fflush(stdout);
    
    // *** Receive part *** //
    // Receive results string
    MPI_Recv(&resultsLength, 1, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);
    //printf("Thread No.: %d: Receive info resultsLength=%d and lineNumber=%d.\n", mpi_self, resultsLength, status.MPI_TAG);
    fflush(stdout);

        
    results = (char*) malloc((resultsLength+1)*sizeof(char));
    MPI_Recv(results, resultsLength, MPI_CHAR, status.MPI_SOURCE, status.MPI_TAG, MPI_COMM_WORLD, &statusNew);
    
    results[resultsLength] = '\0';
    pieceOfWork.results[status.MPI_TAG] = results;
    lineReceive++;
    
    //printf("Thread No.: %d Gets data from line=%d and resultsLength=%d.\n", mpi_self, status.MPI_TAG, resultsLength);
    fflush(stdout);
  }
  
  printf("Thread No.: %d Going to kill everybody!\n", mpi_self);
  fflush(stdout);
  
  // Send kill to all workers
  for (i=1;i<mpi_npes;i++)
  {
    dataLength = -1;
    MPI_Send(&dataLength, 1, MPI_INT, i, 1, MPI_COMM_WORLD);
    printf("Thread No.: %d Sending kill to Thread No.=%d\n", mpi_self, i);
    fflush(stdout);
  }
  
  printf("Thread No.: %d I'm going to haven!\n", mpi_self);
  fflush(stdout);
  
  
  // Print results
  printf("\nFINAL RESULTS: string to find: '%s'\n", stringToFind);  
  fflush(stdout);
  printf("--------------------------------------\n");
  fflush(stdout);
  for (i=0;i<lineCount;i++)
  {
     printf("Thread No.: %d Line[%d]=%s\n", mpi_self, i+1, pieceOfWork.results[i]);
     fflush(stdout);
  }
  printf("---------------------------------------\n\n");
  fflush(stdout);
  
 
}

/**
 * worker
 *
 */
void worker() {
    MPI_Status status;
    int dataLength, findStringLength, lineNumber, resultsLength;
    char *data;
    char *stringToFind, *results;
    
        
    printf("Thread No.: %d: I'm a worker\n", mpi_self);
   
  while(1)
  {
    // Receive info about data vector length
    MPI_Recv(&dataLength, 1, MPI_INT, 0, 1, MPI_COMM_WORLD, &status);
    
    if (dataLength < 0)
    {
	printf("Thread No.: %d I'm going to hell!\n", mpi_self);
	fflush(stdout);
	return;
    }
    
    // Receive data vector from horryble farmer
    data = (char*) malloc((dataLength+1)*sizeof(char));
    MPI_Recv(data, dataLength, MPI_CHAR, 0, 1, MPI_COMM_WORLD, &status);
    //printf("Thread No.: %d Receive dataLength=%d & data=%s", mpi_self, dataLength, data);
    fflush(stdout);
    
    // Receive info about search string
    MPI_Recv(&findStringLength, 1, MPI_INT, 0, 1, MPI_COMM_WORLD, &status);
    stringToFind = (char*) malloc((findStringLength+1)*sizeof(char));
    MPI_Recv(stringToFind, findStringLength, MPI_CHAR, 0, 1, MPI_COMM_WORLD, &status);
    stringToFind[findStringLength] = '\0';
    //printf("Thread No.: %d Receive findStringLength=%d and stringToFind='%s'\n", mpi_self, findStringLength, stringToFind);
    fflush(stdout);
    
    // Receive info about line number
    MPI_Recv(&lineNumber, 1, MPI_INT, 0, 1, MPI_COMM_WORLD, &status);
    //printf("Thread No.: %d Received -> SearchString='%s' | LineNumber=%d | Data=%s", mpi_self, stringToFind, lineNumber, data);
    fflush(stdout);
    
    /**
     * Process data here
     */ 
    char *end, *help, *tempOccurence;
    char *occurences = "\0";
    
    help = strstr(data, stringToFind);
    while (help != NULL)
    {
	end = help+strlen(help);
	int position = (help-data)/sizeof(char);
	// Increase only for formatting
	position++; 
	char *buffer = (char*) malloc(dataLength*sizeof(char));
	sprintf(buffer, "%d; ", position);
	tempOccurence = (char*) malloc((strlen(occurences)+strlen(buffer))*sizeof(char));
	if (strlen(occurences)>0)
	{
	 tempOccurence = strdup(strcat(occurences, buffer));
	 occurences = tempOccurence;
	}
	else
	{
	  occurences = strdup(buffer);
	}
        help +=strlen(stringToFind);//shift after string
        help = strstr(help,stringToFind);   
    }										
    
    //printf("Thread No.: %d Find occurences: '%s' on line=%d\n", mpi_self, occurences, lineNumber);
    results = occurences;
    //results = "4 ; 7";    
    
    resultsLength = strlen(results);
    MPI_Send(&resultsLength, 1, MPI_INT, 0, lineNumber, MPI_COMM_WORLD);    
    MPI_Send(results, resultsLength, MPI_CHAR, 0, lineNumber, MPI_COMM_WORLD);
    
    //printf("Thread No.: %d Succesfully send it's data line=%d to farmer.\n", mpi_self, lineNumber);
    fflush(stdout);
  }
    
} //end of worker

/**
 * main
 * argv[0] - program
 * argv[1] - search string
 * argv[2] - file
 */
int main (int argc, char *argv[]) {
  int ok = 0;
  
  // inicialization MPI
  MPI_Init(&argc, &argv);
  

  if(argc == 1)
  { 
    printf("Enter 3 parameters:\n 1st parameter: program\n 2nd parameter: find string\n 3th parameter: file\n");
    fflush(stdout);
    return 0;
  }
  
  if(argc != 3)
  {
    printf("Bad count of parametrs");
    fflush(stdout);
    printf("Enter 3 parameters:\n 1st parameter: program\n 2nd parameter: find string\n 3th parameter: file\n");
    fflush(stdout);
    return 0;
  }	

  // number of process in COM_WORLD  
  MPI_Comm_rank(MPI_COMM_WORLD, &mpi_self);		
										
  // number of process 
  MPI_Comm_size(MPI_COMM_WORLD, &mpi_npes);
  printf("MPI_COMM_WORLD value is: %d\n", mpi_npes);
  fflush(stdout);
   
  if (mpi_self == 0) 
  {
    printf("Thread No.: %d -> I'm the root process!\n",mpi_self);
    fflush(stdout);
  }
  else 
  {
    printf("Thread No.: %d -> I'm the child process!\n", mpi_self);
    fflush(stdout);
  }
    
  if (mpi_self == 0) 
  {
    printf("Thread No.: %d -> Is going to read a file.",mpi_self);
    fflush(stdout);
    ok = readFile(argv[2]);
  }
  
  MPI_Bcast(&ok, 1, MPI_INT, 0, MPI_COMM_WORLD);
    
  if (ok) 
  {
    if (mpi_self == 0) 
    {
      gettimeofday(&start, NULL);
      farmer(argv[1]);
      gettimeofday(&end, NULL);
      float ptime = ((end.tv_sec - start.tv_sec) + (end.tv_usec - start.tv_usec) / 1e6);
      printf("Thread No.: %d Program finished after: %f\n", mpi_self, ptime);      
    }
    else 
    {
      worker();
    }
  }

   MPI_Finalize();
  
  return 0;
}
