import java.util.*;

/**
 *
 * @author Pavel
 */
public class Task {

	/**
	 * Pripravime praci po jednotlivych segmentech
	 * @param queue fronta pro segmenty prace
	 * @param data reference na datovou tridu
	 * @param praceRadku kolik radku najednou se bude zpracovavat tj. jednotka prace
	 * @throws Exception Pokud je vlaken vice nez pocet segmentu prace
	 */
	public void prepareWork(ArrayList<WorkPiece> queue, Data data, int praceRadku, int threads) throws Exception {
		int segmentu = data.getPocetRadku() / praceRadku;  // jednotlive kusy prace tj. skupina radku
		if (data.getPocetRadku() % praceRadku != 0)		// pokud je zbytek po deleni, tak musim zvednout o jedna
			segmentu++;

		if (threads > segmentu)
			throw new Exception("Error: vlaken je vice nez vypoctenych segmentu prace! Flakali by se.");

		for (int i = 0; i < segmentu; i++) {
			int odRad = i * praceRadku;				// indexy v jedne praci zpracovavanych radku
			int doRad = odRad + praceRadku - 1;
			if (i == segmentu - 1) {
				doRad = odRad + (data.getPocetRadku() - (segmentu - 1) * praceRadku) - 1;   // posledni nesoudelny segment prace, index do
			}

			WorkPiece piece = new WorkPiece(odRad, doRad);
			queue.add(piece);

			//System.out.println("priprava segmentu prace " + odRad + " - " + doRad);

		}
	}

    	/**
	 * Paralelni vypocet.
	 * @param threads pocet vlaken delniku
	 * @param praceRadku je jednotka prace, tedy pocet radku pridelenych delnikovi pro jeden vypocet
	 * @param data reference na datovou tridu
	 * @param polePozic je pole indexovane podle radku a zapisi se do nej nalezene pozice hledaneho retezce
	 */
	public void process(int threads, int praceRadku, Data data, ArrayList<Integer> polePozic[]) {
		try {
			ArrayList<WorkPiece> queue = new ArrayList<WorkPiece>();   // fronta s praci

			prepareWork(queue, data, praceRadku, threads);  // pripravime frontu se segmentama prace (resp. staci nam indexy)

			// nastartujeme vsechna vlakna delkniku
			ThreadCounter tc = new ThreadCounter(threads);
			for (int i = 0; i < threads; i++) {
				Thread worker = new WorkingThread(tc, queue, data, polePozic);
				worker.start();
			}

			// cekame dokud vlakna neskonci
			tc.waitForAll();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}

