package cz.keschaf.ppr;

import java.lang.reflect.Constructor;
import java.util.Arrays;

import cz.keschaf.ppr.functions.Function;
import cz.keschaf.ppr.functions.LinearFunction;
import cz.keschaf.ppr.functions.QuadraticFunction;
import cz.keschaf.ppr.functions.StaticFunction;
import cz.keschaf.ppr.threads.Server;

/**
 * Vstupn tda aplikace. Obsahuje pouze main() a pomocn metody.
 * @author Hrdina Vclav, keschaf@pilsfree.net
 *
 */
public class Exec {

	/**
	 * krok vpotu
	 */
	private static double dx;

	/**
	 * poet vlken
	 */
	private static int threadCount;

	/**
	 * zatek intervalu
	 */
	private static double intervalBegin;

	/**
	 * konec intervalu
	 */
	private static double intervalEnd;

	/**
	 * pouit funkce
	 */
	private static Function function;

	/**
	 * Z pedanho Strigu udl double. Pokud to nelze, ukon program.
	 * @param stringToParse String k parsovn
	 * @param whatToParse nzev, co se parsuje. To se vypisuje do chybov hlky pi
	 * selhn
	 * @return double
	 */
	private static double parseDouble(String stringToParse, String whatToParse) {
		try {
			return Double.parseDouble(stringToParse);
		} catch (Exception ex) {
			System.out.println("Chyba. " + whatToParse + " nema spravny format\nPouzijte napovedu");
			System.exit(1);
		}

		//k tomuhle to nikdy nedojde
		return 0;
	}

	/**
	 * Napln polka tdy pedanmi parametry pkazov dky
	 * @param args parametry pkazov dky
	 */
	private static void parseParams(String[] args) {
		try {
			threadCount = Integer.parseInt(args[0]);
		} catch (NumberFormatException ex) {
			System.out.println("Chyba. 1. parametr nema spravny format\nPouzijte napovedu");
			System.exit(1);
		}

//		try {
			//velikost prace pridelovana vlaknum
//		} catch (NumberFormatException ex) {
//	    }
		dx = parseDouble(args[2], "Dx");
		intervalBegin = parseDouble(args[3], "Zacatek intervalu");
		intervalEnd = parseDouble(args[4], "Konec intervalu");

		String[] subArray = new String[args.length - 5];
		System.arraycopy(args, 5, subArray, 0, subArray.length);
		function = getFunction(subArray);
	}

	/**
	 * Vrt funkci vpotu.
	 * @param args argumenty funkce
	 * @return funkce vpotu.
	 */
	private static Function getFunction(String[] args) {
		if ("1".equals(args[0])) {
			return new StaticFunction(1);
		} else if ("2".equals(args[0])) {
			return new LinearFunction(1, 0);
		} else if ("3".equals(args[0])) {
			return new QuadraticFunction(1, 0, 0);
		} else {
			return getOwnFunction(args);
		}
	}
	
	/**
	 * Vytvo vlastn funkci. Vezme argumenty pkazov dky tkajc se 
	 * vlastn funkce, najde ji v classpath, vytvo jej instanci a tu vrt.
	 * @param <T> Tda vlastn funkce
	 * @param args pole argument pkazov dky; prvn je nzev tdy, dal jsou 
	 * 		parametry konstruktoru. Hled konstruktor obsahujc pouze odpovdajc poet double
	 * @return vytvoenou instanci funkce.
	 */
	@SuppressWarnings("unchecked")
	private static <T extends Function> T getOwnFunction(String[] args) {
		Class<T> clazz = null;
		try {
			clazz = (Class<T>) Class.forName(args[0]);
		} catch (ClassNotFoundException ex) {
			throw new IllegalArgumentException("Trida funkce " + args[0] + " nenalezena");
		}
		
			Double[] constructorParamsValues = new Double[args.length - 1];
			for (int i = 0; i < constructorParamsValues.length; i++) {
				try {
					constructorParamsValues[i] = Double.parseDouble(args[i + 1]);
				} catch (NumberFormatException ex) {
					throw new IllegalArgumentException("Chyba. " + (i + 1) + ". parametr vlastni funkce nelze zkonvertovat na desetinne " +
					"cislo");
			}
		}
		
		Class<Double>[] constructorParamsClass = new Class[args.length - 1];
		Arrays.fill(constructorParamsClass, double.class);
		
		Constructor<T> constr = null;
		try {
			constr = clazz.getConstructor(constructorParamsClass);
		} catch (SecurityException ex) {
			throw new RuntimeException("Doslo k chybe pri vytvareni vlastni funkce: " 
					+ ex.getMessage(), ex);
		} catch (NoSuchMethodException ex) {
			throw new IllegalArgumentException("Trida " + args[0] + " nema prislusny konstruktor." +
					"\nHledany konstruktor je s parametry " + Arrays.toString(constructorParamsClass));
		}
		
		try {
			return constr.newInstance((Object[]) constructorParamsValues);
		} catch (Exception ex) {
			throw new RuntimeException("Doslo k chybe pri vytvareni vlastni funkce: " + 
					ex.getMessage(), ex);
		}
	}

	/**
	 * Vstupni bod aplikace.
	 * @param args argumenty aplikace
	 */
	public static void main(String[] args) {
		try {
			if (args.length < 6) {
				printHelp();
			}
			parseParams(args);
			new Server(intervalBegin, intervalEnd, dx, threadCount, function).start();
		} catch (Exception ex) {
			System.out.println("Chyba pri vykonavani programu:\n" + ex.getMessage());
		}
	}

	/**
	 * Pouze vype npovdu na sysout a ukon program.
	 */
	private static void printHelp() {
		System.out.println("Pouziti: run.bat pocetVlaken mnozstviPraceVlakna krokIntervalu zacatekIntervalu konecIntervalu funkce [arguentyVlastniFunkce]");
		System.out.println("Parametry:");
		System.out.println("\tpocetVlaken: pocet vlaken, ktere se budou podilet na vypoctu");
		System.out.println("\tmnozstviPraceVlakna ???");
		System.out.println("\tkrokIntervalu: krok dx, po kterem se bude provadet vypocet");
		System.out.println("\tzacatekIntervalu: zacatek intervalu, na kterem se provadi vypocet");
		System.out.println("\tkonecIntervalu: konec intervalu, na kterem se provadi vypocet");
		System.out.println("\tfunkce: funkce, pro kterou se provadi vypocet:");
		System.out.println("\t\t1: f(x) = 1");
		System.out.println("\t\t2: f(x) = x");
		System.out.println("\t\t3: f(x) = x ^ 2");
		System.out.println("\t\tvlastni: trida vlastni funkce. Je nutne, aby implementovala\n\t\t\trozhrani cz.keschaf.ppr.functions.Function" +
				" a byla\n\t\t\tdostupna v classpath");
		System.out.println("\tparametryVlastniFunkce: parametry konstruktoru vlastni funkce\n");
		
		System.out.println("Priklad:");
		System.out.println("\trun.bat 5 2 0.1 0 5 1");
		System.out.println("\trun.bat 5 2 0.1 0 2 cz.keschaf.ppr.functions.QuadraticFunction 1 1 2\n");
		
		System.out.println("Zabudovane funkce:\t\t\t\tVyjadreni:");
		System.out.println("cz.keschaf.ppr.functions.StaticFunction\t\ty=c");
		System.out.println("cz.keschaf.ppr.functions.LinearFunction\t\ty=kx + q");
		System.out.println("cz.keschaf.ppr.functions.QuadraticFunction\ty=a x^2 + bx + c");
		
		System.exit(1);
	}

}
