package ppr.gui;

import java.text.SimpleDateFormat;
import java.util.Date;

import ppr.threads.app.Parameters;
import ppr.threads.app.ThreadManager;

/**
 * Vlakno pro zobrazeni cekaciho dialog a pro spusteni vypoctu. 
 *
 */
public class DialogThread extends Thread {
	private MainFrame mainFrame = null;
	private Parameters parameters = null;
	
	/**
	 * Vytvori instanci tridy.
	 * @param mainFrame hlavni okno aplikace
	 * @param parameters parametry vypoctu
	 */
	public DialogThread(MainFrame mainFrame, Parameters parameters) {		
		this.mainFrame = mainFrame;
		this.parameters = parameters;
	}
	
	public void run() {
		final Process process = new Process(mainFrame); // dialog s vypisem
		int i = 0;  // pocet tecek
		final int MAX_DOT = 5;  // maximalni pocet tecek     
				
		new Thread(new Runnable() {
			public void run() {
				process.setVisible(true);
			}							
		}).start();   // vlakno s dialogem
		
		ThreadManager threadManager = new ThreadManager(parameters);
		threadManager.setPriority(Thread.MIN_PRIORITY);
		threadManager.start();
										
		// vypis na dialog
		while (threadManager.isAlive()) {
			i = ++i % MAX_DOT;  // nacitani tecek za vypisem
			
			StringBuilder sb = new StringBuilder("Potm");  // string builder kvuli rychlosti
						
			for (int k = 0; k < i; k++) sb.append('.');   // pridani tecek
			
			process.print(sb.toString());  // vypsani retezce na dialog
			
			try {
				Thread.sleep(500);  // uspani
			} catch (InterruptedException e) {}			
		}
		
		process.setVisible(false);
		
		SimpleDateFormat sdf = new SimpleDateFormat("m'm':s's':S'ms'");
		
		// vypis vysledku a casu
		mainFrame.writeText("Doba vpotu: " + sdf.format(new Date(threadManager.getEndTime() - threadManager.getStartTime())));		
		mainFrame.writeText("Celkov hmotnost: " + threadManager.getTotalWeight());
	}
}