/**
 * Farmer.java - vlakno, ktere provadi rozdeleni vypoctu na male casti a ty
 * prideluje jednotlivym workerum. Vysledky sestavuje do konecne hmotnosti.
 * 
 * @author Jiri Frolik, A06081, KIV/PPR 2007/08
 * */


public class Farmer extends Thread {

	private final double ASSIGNED_DONE = 1.0;	// mnozstvi celkove prace k prirazeni = 1 = 100%
	
	private Zpracovani zpracovani;			// objekt s potrebnymi daty
	private Worker[] workers;				// pole vlaken provadejicich vypocet
	
	private double finalWeight;				// sumacni promenna pro vyslednou hmotnost kvadru
	private double assignedTotal;			// mnozstvi jiz pridelene prace (cislo < 1)
	
	private boolean computing;				// probiha pocitani
	
	private long nanosecs;					// mereni casu vypoctu
	
	
	/**
	 * Konstruktor. V objektu zpracovani jsou ulozeny nactene parametry vypoctu.
	 * */
	public Farmer(Zpracovani zpracovani) {
		super();
		
		this.zpracovani = zpracovani;
		this.workers = new Worker[this.zpracovani.getThreadCount()];
		this.finalWeight = 0.0;			// vysledek zatim 0
		this.assignedTotal = 0.0;		// zatim nebyla pridelena zadna prace
		
		this.computing = true;			// zacinam pocitat
		this.nanosecs = 0;
		
		this.initAllWorkers();			// inicializace vsech workeru
	}
	
	/* setters and getters */

	public long getNanosecs() {
		return nanosecs;
	}
	
	public void setNanosecs(long nanosecs) {
		this.nanosecs = nanosecs;
	}
	
	public boolean isComputing() {
		return computing;
	}
	
	public void setComputing(boolean computing) {
		this.computing = computing;
	}
	
	public Worker getWorkerAt(int index) {
		return this.workers[index];
	}
	
	public void setWorkerAt(int index, Worker worker) {
		this.workers[index] = worker;
	}
	
	public Zpracovani getZpracovani() {
		return this.zpracovani;
	}
	
	public double getFinalWeight() {
		return finalWeight;
	}
	
	public void incFinalWeight(double weight) {
		this.finalWeight += weight;
	}
	
	public double getAssignedTotal() {
		return this.assignedTotal;
	}
	
	public void incAssignedTotal(double assigned) {
		this.assignedTotal += assigned;
	}
	
	
	public String toString() {
		return "Farmer: mezivysledek hmotnosti = " + this.finalWeight;
	}
	
	public String toStringFinal() {
		return "Farmer: vysledna hmotnost = " + this.finalWeight;
	}
	
	/* setters and getters - end */
	
	
	/**
	 * Metoda provede naplneni pole Workeru novymi instancemi.
	 * */
	public void initAllWorkers() {
		for (int i = 0; i < this.zpracovani.getThreadCount(); i++) {
			this.workers[i] = new Worker(i);
		}
	}
	
	/**
	 * Metoda prideli cast prace danemu vlaknu workera k provedeni vypoctu dilci hmotnosti.
	 * Metoda je synchronized, protoze prideleni prace workerovi musi byt atomicke.
	 * */
	synchronized private void assignToWorkerAt(int index) {
		
		// dokud je jeste co pocitat - nebylo prideleno celych 100% casti:
		if (this.assignedTotal < this.ASSIGNED_DONE) {
			
			// budu pocitat posledni kousek - dalsi uz presahuje kvadr
			if ((this.assignedTotal + this.zpracovani.getAssignedPart()) >= this.ASSIGNED_DONE) {
				// pridelim cast prace danemu vlaknu:
				this.workers[index].initWorkerData(this.assignedTotal*this.zpracovani.getKvadr().getX(),
						0,0,	// 0 = fromY = fromZ -> sour. pocatku, pocita se po ctvercich podle x
						this.zpracovani.getKvadr().getX(),
						this.zpracovani.getKvadr().getY(),
						this.zpracovani.getKvadr().getZ(),		// koncove souradnice - posledni vypocet
						this.zpracovani.getDx(), this.zpracovani.getDy(),
						this.zpracovani.getDz(), this.zpracovani.getFunctionType());
				
				this.workers[index].setComputing();				// nastaveni stavu - spusti vypocet
				
				this.assignedTotal = this.ASSIGNED_DONE;		// prace skoncena - vsechno prideleno
			}
			else {			// jeste nejsem na konci kvadru:
				// pridelim cast prace danemu vlaknu:
				this.workers[index].initWorkerData(this.assignedTotal*this.zpracovani.getKvadr().getX(),
						0,0,	// 0 = fromY = fromZ -> sour. pocatku, pocita se po ctvercich podle x
						(this.assignedTotal+this.zpracovani.getAssignedPart())*this.zpracovani.getKvadr().getX(),
						this.zpracovani.getKvadr().getY(),
						this.zpracovani.getKvadr().getZ(),
						this.zpracovani.getDx(), this.zpracovani.getDy(),
						this.zpracovani.getDz(), this.zpracovani.getFunctionType());
				
				this.workers[index].setComputing();				// nastaveni stavu - spusti vypcet
				
				this.assignedTotal += this.zpracovani.getAssignedPart();	// zvetsi se odpracovana cast
			}
			

		}
		else {
			// nechat skoncit vlakno daneho workera - neni co pocitat
			this.workers[index].endWork();
		}
		
	}
	
	
	/**
	 * Metoda behu vlakna Farmera.
	 * */
	public void run() {
		super.run();
		
		boolean isWorkLeft = true;				// zbyva prace na prideleni
		
		for (int i = 0; i < this.workers.length; i++) {		// spusteni vsech workeru
			//System.out.println("start");
			this.workers[i].start();
		}


		while (isWorkLeft) {			// dokud pocitam

			// pokud po cyklu for bude isWorkLeft false, vsechna vlakna jsou mrtva:
			isWorkLeft = false;
			
			for (int i = 0; i < this.workers.length; i++) {

				if (this.workers[i].getCurrentState() != Worker.DEAD) {	// vlakno jeste nebylo umrtveno
					//System.out.println(i + " jeste zije");
					// zmenim na true - aspon jedno vlakno zije, cyklus while bude pokracovat:
					isWorkLeft = true;					 
				}
				else {					// vlakno je mrtve
					continue;			// pokracuji testovani dalsiho vlakna
				}
				
				// testuji stav vlaken, pripadne jim pridelim praci:				
				if (this.workers[i].getCurrentState() == Worker.IDLE) {	// prave bylo vytvorene
					this.assignToWorkerAt(i);							// pridelim mu praci
				}
				if (this.workers[i].getCurrentState() == Worker.DONE) {	// prave dopocitalo
					//System.out.println(this.workers[i].getPartWeight());
					this.finalWeight += this.workers[i].getPartWeight();	// pricteni mezivysledku
					this.assignToWorkerAt(i);							// pridelim dalsi praci
				}
				
			}
						
		}
		this.setNanosecs(System.nanoTime());	// zaznamenani casu konce vypoctu
		this.computing = false;				// dopocitano - vsechna vlakna workeru mrtva
		
		System.out.println("farmer konci");
		//System.out.println("Vysledna hmotnost = " + this.finalWeight);
	}

	
}
