/**
 * Hlavni.java - hlavni trida programu. Nacte pozadovane parametry vypoctu hmotnosti kvadru
 * a spusti proces farmera.
 *
 * @author Jiri Frolik, A06081, KIV/PPR 2007/08
 **/


import java.io.*;

public final class Hlavni {

	
	public static final long TIME_TO_SLEEP = 1000;				// kazdou 1s se provede vypis cinnosti farmera
	
	public static final String CONFIG_PATH = "../config/";		// cesta ke konfiguracnimu souboru
	public static String CONFIG_FILE = "config";				// soubor s konfiguraci
	public static final String LOG_FILE = "../log/log";			// logovaci soubor
	
	public static final long NSECS_TO_MSECS = 1000000;			// prevod z ns na ms (kvuli mereni casu)
	
	/**
	 * Metoda nacte z konfiguracniho souboru ciselne hodnoty potrebne k provedeni vypoctu.
	 * Nactene hodnoty ulozi do objektu tridy Zpracovani. 
	 * */
	private static void configureFromFile(Zpracovani zpracovani) throws Exception {
		
		File configFile = new File(CONFIG_PATH.concat(CONFIG_FILE));	// kompletni cesta k souboru
		
		String radka = "";
    
		FileReader freader = new FileReader(configFile);
		BufferedReader bfreader = new BufferedReader(freader);			
		
		zpracovani.getKvadr().setX(Double.parseDouble(bfreader.readLine()));
		zpracovani.getKvadr().setY(Double.parseDouble(bfreader.readLine()));
		zpracovani.getKvadr().setZ(Double.parseDouble(bfreader.readLine()));
		zpracovani.setDx(Double.parseDouble(bfreader.readLine()));
		zpracovani.setDy(Double.parseDouble(bfreader.readLine()));
		zpracovani.setDz(Double.parseDouble(bfreader.readLine()));
		
		zpracovani.setThreadCount(Integer.parseInt(bfreader.readLine()));
		zpracovani.setAssignedPart(Double.parseDouble(bfreader.readLine()));
		zpracovani.setFunctionType(Integer.parseInt(bfreader.readLine()));
		
		bfreader.close();
		freader.close();
		
		// kontrola nactenych hodnot:
		if (zpracovani.checkAttributes() == false) {	// v souboru byly spatne zadane parametry
			throw (new IOException());
		}

		
	}
	
	
	/**
	 * Metoda ulozi do objektu tridy Zpracovani hodnoty nactene jako parametry prikazove radky.
	 * */
	private static void configureFromArgs(Zpracovani zpracovani, String[] args) throws Exception {
		
		zpracovani.getKvadr().setX(Double.parseDouble(args[0]));
		zpracovani.getKvadr().setY(Double.parseDouble(args[1]));
		zpracovani.getKvadr().setZ(Double.parseDouble(args[2]));
		zpracovani.setDx(Double.parseDouble(args[3]));
		
		if (args.length == 7) {			// dy = dz = dx -> zadano pouze dx
			zpracovani.setDy(zpracovani.getDx());
			zpracovani.setDz(zpracovani.getDx());
			zpracovani.setThreadCount(Integer.parseInt(args[4]));
			zpracovani.setAssignedPart(Double.parseDouble(args[5]));
			zpracovani.setFunctionType(Integer.parseInt(args[6]));
		}
		else {							// hodnoty dx, dy a dz jsou ruzne - zadadny zvlast
			zpracovani.setDy(Double.parseDouble(args[4]));
			zpracovani.setDz(Double.parseDouble(args[5]));
			zpracovani.setThreadCount(Integer.parseInt(args[6]));
			zpracovani.setAssignedPart(Double.parseDouble(args[7]));
			zpracovani.setFunctionType(Integer.parseInt(args[8]));
		}
		
		// kontrola nactenych hodnot:
		if (zpracovani.checkAttributes() == false) {	// byly spatne zadane parametry prikazove radky
			throw (new IOException());
		}
		
	}
	
	

	/**
	 * Metoda main() - spusteni programu:
	 * Bez parametru = vypis napovedy ke spusteni programu
	 * Jeden parametr = jmeno konfiguracniho souboru
	 * Sedm parametru = stejny krok dx=dy=dz, zadan pouze dx
	 * Devet parametru = ruzny krok dx, dy, dz (zadany vsechny)
	 * 
	 * Po nacteni vsech parametru metoda spusti Farmera a tim i cely proces pocitani.
	 */
	public static void main(String[] args) {

		Kvadr kvadr = new Kvadr();						// kvadr, kam se uklada nacteny bod (x,y,z)
		Zpracovani zpracovani = new Zpracovani(kvadr);	// objekt zpracovani - uchovava ostatni data
		Farmer farmer;									// farmer bude rozdelovat praci workerum
		
		Logger logger = new Logger(LOG_FILE);			// logovani do souboru LOG_FILE
		
		long nanosecs;									// atribut pro zaznamenani casu vypoctu
		
		// rozhodovani na zaklade poctu parametru prikazove radky:
		switch (args.length) {
			case 0:		// bez parametru - napoveda
				System.out.println(toStringInfo() + "\n");
				return;
			case 1:		// jeden parametr - konfiguracni soubor
				CONFIG_FILE = args[0];
				try {
					configureFromFile(zpracovani);		// parsovani souboru
				} catch (Exception e) {
					System.err.println("Chyba pri cteni ze zadaneho konfiguracniho souboru '" 
							+ CONFIG_FILE + "'!");
					return;
				}		
			break;
			case 7:
			case 9:			// dve moznosti nacitani z parametru prikazove radky 
				try {
					configureFromArgs(zpracovani, args);	// vytazeni parametru vypoctu
				} catch (Exception e) {
					System.err.println("Chybne zadane parametry programu! Pro zobrazeni napovedy\n" +
						"spustte program bez parametru.");
					return;
				}
			break;
			default:		// chybny pocet parametru
				System.err.println("Chybny pocet parametru programu! Pro zobrazeni napovedy\n" +
					"spustte program bez parametru.");
				return;

		}
	
		farmer = new Farmer(zpracovani);			// vytvoreni objektu farmera
			
		try {
			logger.startLog();						// zacatek logovani
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		System.out.println("Nactena data:");
		System.out.println(zpracovani.getKvadr().toString());
		System.out.println(zpracovani.toString());	// kontrolni vypis nactenych hodnot
		logger.zapis("Nactena data:");
		logger.zapis(zpracovani.getKvadr().toString());
		logger.zapis(zpracovani.toString());		// zalogovani nactenych dat
		
		nanosecs = System.nanoTime();				// zaznamenani casu
		farmer.start();								// spusteni vlakna farmera
		
		// Dokud farmer pocita, budu zobrazovat zpravy o prubeznem stavu:
		while (farmer.isComputing()) {
			
			// zaloguji prubezny stav vsech workeru:
			for (int i = 0; i < zpracovani.getThreadCount(); i++) {
				//System.out.println(farmer.getWorkerAt(i).toString());		// vypis
				logger.zapis(farmer.getWorkerAt(i).toString());				// zalogovani
			}
			
			System.out.println(farmer.toString());	// vypis stavu farmera
			logger.zapis(farmer.toString());		// zalogovani momentalniho stavu farmera
			
			
			try {
				Thread.sleep(TIME_TO_SLEEP);		// uspim hlavni vlakno pred dalsim vypisem
			} catch (InterruptedException e) {
			}
		}
		
		System.out.println(farmer.toStringFinal());		// vysledny vypis na konzoli
		logger.zapis(farmer.toStringFinal());			// zapis vysledku do logovaciho souboru
		
		nanosecs = farmer.getNanosecs() - nanosecs;	// odecteni zmerenych hodnot behem vypoctu
		nanosecs /= NSECS_TO_MSECS;					// uprava na milisekundy
		System.out.println("Doba vypoctu: " + nanosecs + "ms.");
		logger.zapis("Doba vypoctu: " + nanosecs + "ms.");
		
		try {
			logger.stopLog();						// uzavreni loggeru
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	
	/**
	 * Metoda vrati retezec, ktery predstavuje napovedu ke spusteni programu.
	 * */
	private static String toStringInfo() {
		return  "\n--- Navod na spusteni programu: ---\n-\n" + 
				"- Program se spousti bud bez parametru (pak bude zobrazena tato napoveda),\n" +
				"- nebo s jednim parametrem (jmenem konfiguracniho souboru nachazejiciho se\n" +
				"- v adresari 'config'), nebo je mozne zadat vsechny parametry primo, ve tvaru:\n-\n" +
				"- run <x> <y> <z> <dx> [<dy> <dz>] <poc_vlaken> <pridel_cast> <typ_funkce>\n-\n" +
				"- Parametry dy a dz lze vynechat, pokud jsou stejne jako <dx>. Napriklad:\n" +
				"- run 5.5 4.4 3.3 0.01 20 0.01 0\n-\n" +
				"- Typ funkce nabyva hodnot 0 / 1 / 2 pro vysledky funkce ro = 1 / x / x+y+z.\n-\n" +
				"- Pozadovana struktura konfiguracniho souboru je uvedena v dokumentaci.\n";
	}

}	/* end of class Hlavni */
