/**
 * Worker.java - vlakno, ktere provadi castecne vypocty hmotnosti kvadru.
 * Parametry vypoctu jsou pridelovany farmerem (Farmer.java).
 * 
 * @author Jiri Frolik, A06081, KIV/PPR 2007/08
 * */


public class Worker extends Thread {

	public static final int IDLE = 0;		// vlakno bylo vytvoreno a ceka na inicializaci dat
	public static final int WAITING = 1;	// vlakno ceka na prideleni prace
	public static final int WORKING = 2;	// vlakno pracuje
	public static final int DONE = 3;		// vlakno ukoncilo vypocet, vysledek lze odebrat 
	public static final int DEAD = 4;		// vlakno bylo "ukonceno", uz nebude pracovat
	
	private int myId;						// ciselny identifikator vlakna
	
	private int currentState;				// aktualni stav vlakna
	
	private double fromX;
	private double fromY;
	private double fromZ;					// x, y, z souradnice, od kterych se pocita
	private double toX;
	private double toY;
	private double toZ;						// x, y, z souradnice, do kterych se pocita
	private double dx;
	private double dy;
	private double dz;						// x, y, z hodnoty prirustku nactene v konfiguraci
	
	private int functionType;				// typ funkce pro pocitani hmotnosti kvadru
	
	private double partWeight;				// spoctena cast hmotnosti kvadru = dM
	
	
	/**
	 * Konstruktor.
	 * */
	public Worker(int id) {
		super();
		
		this.myId = id;						// nastaveni ciselneho identifikatoru
		
		this.currentState = IDLE;			// zacinam ve stavu, kdy cekam na inicializaci
		
		this.partWeight = 0.0;
		this.fromX = 0.0;
		this.fromY = 0.0;
		this.fromZ = 0.0;
		this.toX = 0.0;
		this.toY = 0.0;
		this.toZ = 0.0;						// inicializace vsech hodnot na 0
	}
	
	
	/* setters and getters */
	
	public int getCurrentState() {
		return currentState;
	}
	
	public double getPartWeight() {
		return partWeight;
	}
	
	public void setPartWeight(double partWeight) {
		this.partWeight = partWeight;
	}
	
	public int getMyId() {
		return this.myId;
	}
			
	/* setters and getters - end */
	
	/**
	 * Worker v textove podobe.
	 * */
	public String toString() {
		return "Worker " + this.myId + " - stav: " + this.currentState 
				+ ", spoctena hmotnost = " + this.partWeight;
	}

	
	/* ******* */
	
	/**
	 * Metoda nastavi atributy pro vypocet z predanych parametru.
	 * */
	public void initWorkerData(double fromX, double fromY, double fromZ,
			double toX, double toY, double toZ,
			double dx, double dy, double dz,
			int functionType) {
		
		this.fromX = fromX;
		this.fromY = fromY;
		this.fromZ = fromZ;
		this.toX = toX;
		this.toY = toY;
		this.toZ = toZ;
		this.dx = dx;
		this.dy = dy;
		this.dz = dz;
		this.functionType = functionType;
		
		this.partWeight = 0.0;				// budu pocitat (odznova) od 0
		this.currentState = WAITING;		// data inicializovana, budu cekat na spusteni vypoctu
	}
	


	/**
	 * Metoda nastavi stav vlakna na ukoncene - uz nebude pocitat nic.
	 * */
	public void endWork() {
		this.currentState = DEAD;
	}
	
	/**
	 * Metoda nastavi stav vlakna na momentalne pracujici.
	 * */
	public void setComputing() {
		this.currentState = WORKING;
	}
	
	
	/**
	 * Metoda provede vypocet hustoty podle pozadovaneho typu funkce ro. 
	 * */
	private double computeRo(double x, double y, double z) {
		
		double ro;
		
		switch (this.functionType) {		// vyber funkce ro
			case 0: ro = Funkce.ro1(x, y, z); break;
			case 1: ro = Funkce.roX(x, y, z); break;
			case 2: ro = Funkce.roXYZ(x, y, z); break;
			default: ro = 0.0; break;
		}
		
		return ro;
	}
	
	
	/**
	 * Metoda provede vlastni vypocet hmotnosti pozadovane casti kvadru podle
	 * zadaneho vzorce (dx*dy*dz*ro(x+dx/2,y+dy/2,z+dz/2)). Pak nastavi stav
	 * vlakna na DONE - ukoncen vypocet a vysledek lze prevzit.
	 * */
	public void computePartWeight() {
		
		// usek kvadru pres rozmer x:
		for (double iterX = this.fromX; iterX < this.toX; iterX += this.dx) {
			// usek kvadru pres rozmer y:
			for (double iterY = this.fromY; iterY < this.toY; iterY += this.dy) {
				// usek kvadru pres rozmer z:
				for (double iterZ = this.fromZ; iterZ < this.toZ; iterZ += this.dz) {
					
					this.partWeight += this.dx * this.dy * this.dz * this.computeRo(
											iterX + this.dx/2,
											iterY + this.dy/2,
											iterZ + this.dz/2); 
					
				}
			}
		}
		
		this.currentState = DONE;	// vypocet skoncen, mezivysledek hotov, lze prevzit Farmerem
	}
	
	
	/**
	 * Metoda behu vlakna Workera.
	 * */
	public void run() {
		super.run();
		
		System.out.println("Worker " + this.myId + " bezi.");
		
		// dokud vlakno zije (dokud je pro nej nejaka prace):
		while (this.currentState != DEAD) {
			// Pokud ma vlakno nastavene hodnoty k pocitani (metodou initWorker()) ...
			if (this.currentState == WORKING) {
				// Provede se vypocet:
				this.computePartWeight();
			}
		}
		
		System.out.println("Worker " + this.myId + " konci.");
	}
	
	
}
