/**
 * Zpracovani.java - uchovava parametry vypoctu, jak byly nacteny bud z prikazove
 * radky nebo z konfiguracniho souboru.
 * 
 * @author Jiri Frolik, A06081, KIV/PPR 2007/08
 * */

public class Zpracovani {

	private int threadCount;		// pocet vlaken, co bude zpracovavat ulohu
	private int functionType;		// jeden ze tri typu funkce pro vypocet hustoty
	private double assignedPart;	// cast vypoctu pridelovana vlaknum ke zpracovani (cislo < 1)
	
	private double dx;
	private double dy;
	private double dz;				// hodnoty dx, dy, dz pro vypocet hmotnosti kvadru
	
	
	private Kvadr kvadr;			// kvadr - obsahuje souradnice protilehlych rohu kvadru
	
	
	/**
	 * Konstruktor.
	 * */
	public Zpracovani(Kvadr kvadr) {
		this.kvadr = kvadr;
	}
	
	/* getters and setters */
	
	
	public double getAssignedPart() {
		return assignedPart;
	}	

	public void setAssignedPart(double assignedPart) {
		this.assignedPart = assignedPart;
	}
	
	public int getFunctionType() {
		return functionType;
	}
	
	public void setFunctionType(int functionType) {
		this.functionType = functionType;
	}
	
	public Kvadr getKvadr() {
		return kvadr;
	}
	
	public void setKvadr(Kvadr kvadr) {
		this.kvadr = kvadr;
	}
	
	public int getThreadCount() {
		return this.threadCount;
	}
	
	public void setThreadCount(int count) {
		this.threadCount = count;
	}
	
	public double getDx() {
		return dx;
	}
	
	public void setDx(double dx) {
		this.dx = dx;
	}
	
	public double getDy() {
		return dy;
	}	

	public void setDy(double dy) {
		this.dy = dy;
	}
	
	public double getDz() {
		return dz;
	}
	
	public void setDz(double dz) {
		this.dz = dz;
	}
	
	/* getters and setters - end */
	
	/**
	 * Vrati parametry vypoctu jako retezec.
	 * */
	public String toString() {
		return "Zpracovani:\nPocet vlaken = " + this.threadCount +
				"\nPridelena cast ( < 1 ) = " + this.assignedPart +
				"\nTyp funkce = " + this.functionType +
				"\ndx = " + this.dx + ", dy = " + this.dy + ", dz = " + this.dz;
	}
	
	
	/**
	 * Metoda provede kontrolu svych atributu. Vrati false v pripade zjisteni nejake chyby.
	 * (Delkove parametry + pocet threadu musi byt kladne, assignedPart
	 * musi lezet v intervalu (0,1> a typ funkce musi byt jedna z hodnot 0,1,2).
	 * */
	public boolean checkAttributes() {
		
		if ((this.kvadr.getX() <= 0) || (this.kvadr.getY() <= 0) || (this.kvadr.getZ() <= 0)
				|| (this.dx <= 0) || (this.dy <= 0) || (this.dz <= 0)
				|| (this.assignedPart <= 0) || (this.assignedPart > 1) || (this.threadCount <= 0)) {
			return false;
		}
		if ((this.functionType < 0) || (this.functionType > 2)) {
			return false;
		}
		return true;
	}
	
	
	
	
	
}	/* end of class Zpracovani */
