/*
 * prace:             semestralni samostatna prace z paralelniho programovani
 *                    cast 1: paralelni program pro system se sdilenou pameti (Java)            
 * autor:             Josef Mlnarik
 * student id:        A00141
 * datum narozeni:    31.10.1981
 * datum vypracovani: leden 2006 
 * predmet:           KIV/PPR
 * cislo zadani:      1 = (31 + 10) % 20
 * formulace zadani:  realizujte vypocet nejmensiho/nejvetsiho prvku ctvercove matice
 *                    s kladnymi celociselnymi prvky a konverze (mormalizace) na realnou
 *                    matici s prvky v intervalu <0,1>
 */

package ppr_mlnarik_a00141;

import java.io.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.*;

// ----------------------------------------
// trida MATICE reprezentuje obecnou matici
// ----------------------------------------
class Matice {
	private double [][] matice; 							/* pole poli = matice */ 
	private int rozmer = 3;								/* rozmer matice (pokud nebude zadan uzivatelem = 3) */
	private double nejvetsiPrvek = - 1; 						/* hledany nejvetsi prvek matice */
	private double nejmensiPrvek = - 1; 						/* hledany nejmensi prvek matice */
	public int RAND_MAX = 99;							/* maximalni hodnota cisel v generovane matici */

	// konstruktor pro objekt MATICE
	Matice(int rozmer) {
		this.rozmer = rozmer;							/* nastav rozmer matice */	
		generujMatici();							/* vygeneruj matici */
	}

	// metoda pro ziskani nejvetsiho prvku, ten budou potrebovat normalizacni vlakna
	double dejNejvetsi() {
		return this.nejvetsiPrvek;						/* vrati nejvetsi prvek matice */
	}
	
	// metoda pro ziskani radku matice, kazdemu vlaknu vrati jemu prideleny radek matice
	double[] dejRadek(int radek) { 
		return matice[radek];							/* vrati zadany radek matice */								
	}
	
	// metoda pro prepsani radku matice, kazde vlakno prepise svuj radek normalizovanym
	void nastavRadek(int cisloRadku, double[] radek) {
		matice[cisloRadku] = radek;						/* prepise v matici zadany radek poslanym radkem */
	}
	
	// metoda pro nastaveni nejvetsiho hledaneho cisla
	void nastavNejvetsi(double nejvetsi) {						/* vlakno posle svuj 'nejvetsi' */
		this.nejvetsiPrvek = (this.nejvetsiPrvek > nejvetsi ? this.nejvetsiPrvek : nejvetsi);
											/* pokud je poslany z vlakna 'nejvetsi' > nejvetsi prvek matice */
											/* pak nejvetsim prvkem matice je odted tento poslany 'nejvetsi' */
	}
	
	// metoda pro nastaveni nejvetsiho hledaneho cisla
	void nastavNejmensi(double nejmensi) {						/* vlakno posle svuj 'nejmensi' */
		if (this.nejmensiPrvek == -1) this.nejmensiPrvek = nejmensi;		/* neni-li jeste nejmensi prvek matice nastaven (= -1) */
											/* pak poslany je nejmensi prvek matice */
		this.nejmensiPrvek = (this.nejmensiPrvek < nejmensi ? this.nejmensiPrvek : nejmensi);
											/* pokud je poslany z vlakna 'nejmensi' < nejmensi prvek matice */
											/* pak nejmensim prvkem matice je odted tento poslany 'nejmensi' */
	}
  
	// metoda pro tisk nejvetsiho prvku (prvni vysledek)
	void tiskniNejvetsi(RandomAccessFile soubor) throws IOException {
		String text = " - nejvetsi prvek: "+nejvetsiPrvek+"\r\n";		/* co se bude tisknout */
		System.out.print(text);							/* na standardni vystup */
		soubor.writeUTF(text);							/* do vystupniho souboru */
	}
  
	// metoda pro tisk nejmensiho prvku (druhy vysledek)
	void tiskniNejmensi(RandomAccessFile soubor) throws IOException {
		String text = " - nejmensi prvek: "+nejmensiPrvek+"\r\n";		/* co se bude tisknout */
		System.out.print(text);							/* na standardni vystup */
		soubor.writeUTF(text);							/* do vystupniho souboru */
	}
	
	// metoda vygeneruje nahodnou matici rozmeru rozmer
	void generujMatici() {
		Random generator = new Random();					/* generator */
		matice = new double[this.rozmer][this.rozmer]; 				/* matice */
		for(int i = 0; i < this.rozmer; i++){					/* pro kazdy radek matice */
			for (int j = 0; j < this.rozmer; j++)				/* pro kazdy prvek radku */
				matice[i][j] = (double) generator.nextInt(this.RAND_MAX);	
											/* generuj hodnotu int jako prvek matice */
		}
	}
	// metoda vytiskne matici
	void vytiskniMatici(RandomAccessFile soubor, int par) throws IOException {
		NumberFormat formatter = null;						/* formatovani matice */
		switch (par){
			case 0:								/* vstupni matice */
				formatter = new DecimalFormat("00");			/* je formatovana celociselne na dva rady */
				break;
			case 1:								/* vystupni matice */
				formatter = new DecimalFormat("0.00");			/* je formatovana realne na jeden rad a dve desetinna mista */
				break;
		}
		for(int i = 0; i < this.rozmer; i++) {					/* pro kazdy radek matice */
			for (int j = 0; j < this.rozmer; j++) { 			/* pro kazdy prvek radku */
				String text = " "+formatter.format(matice[i][j])+" ";	/* tiskni prvkek matice */
				System.out.print(text);					/* na standardni vystup */
			 	soubor.writeUTF(text);					/* do vystupniho souboru */
			}
			System.out.println();						/* novy radek na standardni vystup */
			soubor.writeChars("\r\n");					/* novy radek do vystupniho souboru */
		}
	}
}


// --------------------------------------------------------------------------------------
// trida NORMALIZUJMATICI reprezentuje vlakno, ktere bude normalizovat jeden radek matice
// --------------------------------------------------------------------------------------
class NormalizujMatici extends Thread {
	private int mytag;								/* identifikator vlakna 0,1,2,.. */
	private Matice matice;           						/* objekt MATICE pro ukladani vysledku */
	private double radek[];								/* radek matice */
	int rozmer;									/* rozmer matice */
	
	// konstruktor zavede vlakno, ktere znormalizuje prislusny radek a vysledek ulozi
	NormalizujMatici(int tag, double[] radek, int rozmer, Matice result) {
		this.mytag = tag;							/* identifikator vlakna 0,1,2,.. */
		this.matice = result;							/* vysledek */		
		this.radek = radek;							/* vstupni radek */
		this.rozmer = rozmer;							/* rozmer matice */
	}

	// vlastni RUN spusti normalizacni vlakno
	public void run() {
  		System.out.println(" Normalizuji "+mytag+". radek.");			/* vypise, ze pracuje na standardni vystup */
		for (int i = 0; i <  rozmer; i++) {					/* pro kazdy prvek zpracovavaneho radku */
			radek[i] = radek[i]/matice.dejNejvetsi();			/* podeli kazdy prvek nalezenym nejvetsim prvkem v matici = normalizace */ 
		}
		matice.nastavRadek(mytag,radek);					/* ulozi vypocitany (= normalizovany) radek */ 
	}
}


// ---------------------------------------------------------------------------------
// trida NAJDIEXTREMY reprezentuje vlakno, ktere bude prohledavat jeden radek matice
// ---------------------------------------------------------------------------------
class NajdiExtremy extends Thread {
	private double[] radek;								/* radek matice */
	private int tag;								/* identifikator vlakna 0,1,2,.. */
	private int rozmerMatice;							/* rozmer matice*/
	private Matice matice;								/* matice */
	
	// konstruktor zavede vlakno, ktere hleda minimum a maximum prislusneho radku a vysledek nastavi
	NajdiExtremy(int mtag, double[] radek_matice,int rozmer, Matice matice) {
		this.tag = mtag;							/* identifikator vlakna 0,1,2,.. */
		this.radek = radek_matice;						/* vstupni radek */
		this.rozmerMatice = rozmer;						/* rozmer matice */
		this.matice = matice;							/* matice */
	}

	// vlastni RUN spusti normalizacni vlakno
	public synchronized void run() {
		 double nejvetsi;							/* nejvetsi prvek tohoto vlakna (=radku) */
		 double nejmensi;							/* nejmensi prvek tohoto vlakna (=radku) */
		 
		 nejvetsi = nejmensi = radek[0];					/* na zacatku nastavi nejmensi i nejvetsi = prvni zpracovavany prvek */
		 System.out.print(" Prohledavam "+tag+". radek.");	 		/* vypise, ze pracuje na standardni vystup */
		 for (int i = 1; i < rozmerMatice; i++) {				/* pro kazdy prohledavany prvek radku */
			 nejvetsi = (radek[i] > nejvetsi ? radek[i] : nejvetsi);	/* pokud je prohledavany prvek > doted nastaveny nejvetsi vlakna */
				 							/* pak nejvetsim tohoto vlakna je odted tento prohledavany prvek */
			 nejmensi = (radek[i] < nejmensi ? radek[i] : nejmensi);	/* pokud je prohledavany prvek < doted nastaveny nejmensi vlakna */
		 }		 															/* pak nejmensim tohoto vlakna je odted tento prohledavany prvek */
		 matice.nastavNejvetsi(nejvetsi);					/* nastavi nejvetsi prvek matice volanim metody nastavNejvetsi objektu MATICE */
		 matice.nastavNejmensi(nejmensi);					/* nastavi nejmensi prvek matice volanim metody nastavNejmensi objektu MATICE */
		 System.out.println(" Vysledek "+tag+": max="+nejvetsi+" min="+nejmensi+"");
											/* nez skonci, vypise vlakno vysledek sve prace na standardni vystup */
	}
}


// ------------------------------------------------
// trida PPR reprezentuje vlastni spusteni programu
// ------------------------------------------------
public class PPR {

	// vlastni MAIN spusti program, vstupni argument mainu je pole argumentu programu
	public static void main(String[] argv) throws IOException {
	int pocetDelniku = 3; 								/* pocet delniku (pokud nebude zadan uzivatelem = 3) */

	if (argv.length > 1) {								/* pokud je zadano vic argumentu nez jeden */
		System.out.println("zadejte pouze jeden argument");			/* vypise upozorneni */
		return;									/* a skonci */
	} 
	if (argv.length == 1) {	    							/* pokud je zadan jeden argument */
   		try {
   			pocetDelniku = Integer.valueOf(argv[0]).intValue();		/* pak pocet delniku = tento argument */
   			if (pocetDelniku < 2 ) {					/* ale nesmi byt mensi nez dva (rad matice) */
   				System.out.println("hodnotu argumentu zadejte > 1");	/* vypise chybu hodnoty */
   				return;							/* a skonci */
   			}
   		}
   		catch(NumberFormatException e){						/* v pripade ze doslo k vyjimce */
   			System.out.println("zadejte ciselny format argumentu");		/* vypise chybu formatu */
   			return;								/* a skonci */
   		}	
	}
    
	NajdiExtremy[] hledaciVlakna = new NajdiExtremy[pocetDelniku]; 			/* inicializace pole prohledavacich vlaken */
	NormalizujMatici[] normalizacniVlakna = new NormalizujMatici[pocetDelniku];
    											/* inicializace pole normalizacnich vlaken */ 
	Matice matice = new Matice(pocetDelniku);  					/* vygenerovani nahodne matici zadaneho rozmeru */
   
	File soubor = new File("out.txt");						/* nastavi soubor pro vystup */
 	if (soubor.exists()) soubor.delete();						/* soubor smaze pokud existuje */
	RandomAccessFile vystupniSoubor = new RandomAccessFile("out.txt","rw"); 	/* otevre soubor pro zapis */
    
	String text0 =  "+---------------------+\r\n"+					/* text0, co se bude vypisovat */
	                "| VSTUPNI MATICE "+pocetDelniku+"x"+pocetDelniku+"  |\r\n"+
	                "+---------------------+\r\n";
	System.out.print(text0);							/* vypise text0 na standardni vystup */
	vystupniSoubor.writeUTF(text0);							/* vypise text0 do vystupniho souboru */
	matice.vytiskniMatici(vystupniSoubor, 0);					/* vytiskne zadanou (=vygenerovanou) matici do souboru i na standardni vystup */

	System.out.println("+---------------------+");					/* vypise na standardni vytup info o zapocati prace prohledavani */
	System.out.println("| PROHLEDAVANI MATICE |");
	System.out.println("+---------------------+");
	for (int i = 0; i < pocetDelniku; i++) { 					/* spusti dany pocet prohledavacich vlaken */
		hledaciVlakna[i] = new NajdiExtremy(i, matice.dejRadek(i), pocetDelniku, matice);
	    	hledaciVlakna[i].start();
	}
	for (int p = 0; p < pocetDelniku; p++) { 					/* cekani na ukonceni prohledavacich vlaken */
	    	try {hledaciVlakna[p].join();}
	    	catch(InterruptedException e) {						/* zachyceni vyjimky */
    			System.out.println("Vlakno preruseno!");
    		}
	}
    
	System.out.println("+---------------------+");					/* vypise na standardni vytup info o zapocati prace normalizovani */
	System.out.println("| NORMALIZACE MATICE  |");
	System.out.println("+---------------------+");
	for (int i = 0; i < pocetDelniku; i++) { 					/* spusti dany pocet normalizacnich vlaken */
		normalizacniVlakna[i] = new NormalizujMatici(i, matice.dejRadek(i), pocetDelniku, matice);
	    	normalizacniVlakna[i].start();
	}
	for (int p = 0; p < pocetDelniku; p++) { 					/* cekani na ukonceni normalizacnich vlaken */
		try {normalizacniVlakna[p].join();}
		catch (InterruptedException e) {					/* zachyceni vyjimky */
			System.out.println("Vlakno preruseno!");
		}
	}
    
	String text1 =  "+---------------------+\r\n"+					/* text1, co se bude vypisovat */
    			"| VYSLEDEK            |\r\n"+
    			"+---------------------+\r\n";
	String text2 =  " - normalizovana matice:\r\n";					/* text2, co se bude vypisovat */
	System.out.print(text1);							/* vypise text1 na standardni vystup */		
	vystupniSoubor.writeUTF(text1);							/* vypise text1 do vystupniho souboru */
	matice.tiskniNejmensi(vystupniSoubor);						/* vytiskne nejmensi nalezeny prvek do souboru i na standardni vystup */
	matice.tiskniNejvetsi(vystupniSoubor);						/* vytiskne nejvetsi nalezeny prvek do souboru i na standardni vystup */
	System.out.print(text2);							/* vypise text2 na standardni vystup */
	vystupniSoubor.writeUTF(text2);							/* vypise text2 do vystupniho souboru */
	matice.vytiskniMatici(vystupniSoubor, 1); 					/* vytiskne vyslednou normalizovanou matici do souboru i na standardni vystup */
	}
}
