/*
 * prace:             semestralni samostatna prace z paralelniho programovani
 *                    cast 2: paralelni program pro system s distribuovanou pameti (PVM)
 * autor:             Josef Mlnarik
 * student id:        A00141
 * datum narozeni:    31.10.1981
 * datum vypracovani: leden 2006 
 * predmet:           KIV/PPR
 * cislo zadani:      1 = (31 + 10) % 20
 * formulace zadani:  realizujte vypocet nejmensiho/nejvetsiho prvku ctvercove matice
 *                    s kladnymi celociselnymi prvky a konverze (mormalizace) na realnou
 *                    matici s prvky v intervalu <0,1>
 */

#include <stdio.h>
#include <pvm3.h>
#include "help.h"

#define RAND_MAX 21400000											/* konstanta pro omezeni generovanych cisel */
  
main(int argc, char *argv[]) {
	int mytid;      												/* identifikator procesu sef */
	int tids[32];   												/* identifikatory procesu delnici */
	int i, j, tid;  												/* pomocne promenne */
	int vytvoreno = 0;  											/* pocet uspesne vytvorenych delniku */
	int* tidc;      												/* pomocny ukazatel pro vytvareni delniku */
	double** v_matice = 0; 											/* zpracovavana matice */
	double* radka;     												/* pomocna radka */	
	double nejvetsi = -1;											/* nejvetsi prvek zpracovavane matice */
	double nejmensi = -1;											/* nejmensi prvek zpracovavane matice */
	double min = 0;													/* nejvetsi prvek pro kazdeho delnika = radek matice */
	double max = 0;													/* nejmensi prvek pro kazdeho delnika = radek matice */
	FILE *soubor;													/* soubor se vstupni matici */
	int radek = 0;													/* cislo zpracovavaneho radku */
	int rozmer;														/* rozmer matice */

	// zpracovani vstupnich argumentu
	printf("\n");													/* pro prehlednost odradkuje */
	if (argc == 1) {												/* spusteno bez argumentu */
    	N = rozmer = 3;												/* prednastaveny rozmer */
    	printf("program spusten bez argumentu\n(jako ciselny argument muzete zadat pocet delniku, nebo soubor se vstupni matici)\n");
   		printf("generuji ctvercovou matici o prednastavenych rozmerech %dx%d\n", rozmer, rozmer);
        v_matice = alokujMatici();									/* alokuje pamet pro vstupni matici */
		srand((unsigned)time(NULL));								/* semeno randomizeru */
		for (i=0; i<rozmer; i++) {
		    for (j=0; j<rozmer; j++) {
				v_matice[i][j] = (int)rand() / RAND_MAX;			/* vygeneruje madici s nahodnymi prvky */
		    }
		}    	
    }
    if (argc == 2) {												/* spusteno s jednim argumentem */
    	printf("spusteno s jednim argumentem\n");
	    if ((rozmer = atoi(argv[1])) > 1) {							/* je-li argument > 1 */
	   		N = rozmer;												/* pak tento argument je poctem delniku = rozmerem matice*/
   	    	printf("argument prijat jako pocet delniku\n");
	   		printf("generuji ctvercovou matici o zadanych rozmerech %dx%d\n", rozmer, rozmer);
	        v_matice = alokujMatici();								/* alokuje pamet pro vstupni matici */
			srand((unsigned)time(NULL));							/* semeno randomizeru */
			for (i=0; i<N; i++) {
			    for (j=0; j<N; j++) {
					v_matice[i][j] = (int)rand() / RAND_MAX;		/* vygeneruje madici s nahodnymi prvky */
			    }
			}
		}
		else {														/* argument neni ciselny nebo nema smysl vzhledem k zadani (> 1) */
																	/* zkusim tedy otevrit vstupni soubor toho jmena */
			printf("argument prijat jako nazev vstupniho souboru\n");
			printf("(je-li argument ciselny, pak nema smysl vzhledem k zadani: pozadovano cislo > 1)\n");
        	soubor = otevriSoubor(argv[1],"r");					/* otevre vstupni soubor */
	        v_matice = nactiMatici(soubor);							/* nacte matici ze souboru */
	    }
    }
    if (argc > 2) {													/* spusteno s vice nez jednim argumentem */
    	printf("chyba: spusteno s vice argumenty, zadejte pouze jeden\n");
    	printf("jako ciselny argument muzete zadat pocet delniku, nebo soubor se vstupni matici\n");
    	return;														/* vypise upozorneni a skonci */
    }
   
    // inicializace        
    radka = (double *)malloc (N*sizeof(double));					/* alokuje pamet pro radku */
	mytid = pvm_mytid();  											/* prihlaseni do PVM */

	printf("+--------------------+\n");								/* vytiskne hlavicku */
	printf("| VSTUPNI MATICE %dx%d |\n", N, N);
	printf("+--------------------+\n");
    vypisMatici(v_matice,0);										/* a vstupni matici */
    
	// vytvareni delniku
	tidc = tids;													/* identifikatory delniku */
	while (N > vytvoreno) {											/* dokud jich neni kolik je treba */
		if (pvm_spawn("worker", (char**)0, 0, "", 1, tidc)) {		/* vytvor delnika */
			vytvoreno++;											/* pokud uspesne, tak zvys pocet vytvorenych */
			tidc++;													/* pokud uspesne, tak zvys identifikator */
		} 
		else {														/* neuspech */
			printf("chyba %d pri vytvareni delnika\n", *tidc);		/* vytiskne upozorneni */
			return;													/* a skonci */
		}
	}

	// uvodni komunikace farmare s delniky (registrace pracovnich sil)
	pvm_initsend(PvmDataDefault);   								/* inicializace bufferu */
	pvm_pkint(&mytid, 1, 1);    									/* ulozeni identifikace */
	pvm_mcast(tids, N, 4);  										/* vsem delnikum broadcast zpravy s kodem 4:broadcast(id sefa->delnici) */

	// cekani na odezvu od delniku
	printf(" - najati delnici: ");									/* vypise najate delniky */
	for(i=0; i<N; i++) { 
		pvm_recv(-1, 5);         									/* posilani zpravy s kodem 5:odpoved(id delnika->sef) */
		pvm_upkint(&tid, 1, 1);  									/* cislo delnika */  
		printf("%d, ", tid);										/* vypise kdo je pripraven */
	}
	printf("\n");													/* odradkuje */
	
	// pocatecni ukolovani delniku
	for(i=0; i<N; i++) { 
		pvm_initsend(PvmDataDefault);   							/* inicializace bufferu */
		pvm_pkint(&N, 1, 1);
		pvm_pkdouble(v_matice[i], N, 1);
		pvm_pkint(&i, 1, 1);
		pvm_send(tids[i], 6);           							/* zaslani zpravy s kodem 6:zadani ukolu(sef->delnik) */
	}
	
	// kdo dokoncil, dostane dalsi ukol: prohledavani
    for (j=0; j<N; j++){
		pvm_recv (-1, 7);           								/* cekani na vysledek */
        pvm_upkint (&tid, 1, 1);
        pvm_upkdouble (&max, 1, 1);    								/* kopirovani vysledku do max */
		pvm_upkdouble(&min, 1, 1);  								/* kopirovani vysledku do min */
		if (nejmensi == -1) nejmensi = min;							/* pokud je nejmensi -1, pak jeste nebyl porovnavan */
		nejvetsi = (nejvetsi > max ? nejvetsi : max);				/* nejvetsi = vysledek delnika, pokud je vetsi */ 
		nejmensi = (nejmensi < min ? nejmensi : min);				/* nejmensi = vysledek delnika, pokud je mensi */ 
	}

	// kdo dokoncil, dostane dalsi ukol: normalizace
	for(i=0; i<N; i++) { 
	    pvm_initsend(PvmDataDefault);   							/* inicializace bufferu */
	    pvm_pkdouble(&nejvetsi, 1, 1);            					/* ulozeni hodnoty nejvetsi */
    	pvm_send (tids[i], 6);          							/* zaslani zpravy s kodem 6:zadani ukolu(sef->delnik) */
	}

	// uvolnovani delniku
	printf(" - propusteni delnici: ");								/* vypise propustene delniky */
	for(i=0; i<N; i++) { 
		pvm_recv (-1, 7);          									/* cekani na ukonceni s kodem 7:vysledek(delnik->sef) */
		pvm_upkint (&tid, 1, 1);
		pvm_upkdouble (radka, N, 1);								/* normalizovany radek */
		pvm_upkint (&radek, 1, 1);        							/* cislo normalizovaneho radku */
        for (j=0; j<N; j++) {										/* prepise radek v matici normalizovanym radkem od delnika */
			v_matice[radek][j] = radka[j];						
	    }
	    printf("%d, ", tid);										/* vypise kdo dokoncil */
	}
	printf("\n");													/* odradkuje */

	// vypis vysledku
	printf("+--------------------+\n");								/* vytiskne hlavicku */
	printf("| VYSLEDEK           |\n");
	printf("+--------------------+\n");
    printf(" - nejvetsi prvek: %3.0f\n", nejvetsi);
    printf(" - nejmensi prvek: %3.0f\n", nejmensi);
	printf(" - normalizovana matice:\n");
    vypisMatici(v_matice,1);										/* a vystupni matici */
    printf("\n");													/* odradkuje */
    
	pvm_exit();														/* ukonceni cinnosti v PVM */
}

