#include <stdio.h>
#include <stdlib.h>
#include <pvm3.h>

/* PPR - Jaroslav Rychna							          */
/* Zadani cislo 9 - vypocet intenzity elektrickeho pole		  */
/*															  */
/* Farmer zadava workerum postupne sloupecky matice           */
/* a delnici pocitaji intenzitu a pro kazdy prvek matice      */
/* posilaji zpet vysledek - intenzitu i s jejimi souradnicemi */
/* Pouzite kody zprav:                                        */
/*   4 ... broadcast (id sefa  -> delnici)                    */
/*   5 ... odpoved   (id delnika -> sef)                      */
/*   6 ... zadani ukolu (sef -> delnik),                      */
/*   7 ... vysledek     (delnik -> sef),                      */

int main(int argc, char *argv[]) {
  int mytid;      		/*  identifikator procesu sef  */
  int tids[32];   		/*  identifikatory procesu delnici */
  int nproc;      		/*  pocet delniku */

  int vytvoreno = 0;  	/*  pocet uspesne vytvorenych delniku */
  int end_param = 0;    /*  pomocna ridici poromenna */
  int i, j, tid;
  int* tidc;      		/* pomocny ukazatel pro vytvareni delniku */

  /* Pridane promenne*/
  int souradniceZ;			/* souradnice naboje v ose z */
  int rozmerX;	                   /* sirka matice */
  int rozmerY;	                    /* vyska matice */
  double Q;                            /* velikost naboje Q */
  double e0 = 0.000000000008854;       /* permitivita vakua */
  double eR;                           /* relativni permitivita */
  int odX, doX;                        /* zacatek a konecprace pro delnika */
  double **p_matice;				   /* dynamicka matice */
  double intenzita;                    /* pom. intenzita - jeden vysledek */
  int zbyva_radku; 				       /* kolik radku zbyva spocitat */
  int zbyva_polozek;				   /* kolik polozek zbyva spocitat */

  /* zadani vstupnich parametru vypoctu - vstup od uzivatele*/
  while (! end_param) {
    printf("Zadej umisteni naboje na ose z: ");
    scanf("%d", &souradniceZ);
    printf("Zadej sirku matice: ");
    scanf("%d", &rozmerX);
    printf("Zadej vysku matice: ");
    scanf("%d", &rozmerY);
    printf("Zadej Q: ");
    scanf("%lf", &Q);
    printf("Zadej relativni permitivitu:");
    scanf("%lf", &eR);
    printf("Zadej pocet delniku 1-32: ");
    scanf("%d", &nproc);
    if(nproc > rozmerX) printf("Prilis mnoho delniku, zbytecne mzdove vydaje!\n Prosim zadejte udaje znovu.\n");
    else end_param = 1;
  }

  /* priprava matice intenzit - dynamicka alokace pameti*/
  zbyva_radku = rozmerX;
  zbyva_polozek = rozmerX*rozmerY;
  p_matice = (double **) malloc (rozmerX*sizeof(double*) );
  if(p_matice == NULL) {
	  printf("Nedostatek pameti!");
	  pvm_exit();
	  exit(1);
  }

  for(i=0;i<rozmerX;i++) {
	  p_matice[i]= (double*) malloc(rozmerY*sizeof(double));
	  if(p_matice[i] == NULL) {
		  printf("Nedostatek pameti!");
		  pvm_exit();
		  exit(1);
  	  }
  }

  /* prihlaseni do PVM */
  mytid = pvm_mytid();

  /* vytvoreni delniku */
  tidc = tids;
  while (nproc>vytvoreno) {
   if (pvm_spawn("worker", (char**)0, 0, "", 1, tidc)) {
      printf("Vytvoren delnik %d.\n", *tidc);
      vytvoreno++;
      tidc++;
   } else {
      printf("Chyba pri vytvareni delnika. Cislo chyby %d\n", *tidc);
   }
  }

  /* uvodni komunikace sefa s delniky (registrace prac. sil) */
  pvm_initsend(PvmDataDefault);       /* inicializace bufferu */
  pvm_pkint(&mytid, 1, 1);            /* ulozeni identifikace */
  pvm_mcast(tids, nproc, 4);          /* broadcast zpravy s kodem 4 */
                                      /*              vsem delnikum */

  /* cekani na odezvu od delniku */
  for(i=0; i<nproc; i++) {
    pvm_recv(-1, 5);                  /* 5 je kod zpravy */
    pvm_upkint(&tid, 1, 1);           /* cislo delnika */
    printf("Delnik cislo %d k praci pripraven! \n", tid);
  }


  /* pocatecni ukolovani delniku */
  odX = doX = 0;
  for(i = 0; i < nproc; i++) {
    doX = odX+1;
    pvm_initsend(PvmDataDefault);	/* inicializace bufferu */
    pvm_pkdouble(&Q, 1, 1);
    pvm_pkdouble(&e0, 1, 1);
    pvm_pkdouble(&eR, 1, 1);
    pvm_pkint(&odX, 1, 1);
    pvm_pkint(&doX, 1, 1);
    pvm_pkint(&rozmerY, 1, 1);
    pvm_pkint(&souradniceZ, 1, 1);
    pvm_send(tids[i], 6);		/* zaslani zpravy s kodem 6 */
    printf("Prace delnikovi %d pridelena \n", tids[i]);
    odX++;
    zbyva_radku--;                  /* snizi se pocet zbyvajicich radku */
  }

  /* prijem vysledku od delniku */
  while(zbyva_radku > 0) {
    pvm_recv(-1,7);
    pvm_upkint(&tid,1,1);
    pvm_upkint(&i,1,1);
    pvm_upkint(&j,1,1);
    pvm_upkdouble(&intenzita,1,1);

    /* delnik skoncil zadanou praci, dostane novou */
    if(i == -1) {
        doX = odX+1;
        pvm_initsend(PvmDataDefault);	/* inicializace bufferu */
        pvm_pkdouble(&Q, 1, 1);
        pvm_pkdouble(&e0, 1, 1);
        pvm_pkdouble(&eR, 1, 1);
        pvm_pkint(&odX, 1, 1);
        pvm_pkint(&doX, 1, 1);
        pvm_pkint(&rozmerY, 1, 1);
	pvm_pkint(&souradniceZ, 1, 1);
        pvm_send(tid, 6);		/* zaslani zpravy s kodem 6 */
        printf("Prace delnikovi %d pridelena \n", tid);
        odX++;
        zbyva_radku--;                  /* snizi se pocet zbyvajicich radku */
    } else {
        p_matice[i][j] = intenzita;
        zbyva_polozek--;
    }
  }

  /* dokonceni prace delniku, prijem poslednich polozek */
  while(zbyva_polozek > 0) {
    pvm_recv(-1,7);
    pvm_upkint(&tid,1,1);
    pvm_upkint(&i,1,1);
    pvm_upkint(&j,1,1);
    pvm_upkdouble(&intenzita,1,1);
    if(i != -1) {
        p_matice[i][j] = intenzita;
		zbyva_polozek--;
    }
  }

  /* vypis vysledku na obrazovku */
  souradniceZ = 0-souradniceZ;
  printf("Vysledky vypoctu:\n");
  for(i=0;i<rozmerX;i++) {
     for(j=0;j<rozmerY;j++) {
        printf("[%d,%d,%d] = %E \n", i, j, souradniceZ, p_matice[i][j]);
     }
  }

  /* free matice */
  for(i=0;i<rozmerX;i++) free(p_matice[i]);
  free(p_matice);

  /* ukonceni cinnosti v PVM */
  pvm_exit();
  return(0);
}

