#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <mpi.h>

#define M 800 // M,N - rozmery pro nahodne vygenerovana data
#define N 600
#define MAX 255 // maximalni mozna hodnota jasu

int n_proc; // pocet procesu
int id_proc; // id procesu
int* img; // matice reprezentujici
int k; // maximalni jas (zada uzivatel, pomoci nej se prepocita obrazek)
int step; // velikost dat zpracovanych jednim procesem
int m; // rozmery matice obrazku
int n;

/* nacte matici ze souboru a ulozi do globalni promenne img */
int read_file(char* filename) {
  FILE* file;
  int i,j; // pomocne promenne
  
  if ((file = fopen(filename, "r")) == NULL) {
    printf("File %s not found!\n", filename);
    return 0;
  }

  fscanf(file, "%d %d \n\n", &m, &n); // nactu prvni radek (hodnoty sirka,delka matice), pak je radek volny
  if (m*n <= 0) {
    printf("Wrong format. First line should consist of 'm n',\n where m is height and n width of the image.");
    fclose(file);
    return 1; // error
  }
  if((img = (int *)malloc(m*n*sizeof(int))) == NULL) {
        printf("Cannot allocate memory for image values.");
        exit(1);
      }

/* a nacteni hodnot matice */
  for(i = 0; i < m; i++){
      for(j = 0; j < n; j++) {
        fscanf(file,"%d ",&img[i*n+j]);
        }
      fscanf(file,"\n");
    }
  fclose(file);
  printf("File '%s' loaded.\n",filename);
  return 0; // success
}

/* ulozi hodnoty z globalni promenne img do souboru, ktery nejprve vytvori */
int save_file(char* filename) {
  FILE* file;
  int size = 0;
  int i,j = 0;

  if ((file = fopen(filename, "wb")) == NULL) {
    printf("Can't create file.\n");
    return 1;
  }
    fprintf(file, "%d %d \n\n", m, n);
    size = m*n;
    for(i = 0; i < m; i++){
      for(j = 0; j < n; j++)
        fprintf(file,"%d ",img[i*n+j]);
      fprintf(file,"\n");
    }
    fprintf(file,"\n");
    printf("Saved to file '%s'\n",filename);
    fclose(file);
  return 0;
}

/* vlastni prepocet matice */
void compute(void) {
  int i; // pomocna promenna
  int tmp_min = 256; // lokalni hodnoty min,max
  int tmp_max = -1;
  int min = 256; // globalni hodnoty min,max
  int max = -1; 
  int* img_part; // cast obrazku pocitana jednim procesem 
    
  /* broadcast - kazdemu priradim pocet hodnot, ktere ma spocitat*/
  MPI_Bcast(&step, 1, MPI_INT, 0, MPI_COMM_WORLD);

  if((img_part = (int *)malloc(step*sizeof(int))) == NULL) {
        printf("Cannot allocate memory for image values.");
        exit(1);
      }
      
  /* rozeslu praci (rozdelim img do img_part) */
  MPI_Scatter(img, step, MPI_INT, img_part, step, MPI_INT, 0, MPI_COMM_WORLD);

  /* spocitam lokalni extremy */
  for (i = 0; i < step; i++) {
    if (img_part[i] < tmp_min) {
      tmp_min = img_part[i];
    }
    if (img_part[i] > tmp_max) {
      tmp_max = img_part[i];
    }
  }
  /* seskupeni lokalnich extremu do globalnich */
  MPI_Allreduce(&tmp_min, &min, 1, MPI_INT, MPI_MIN, MPI_COMM_WORLD);
  MPI_Allreduce(&tmp_max, &max, 1, MPI_INT, MPI_MAX, MPI_COMM_WORLD);

  /* prepocet jasu */
  for (i = 0; i < step; i++) {
    img_part[i] = (int) (((img_part[i] - min) * k) / ((double)max-min));
  }
  
  /* sesbirani od procesu celeho obrazku */
  MPI_Gather(img_part, step, MPI_INT, img, step, MPI_INT, 0, MPI_COMM_WORLD);

  free((void*) img_part); // to uz nebudu potrebovat
}

int main (int argc, char *argv[]) {
int rand_matrix; // zda pocitat nahodnou matici 
int size; // velikost matice(pole)
int i; // pomocna promenna
int err = 0; // zda doslo nekde k chybe
char* filename = "out.kro"; // implicitni jmeno vystupniho souboru

  MPI_Init(&argc, &argv); // inicializace
  MPI_Comm_rank(MPI_COMM_WORLD, &id_proc); // zjisteni id aktualniho procesu (sama sebe)
  MPI_Comm_size(MPI_COMM_WORLD, &n_proc); // zjisteni poctu procesu

  if (id_proc == 0) { // pouze root provede tenhle usek
    switch(argc) {
      case 2: printf("Using random matrix 800x600 and implicit filenames 'in.kro' and 'out.kro'.\n");
        rand_matrix = 1;
        break;
      case 3: printf("Using random matrix 800x600 and implicit filename 'in.kro'.\n");
        rand_matrix = 1;
        filename = argv[2];
        break;
      case 4: printf("Everything set by user.\n");
        rand_matrix = 0;
        filename = argv[3];
        break;
      default: 
        printf("Error: wrong number of arguments.\n Should be used as 'convert <max_bright_value> [<in_file>] [<out_file>]. \n If there are two arguments, the second one is considered to be output file.\n");
        err = 1;
        break;
    }
    if(err == 0) { // pokud nedoslo k chybe, pokracuje se dal
        k = atoi(argv[1]); // nacteme k - v pripade retezce vrati funkce nulu, v pripade nuly taky, takze to tezko osetrim
        if((k<0)||(k>MAX)) { // k musi byt mezi 0-255
          printf("negative value of parameter k - using implicit value of 150\n");
          k = 150;
        }
          
        if(rand_matrix) { // pokud mam generovat nahodnou matici
          size = M*N;
          m = M;
          n = N;
          srand((unsigned int)time(NULL)); // inicializace random
          if((img = (int *)malloc(size*sizeof(int))) == NULL) {
            printf("Cannot allocate memory for image values.");
            err = 1;
          }
          if(err == 0) {
             for(i = 0; i < size; i++) {
              img[i] = rand()/(RAND_MAX / MAX + 1); // vytvoreni nahodne hodnoty 0-255
            }
            printf("Input: ");
            save_file("in.kro"); // ulozim nahodne hodnoty do souboru (pro kontrolu)
          }
        }else {
          if(read_file(argv[2])!=0) { // nacteni souboru 
            err = 1;
            }
          if(err == 0)
            size = m*n;
        }
        if(err == 0) {
        /* vypocet velikosti casti pro jeden proces */
          if(size%n_proc == 0)
              step = size/n_proc;
            else
              step = size/n_proc + 1;
            }
        }
  }
  
  if(err == 0) {
  MPI_Bcast(&k, 1, MPI_INT, 0, MPI_COMM_WORLD); // distribuuju hodnotu k, tu musi znat vsichni  
  compute(); // a vypocet
    
    if (id_proc == 0) { // opet jen root ulozi vysledek
        printf("Output: ");
        save_file(filename); // it doesn't matter if it is saved or not - the computation ends anyway
        free((void*) img);
      }
  }
  MPI_Finalize(); // proces se ukonci
    
  return 0;
}
