#include <stdio.h>
#include <stdlib.h>
#include <pvm3.h>
#include "ppr.h"

int ntask;
int pixels;
int k;
long bmp_length;

/*
 * Zkontroluje zda zadany soubor je bitmapa a zda ma spravny format.
 * Musi byt nekomprimovana a v 24bitove palete.
 */
int checkHeader(byte * header){
    //kontrola zda je to bitmapa
    if(header[0] != 0x42 || header[1] != 0x4D)
        return 0;
    //kontrola na pocet barev
    if(((header[28] << 8) | header[29]) != 0x1800)
        return 0;
    //a kontrola zda neni bitmapa komprimovana
    if(((header[30] << 24) | (header[31] << 16) | (header[32] << 8) | header[33]) != 0x0)
        return 0;
    
    return 1;
}

/*
 * Nacte bitmapu a vrati odkaz na matici dat,
 * rozdelenych podle parametru programu.
 * Druhy rozmer je vzdy velikost pridelene prace (pocet pixelu ke zpracovani) * 3.
 * Je 3x vetsi, protoze truecolor bitmapa pouziva 3 byty pro jeden pixel.
 */
byte ** loadBitmap(byte * bmp, byte * header, int * size){
    int len, mod, i, bytes;
    byte ** data;
    FILE * fr;
    
    printf("Nacitam bitmapu %s.\n", bmp);
    
    fr = fopen(bmp, "rb");
    if(fr == NULL) return (byte **)0;
    
    //potrebuju zjistit velikost bitmapy abych mohl urcit prvni rozmer pole
    fseek(fr, 0L, SEEK_END);
    bmp_length = ftell(fr)-HEADER_LENGTH;
    fseek(fr, 0L, SEEK_SET);
    
    bytes = pixels*BYTES_PER_PIXEL;
    mod = bmp_length%(bytes);
    len = bmp_length/(bytes);
    *size = len;
    if(mod != 0) len++;
    
    fread(header, sizeof(byte), HEADER_LENGTH, fr);
    
    if(checkHeader(header) == 0){
        printf("Zadany soubor neni bitmapa nebo nema spravny format.\n");
        printf("Aplikace akceptuje pouze truecolor nekomprimovane bitmapy.\n");
        return (byte **)0;
    }
    
    data = (byte **)malloc(sizeof(byte *)*len);
    for(i = 0; i < len-1; i++){
        data[i] = (byte *)malloc(sizeof(byte)*bytes);
        fread(data[i], sizeof(byte), bytes, fr);
    }
    //donactu blokove zbytek souboru
    data[len-1] = (byte *)malloc(sizeof(byte)*bytes);
    fread(data[len-1], sizeof(byte), bmp_length-(bytes*(len-1)), fr);
    printf("Bitmapa nactena.\n");
    
    fclose(fr);
    return data;
}

/*
 * Ullozi vysledek transformace zpatky jako bitmapu.
 */
int saveBitmap(byte * bmp, byte * header, byte ** data, int data_len){
    int i, len;
    FILE * fw;
    
    fw = fopen(bmp, "wb");
    if(fw == NULL) return 0;
    
    fwrite(header, sizeof(byte), HEADER_LENGTH, fw);
    fflush(fw);
    
    printf("Ukladam bitmapu do %s.\n", bmp);
    for(i = 0; i < data_len-1; i++){
        fwrite(data[i], sizeof(byte), pixels*BYTES_PER_PIXEL, fw);
    }
    
    len = bmp_length + (1-data_len)*pixels*BYTES_PER_PIXEL;
    fwrite(data[data_len-1], sizeof(byte), len, fw);
    
    fclose(fw);
    return 1;
}

/*
 * Zasle praci (pole bytu) delnikovi se zadanym tid.
 * Posila se i index, aby pri prijmuti vedel farmar, kam ma dane pole vratit.
 */
void send_work(int index, int tid, byte * data){
    pvm_initsend(PvmDataDefault);
    pvm_pkint(&index, 1, 1);
    pvm_pkint(&pixels, 1, 1);
    pvm_pkint(&k, 1, 1);
    pvm_pkbyte(data, pixels*BYTES_PER_PIXEL, 1);
    printf("Posilam data ke zpracovani delnikovi %d.\n", tid);
    pvm_send(tid, MSG_TODO);
}

/*
 * Hlavni smycka programu, prideluje data dokud nejaka jsou a vsichni delnici neskonci.
 */
void run(byte ** bmp_data, int data_len, int * tids){
    int pracujici, i, index, tid;
    pracujici = 0;
    
    for(i = 0; i < ntask;i++){
        send_work(i, tids[i], bmp_data[i]);
        pracujici++;
    }
    
    while(1){
        if(pvm_recv(-1, MSG_RESULT) > 0){
            pvm_upkint(&tid, 1, 1);
            pvm_upkint(&index, 1, 1);
            printf("Prijimam data od delnika %d.\n", tid);
            pvm_upkbyte(bmp_data[index], pixels*BYTES_PER_PIXEL, 1);
            
            if(i < data_len){
                send_work(i++, tid, bmp_data[i]);
            }else pracujici--;
            if(pracujici == 0) break;
        }
    }
}
/*
 * Vypise napovedu o programu.
 */
void printInfo(void){
    printf("Vytvoreni negativu k puvodnimu obrazku\n");
    printf("======================================\n\n");
    printf("Semestralni prace na PPR\n");
    printf("Milan Balcar\tA06072\n\n");
    printf("Aplikaci je nunto spoustet s 5 parametry.\n\n");
    printf("./farmer.exe <pocet vlaken> <velikost prace> <k> <vstupni soubor> <vystupni soubor>\n");
    printf("   <pocet vlaken>      - pocet vlaken, v kterych vypocet pobezi\n");
    printf("   <velikost prace>    - pocet pixelu, ktera ma jedno vlakno zpracovat\n");
    printf("   <k>                 - jas pixelu v rozsaho <0;255>\n");
    printf("   <vstupni soubor>    - bitmapa s obrazkem\n");
    printf("   <vystupni soubor>   - vysledek bude ulozen do tohoto souboru\n");
}

int main(int argc, char ** args){
    int mytid,i,size;
    byte * bmp_original;
    byte * bmp_result;
    byte bmp_header[HEADER_LENGTH];
    byte ** bmp_data;
    int * tids;
    
    if(argc != 6){
        printf("Spatny pocet argumentu.\n");
        printInfo();
        exit(1);
    }
    
    ntask = atoi(args[1]);
    pixels = atoi(args[2]);
    k = atoi(args[3]);
    bmp_original = args[4];
    bmp_result = args[5];
    
    if(ntask <= 0 || pixels <= 0 || k < 0 || k > 255){
        printf("Je nam lito, ale zadane argumenty jsou mimo jejich platne rozsahy.\n");
        printf("pocet vlaken musi byt vetsi nez 0.\n");
        printf("pocet pixelu musi byt vetsi jak 0.\n");
        printf("k musi byt z rozsahu <0;255>.\n");
        return 1;
    }
    
    tids = (int *)malloc(sizeof(int)*ntask);
    
    //nacteme bitmapu
    if((bmp_data = loadBitmap(bmp_original, bmp_header, &size)) == NULL){
        printf("Chyba pri nacitani bitmapy.\n");
        exit(1);
    }
    
    //nastartujem pvm
    mytid = pvm_mytid();
    printf("Spoustim delniky...\n");
    i =	pvm_spawn("worker.exe", (char **)0, PvmTaskDefault, (char *)0, ntask, tids);
    if(ntask != i){
        printf("Nebylo mozne spustit vsechny delniky, koncim.\n");
        pvm_exit();
        exit(1);
    }
    
    for(i = 0; i < ntask; i++){
        printf("Delnik %d pripraven.\n" , i);
        pvm_recv(-1, MSG_READY);
    }
    //zacnem pridelovat praci
    run(bmp_data, size, tids);
    
    printf("Ukoncuji delniky.\n");
    //ukoncime delniky
    pvm_mcast(tids, ntask, MSG_QUIT);
    //ulozime negativ
    if(saveBitmap(bmp_result, bmp_header, bmp_data, size) == 0)
        printf("Bitmapa nebyla ulozena.\n");
    //vymazeme bitmapu z pameti
    free(bmp_data);
    bmp_data = 0;
    
    pvm_exit();
    
    return 0;
}
