import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * 
 * Hlavn tda programu, oetuje vstup a spout vpoet
 *
 */
public class Main 
{

	public static void printInfo()
	{
		System.out.println(
				"Parametry pro spusteni:\n\n" +
				"Nahodna matice (sleduje se pouze doba vypoctu):\n" +
				"\t\t-r <dimenze matice> <exponent> <pocet vlaken> <objem prace>\n\n" +
				"Cteni z externiho souboru:\n" +
				"\t\t-f <nazev souboru> <exponent> <pocet vlaken> <objem prace>\n\n" +
				"Cteni ze standardnho vstupu:\n" +
				"\t\t-i <exponent> <pocet vlaken> <objem prace>");
		
		System.exit(0);
	}
	
	public static void main(String[] args) throws FileNotFoundException 
	{
		Matrix mx = new Matrix();
		
		long[][] sample = {{1,2,3},{4,5,6},{7,8,9}};
		
		int threadCount = 3;
		int dimension = 3;
		int exponent = 2;
		int workCount = 2;
		
		mx.setDimension(dimension);
		mx.setElements(sample);
		
		if (args.length > 0)
		{
			if (args[0].equals("-r"))
			{
				if (args.length != 5)
				{
					printInfo();
				}
				
				try
				{
					dimension   = Integer.parseInt(args[1]);
					exponent    = Integer.parseInt(args[2]);
					threadCount = Integer.parseInt(args[3]);
					workCount   = Integer.parseInt(args[4]);
				}
				catch(NumberFormatException nfe)
				{
					System.out.println("Neplatn parametry !!!");
					printInfo();
				}
				
				mx.setDimension(dimension);
				mx.setElements(new long[dimension][dimension]);
				
				mx.generateRandomElements();
				
			}
			
			if (args[0].equals("-f"))
			{
				if (args.length != 5)
				{
					printInfo();
				}
				
				try
				{
					mx = Matrix.readFromInput(new FileInputStream(args[1]));
					
					exponent    = Integer.parseInt(args[2]);
					threadCount = Integer.parseInt(args[3]);
					workCount   = Integer.parseInt(args[4]);
				}
				catch(NumberFormatException nfe)
				{
					System.out.println("Neplatn parametry !!!");
					printInfo();
				}
				catch(IOException ie)
				{
					System.out.println("Nelze nacist soubor !!!");
					printInfo();
				}
			}
			
			if (args[0].equals("-i"))
			{
				if (args.length != 4)
				{
					printInfo();
				}
				
				try
				{
					mx = Matrix.readFromInput(System.in);
					
					exponent    = Integer.parseInt(args[1]);
					threadCount = Integer.parseInt(args[2]);
					workCount   = Integer.parseInt(args[3]);
				}
				catch(NumberFormatException nfe)
				{
					System.out.println("Neplatne parametry !!!");
					printInfo();
				}
				catch(IOException ie)
				{
					System.out.println("Chyba pri cteni ze vstupu !!!");
					printInfo();
				}
			}
		}
		else
		{
			printInfo();
		}

		Monitor m = new Monitor(mx, exponent, threadCount, workCount);
		
		long timeStamp = System.currentTimeMillis();
		
		m.startWorkers();
		
		m.doWork();
		
		double runTime = ((double) (System.currentTimeMillis() - timeStamp) / 1000);
				
		System.out.println("\nZdrojova matice:");
		
		if (mx.getDimension() > 15)
		{
			System.out.println("Dimenze matice je prilis velka, nebude se vypisovat na terminal");
		}
		else
		{
			mx.writeToOutput();
		}
		
		System.out.println("\nExponent = " + exponent);
		
		System.out.println("\nVysledek vypoctu :");
		
		if (mx.getDimension() > 15)
		{
			System.out.println("Dimenze matice je prilis velka, nebude se vypisovat na terminal");
		}
		else
		{
			m.getResult().writeToOutput();
		}
		
		System.out.println("\nDoba vypoctu : "  + runTime  + "s");
	}

}
