import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Random;

/**
 * Tda poskytujic metody pro operace nad matic 
 */
public class Matrix 
{
	private long elements[][];
	private int dimension;
	
	public Matrix()
	{
		this.dimension = 0;
		this.elements = null;
	}
	
	public Matrix(Matrix m)
	{
		this.dimension = m.getDimension();
		this.elements = new long[dimension][dimension];
		
		long [][] cpElements = m.getElements(); 
		
		for (int i = 0; i < dimension; i++)
		{
			for (int j = 0; j < dimension; j++)
			{
				elements[i][j] = cpElements[i][j];
			}
		}
	}
	
	public Matrix(int dimension) 
	{
		this.dimension = dimension;
		this.elements = new long[dimension][dimension];
	}
	
	public int getElementCount()
	{
		return dimension * dimension;
	}
	
	public void setDimension(int dimension)
	{
		this.dimension = dimension;
	}
	
	public int getDimension()	
	{
		return dimension;
	}
		
	public long[] getColumn(int index)
	{
		long[] column = new long[dimension];
		
		for (int i = 0; i < dimension; i++)
		{
			column[i] = elements[i][index];
		}
		
		return column;
	}
	
	public long[] getRow(int index)
	{
		long[] row = new long[dimension];
		
		for (int i = 0; i < dimension; i++)
		{
			row[i] = elements[index][i];
		}
		
		return row;
	}
	
	public long getElement(int row, int column)
	{
		return elements[row][column];
	}
	
	public void setElement(long value, int row, int column)
	{
		elements[row][column] = value;
	}
		
	public void setElements(long elements[][])
	{
		this.elements = elements;
	}
	
	public long[][] getElements()
	{
		return elements;
	}
	
	public static Matrix readFromInput(InputStream is) throws IOException
	{
		Matrix m = new Matrix();
		
		InputStreamReader converter = new InputStreamReader(is);
		BufferedReader in = new BufferedReader(converter);
        
		if (is == System.in)
		{
			System.out.print("Rad matice =");
		}
		
		String dimLine = in.readLine().trim();
		
		int dimension = Integer.parseInt(dimLine); 
		
		long elements[][] = new long[dimension][dimension];
		
		System.out.println("Zadavejte matici po radcich, jednotlive prvky oddelujte mezerou, zadavani radku ukoncete klavesou enter");
		
		for (int i = 0; i < dimension; i++)
		{
			
			if (is == System.in)
			{
				System.out.print(i + ".radek :  ");
			}
			
			String line = in.readLine();
			
			String[] values = line.split(" ");
			
			for (int k = 0; k < Math.min(dimension, values.length); k++)
			{
				try
				{
					elements[i][k] = Long.parseLong(values[k]);
				}
				catch(NumberFormatException nfe)
				{
					elements[i][k] = 0;
				}
			}
		}

		m.setElements(elements);
		m.setDimension(dimension);
			
		return m;
	}
	
	public void writeToOutput()
	{
		
		for (int i = 0; i < dimension; i++)
		{
			for (int j = 0; j < dimension; j++) 
			{
				System.out.printf(" %d", elements[i][j]);
			} 
			
			System.out.println();
			
		} 
		
	}
	
	public void setElementsToValue(int value)
	{
		for (int i = 0; i < dimension; i++)
		{
			for (int j = 0; j < dimension; j++)
			{
				elements[i][j] = value;
			}
		}
	}
	
	public void generateRandomElements()
	{
		Random ran = new Random();
		
		for (int i = 0; i < dimension; i++)
		{
			for (int j = 0; j < dimension; j++)
			{
				elements[i][j] = ran.nextLong() % 2;
			}
		}
	}
}
