/**
 * 
 * Monitor pro zen dlnk a zpracovvn vsledk, kter se od nich pijmou
 *
 */
public class Monitor 
{
	
	//vstupni matice
	private Matrix source;
	//vysledna matice
	private Matrix result;
	//matice pro ulozeni mezivysledku 
	private Matrix temp;
	
	//pocet elementu matice ktery je zapotrebi jeste zpracovat
	private int elementCounter;
	private int exponent;
	private int workCount;
	
	private int threadCount;
	private int barierCounter;
	
	private Worker[] workers;
	
	public Monitor(Matrix source, int exponent, int threadCount, int workCount) 
	{
		this.source = source;
		this.result = new Matrix(source);
		this.temp = new Matrix(source);
			
		this.exponent = exponent;
		this.threadCount = threadCount;
		this.elementCounter = source.getElementCount(); 
		this.workCount = workCount;
		
		this.workers = new Worker[threadCount];
		
		if (exponent == 0)
		{
			this.result.setElementsToValue(1);
		}
	}
	
	//Startuje delniky
	public void startWorkers()
	{
		if (exponent < 2)
		{
			return;
		}
		
		for (int i = 0; i < threadCount; i++)
		{
			workers[i] = new Worker(source, this);
			workers[i].start();
		}
	}
	
	//Ceka dokud vsichni delnici neskonci svoji praci
	public void doWork()
	{
		if (exponent < 2)
		{
			return;
		}
		
		for (int i = 0; i < threadCount; i++)
		{
			try 
			{
				workers[i].join();
			} 
			catch (InterruptedException e) 
			{
				e.printStackTrace();
			}
		}
	}
	
	//Vyrizuje zadost delnika o dalsi kus prace
	public synchronized WorkPart getNextWorkPart()
	{
		if (elementCounter == 0)
		{
			return null;
		}
		
		if (elementCounter - workCount < 0)
		{
			workCount = elementCounter;
			elementCounter = 0;
		}
		else
		{
			elementCounter -= workCount;			
		}
		
		WorkPart wp = new WorkPart();
		
		wp.setColumnIndex(elementCounter % source.getDimension());
		wp.setRowIndex(elementCounter / source.getDimension());
		wp.setWorkCount(this.workCount);
		
		return wp;
	}
	
	//Nastavi noveho nasobence pro vsechny delniky
	public synchronized boolean getNextMultiplicant()
	{
		if (exponent == 2)
		{
			return false;			
		}
		
		barierCounter++;
		
		if (barierCounter != threadCount)
		{
			try 
			{
				wait();
			} catch (Exception e) 
			{
				e.printStackTrace();
			}
		}
		else
		{
			elementCounter = source.getElementCount();
			exponent--;
			
			temp = new Matrix(result);
			
			for (int i = 0; i < threadCount; i++)
			{
				workers[i].setMultiplicant(temp);
			}
			
			notifyAll();
			
			barierCounter = 0;
		
		}
		
		return true;
	}
	
	//Preda vysledek prace delnika do mezivysledku mocniny 
	public synchronized void updateResult(long[] localResult, WorkPart wp) 
	{
		for (int i = 0; i < wp.getWorkCount(); i++)
		{
			int columnIndex = (wp.getColumnIndex() + i) % source.getDimension();
			int rowIndex = wp.getRowIndex() + (wp.getColumnIndex() + i) / source.getDimension();
		
			result.setElement(localResult[i], rowIndex, columnIndex);
		}
	}
	
	public Matrix getResult()
	{
		return result;
	}
	
}
