/**
 * Logger.java - trida pro logovani udalosti do textoveho souboru.
 *
 * @author Jiri Frolik, A06081, KIV/PPR 2007/08
 **/

import java.io.*;
import java.util.Calendar;

public class Logger {
    
    BufferedWriter out;
    String jmeno;
    Calendar cal;
    int d, m, r, h, min, s, ms;			// pro ukladani casu
    String cas;
    boolean isZapis;                            // umoznen zapis do logovaciho souboru
    
    
    /**
     * Konstruktor.
     */
    public Logger(String jmeno) {
        
        this.jmeno = jmeno;     
        this.isZapis = true;
    }
    
    
    /**
     * Vytvori instanci writeru do souboru zvoleneho jmena.
     */
    public void startLog() throws IOException {
        
        try {
            this.out = new BufferedWriter(new FileWriter(this.jmeno, true));
            this.out.write("\n***** Novy log *****\n");
        }
        catch (IOException ex) {
            this.isZapis = false;	    	
            throw new IOException("Nepodarilo se otevrit logovaci soubor " 
                        + this.jmeno + " pro zapis!"
	    		+ "\n Logovani nebude umozneno.");
        } 	
        
    }
    
    
    /**
     * Zpracuje aktualni cas a spolu s predanou zpravou zapise do souboru a odradkuje.
     */
    synchronized public void zapis(String zprava) {

        if (this.isZapis == false) {		// neumozneno logovani - nezapise se nic
            return;
        }

        this.cal = Calendar.getInstance();

        d = this.cal.get(Calendar.DAY_OF_MONTH);
        m = this.cal.get(Calendar.MONTH) + 1;
        r = this.cal.get(Calendar.YEAR);
        h = this.cal.get(Calendar.HOUR_OF_DAY);
        min = this.cal.get(Calendar.MINUTE);
        s = this.cal.get(Calendar.SECOND);
        ms = this.cal.get(Calendar.MILLISECOND);

        cas = d + "/" + m + "/" + r + " " + h + ":" + min + ":" + s + "." + ms;
        
        try {
            this.out.write(cas + " - " + zprava);
            this.out.newLine();
        } catch (IOException ex) {
            System.out.println("Nastala chyba pri zapisu do logovaciho souboru!\n");
        }
    }

    
    /**
     * Uzavre writer do logovaciho souboru.
     */
    public void stopLog() throws IOException {
        
        try {
            this.out.close();
        }
        catch (IOException ex) {
            this.isZapis = false;
            throw new IOException("Chyba - nepodarilo se uzavrit logovaci soubor"
	    		+ this.jmeno + "!\n");
        }
        
    }
    
}
