import java.io.*;
import java.util.*;
import javax.swing.JTextArea;

/**
 * Trida reprezentujici nacitany textovy soubor
 * @author Pavel
 *
 */

public class Data {
	
	private ArrayList<String> radky;		// seznam nactenych radek textaku
	private String needle;					// hledany podretezec
	private boolean caseSensitive;


    /**
     * Vytvorime datovou tridu
     * @param needle hledany retezec
     * @param caseSensitive jestli se bere ohled na velka a mala pismena
     */
	public Data(String needle, boolean caseSensitive) {
		this.radky = new ArrayList<String>();

        this.caseSensitive = caseSensitive;

		if (caseSensitive == false)
			this.needle = needle.toLowerCase();  		// na mala pismenka pokud neni case sensitive vyhledavani
		else
			this.needle = needle;

	}
	
	/**
	 * @return vraci pocet radku nacteneho textu
	 */
	public int getPocetRadku() {
		return this.radky.size();
	}
	
	/**
	 * Nacte textovy soubor do struktury seznamu radku.
	 * @param soubor file
	 * @return pocet nactenych radek
	 */
	public int loadFromFile(File f, JTextArea textArea, boolean vypsat) {
		try {
			FileReader fr = new FileReader(f);
			BufferedReader in = new BufferedReader(fr);
			String radka;
			
			while ((radka = in.readLine()) != null) {   // cti po radkach
                if (vypsat)
                    textArea.append(radka + "\n");

				if (caseSensitive == false)
					radka = radka.toLowerCase();		// na mala pismenka pokud neni case sensitive vyhledavani
				
				this.radky.add(radka);
				
				//System.out.println(radka);
			}
			
			fr.close();				// zavri soubor

			return radky.size();	// vracim pocet nactenych radek
			
		} catch (IOException e) {
			System.err.println("\nError: pri nacitani textoveho souboru '" + f.getPath() + "'");
			e.printStackTrace();
			return 0;
		}
	}

    /**
     * Vygeneruje text nahodnym generatorem Lorem Ipsum
     * @param radku pocet radku textu
     * @param randomizer inicializace generatoru
     * @param textArea
     * @param vypsat ano ne
     * @return
     */
	public int generateDummy(int radku, int randomizer, JTextArea textArea, boolean vypsat) {
        LoremIpsum lorem = new LoremIpsum();
        int kolikSlov = 20;

        for (int i = 0; i < radku; i++) {
            int startIndex = (((randomizer + i) * 7) + 19) % 40;  // indexy od 0..40
            String radka = lorem.getWords(kolikSlov, startIndex);
            if (vypsat)
                textArea.append(radka + "\n");

            if (caseSensitive == false)
                radka = radka.toLowerCase();		// na mala pismenka pokud neni case sensitive vyhledavani

            this.radky.add(radka);
        }

        return radku;
    }

    /**
     * Hledani podretezce ve svych datech v radcich podle zadanych indexu
     * @param odRadku index odkud
     * @param doRadku index kam
     * @param polePozic pole pro vysledne seznamy pozic
     */
	public void processFind(int odRadku, int doRadku, ArrayList<Integer> polePozic[]) {    // cislovani radku od 0
		for (int r = odRadku; r <= doRadku; r++) {			// zpracujeme po radkach, jen tolik radku kolik je nastavena prace
			String radka = this.radky.get(r);
			polePozic[r] = new ArrayList<Integer>();		// vytvorime prazdny seznam vysledku pro dany radek
			
			//System.out.print("\nradka " + r + " : ");
			
			for (int i = 0; i < radka.length(); i++) {   // resime i prekryvani retezcu, protoze zpracovavam vzdy celou radku
				boolean matched = true;
				for (int j = 0; j < needle.length(); j++) {
					if ((i+j) >= radka.length()) {	// uz vime ze se slovo nevejde do konce radky
						matched = false;
						break;
					}
					if (radka.charAt(i + j) != needle.charAt(j)) {	// alespon 1pismenko neshoduje tak neni shoda
						matched = false;
						break;
					}
				}
				if (matched) {
					//System.out.print(i + ", ");
					polePozic[r].add(i);				// pridame do vysledku daneho radku nalezenou pozici podretezce
				}
			}
		}
		
	}
	
}
