/*
   Jmeno souboru : worker.c
   verze         : 1.1
   kde, kdy, kdo : V Plzni, 1.2.2001 (d.m.r), Milos Siebert

   POPIS:
   Semestralni prace z predmetu Paralelni programovani KIV/PPR
   Program predelany z puvodniho WORKER.C od Stanislava Racka
*/


#include "pvm3.h"

#include <stdlib.h>

#include <stdio.h>

#include <math.h>

typedef struct interval { /* Interval definovany svym zacatkem a koncem */
  double beg, end;
}INTERVAL;

typedef struct obdelnik { /* Obdelnik definovany dvema intervaly */
  INTERVAL intervalX, intervalY;
}OBDELNIK;


double fce (double x, double y, int c) {
  switch (c) {
    case 4: return(x * x + y * y);
    case 3: return(10 * x + 3);
    case 2: return(2 * x + 2 * y + 10);
    default :                   /* Defaultne nastaveno na fci s cislem 1 */
    case 1: return(2 * x + 3);
  }
}

/* nastavi promennou typu Interval tak aby platilo: a < b */
INTERVAL set_i(double a, double b) {

  INTERVAL i;

  if (a <= b) {
    i.beg = a;
    i.end = b;
  }
  else {
    i.beg = b;
    i.end = a;
  }
  return(i);
}

/* Vraci delku intervlu */
double length_i(INTERVAL i) {
  return(i.end  - i.beg);
}

/* Nastaveni konkretni obdelnikove oblasti */
OBDELNIK omega(void) {

  OBDELNIK o;

  o.intervalX = set_i(0, 1);	
  o.intervalY = set_i(-1, 1);	

  return(o);
}


main() {
  int mytid;      		/*  identifikator procesu delnik */
  int chief_tid;  		/*  identifikator procesu sef  */
  int i, k, kolik;   		/*  pomocna promenna */
  double suma;    		/*  vysledek vypoctu */
  int cislo;      		/*  cislo funkce */
  OBDELNIK oblast;              /*  oblast vypoctu funkce */
  double talfa;                 /*  derivace ve smeru x */
  double tbeta;                 /*  derivace ve smeru y */
  double x;                     /*  x-ova souradnice bodu */
  double y;                     /*  y-ova souradnice bodu */
  double x_delka;               /*  sirka oblasti ve smeru x */
  double y_delka;               /*  sirka oblasti ve smeru y */
  double hX;                    /*  prirustek argumentu fce  ve smeru x */
  double hY;        		/*  prirustek argumentu fce  ve smeru y */


  /* Inicializace */
  oblast = omega();
  x_delka = length_i(oblast.intervalX);
  y_delka = length_i(oblast.intervalY);
  hX = 1e-8;
  hY = 1e-8;

  mytid = pvm_mytid();  /* prihlaseni do PVM */

  /* cekani na zpravu od sefa */
  pvm_recv(-1, 4);               	/* 4 je kod zpravy */
  pvm_upkint(&chief_tid, 1, 1);  	/* zjisti cislo sefa */

  /* kontrola zda je sef ten pravy - totozny s otcem */
  if (chief_tid == pvm_parent()) {
    /* odpoved sefovi */
    pvm_initsend(PvmDataDefault);	/* inic. bufferu */
    pvm_pkint(&mytid, 1, 1);    	/* ulozeni identifikace */
    pvm_send(chief_tid, 5);     	/* kod odpovedi 5 */
  }

  /* delnik pracuje */
  while (1) {                  /* !! zadne odbory */
    pvm_recv (chief_tid, 6);   	/* cteni  prikazu */
    pvm_upkint (&cislo, 1, 1); 	/* ulozeni hodnoty zkoumaneho cisla */
    pvm_upkint (&kolik, 1, 1);

    if (!kolik) break;          /* zaporna hodnota znamena konec prace */
    suma = 0.0;

    for(i=0; i<kolik; i++) {
      /* Vygenerovani nahodneho bodu (x, y)
         s rovnomernym rozdelenim na oblasti */
      x = oblast.intervalX.beg + (rand() / (double)RAND_MAX * x_delka);
      y = oblast.intervalY.beg + (rand() / (double)RAND_MAX * y_delka);

      /* vypocet derivaci ve smeru x a y */
      talfa = (fce(x + hX, y, cislo) - fce(x, y, cislo)) / hX;
      tbeta = (fce(x, y + hY, cislo) - fce(x, y, cislo)) / hY;

      k = sqrt(talfa * talfa  +  tbeta * tbeta  +  1);

      suma += k;
    }

    /* odeslani vysledku zkoumani */
    pvm_initsend(PvmDataDefault);  /* inic. bufferu */
    pvm_pkint(&mytid, 1, 1);       /* ulozeni identifikace */
    pvm_pkdouble(&suma, 1, 1);     /* ulozeni vysledku */
    pvm_send(chief_tid, 7);        /* kod odpovedi 5 */
  }

  /* ukonceni cinnosti */
  pvm_exit();
}
