#include <stdio.h>
#include <stdlib.h>
#include <pvm3.h>

#define KONEC_PRACE -1
/**************************************************************/
/* PPR - Uloha c.9                                            */
/* Martin Podval <martin@m4rtin.com>                          */
/* Zadani: vypoctete intenzitu elektrickeho pole pro          */
/*         zadany naboj a vlastnosti prostredi                */
/* Farmar nacte vstupni hodnoty a podle poctu vlaken          */
/* rozdeli praci kazdemu z nich. To vypocte zadany sloupec    */
/* a ulozi jej. V skonceni vsech vypise vysledky              */
/* Pouzite kody zprav:                                        */
/*   4 ... broadcast (id sefa  -> delnici)                    */
/*   5 ... odpoved   (id delnika -> sef)                      */
/*   6 ... zadani ukolu (sef -> delnik),                      */
/*   7 ... vysledek     (delnik -> sef),                      */
/**************************************************************/
 
 
int main(int argc, char *argv[]) { 
    int i=0,j=0, tid;                                               /* Pomocne promenne */
    int mytid;                                                      /* Moje ID */
    
    int tids[32];   	                                            /* Identifikatory procesu delnici */
    int* tidc;      	                                            /* Pomocny ukazatel pro vytvareni delniku */
    int vytvoreno = 0;  	                                        /* Pocet uspesne vytvorenych delniku */    
    
    int naboj_z;                                                    /* Z-tova souradnice */
    double naboj_hodnota;                                           /* Hodnota naboje */
    
    int plocha_x, plocha_y;                                         /* Rozmery plochy */
    double hodnota_intenzity;                                       /* Hodnota prijate intenzity */
    int poslana_x, poslana_y;                                       /* Souradnice, pro ktere prisla intenzita */
    
    int odeslane_x = 0;                                             /* Cislo posledniho odeslaneho sloupce matice intenzit delnikum */
    int prijate_vysledky = 0;                                       /* Pocet prijatych vysledku */
    int celkovych_vysledku;                                         /* Konstantni pocet celkovych vysledku */
    
    double permeabilita = 12.556e-7;                                /* Permeabilita vakua */        
    double permeabilita_r;                                          /* Relativni permeabilita */
    
    int pocet_procesu;                                              /* Pocet spustenych procesu */    
    
    double **intenzita;                                             /* Matice hodnot intenzit */

    /* Zadani parametru */
    printf("Zadejte z-ovou souradnici naboje: ");
    scanf("%d",&naboj_z);
    printf("Zadejte hodnotu naboje: ");
    scanf("%lf",&naboj_hodnota);
    printf("Zadejte x-ovou velikost plochy: ");
    scanf("%d",&plocha_x);
    printf("Zadejte y-ovou velikost plochy: ");
    scanf("%d",&plocha_y);
    printf("Zadejte relativni permeabilitu: ");
    scanf("%lf",&permeabilita_r);
    printf("Pocet procesu: ");
    scanf("%d",&pocet_procesu);

    /* Dynamicka alokace pameti pro matici intenzit */
    intenzita = (double **) malloc (plocha_x * sizeof(double*));
    if(intenzita == NULL){
        printf("Nelze alokovat pamet pro matici intenzit.");
        exit(-1);
    }
    
    for(i = 0; i < plocha_x ; i++){
          intenzita[i] = (double *) malloc(plocha_y * sizeof(double));      
          if(intenzita[i] == NULL){
              printf("Nelze alokovat pamet pro matici intenzit.");
              exit(-1);
          }          
    }
    
    /* Prihlaseni do PVM */
    mytid = pvm_mytid();

    /* vytvoreni delniku */
    tidc = tids;

    while (pocet_procesu>vytvoreno) {
       if (pvm_spawn("worker", (char**)0, 0, "", 1, tidc)) {
          printf("Vytvoren delnik %d.\n", *tidc);
          vytvoreno++;
          tidc++;
       } else {
         printf("Chyba pri vytvareni delnika. Cislo chyby %d\n", *tidc);
       }
    }
    
    /* uvodni komunikace sefa s delniky (registrace prac. sil) */
    pvm_initsend(PvmDataDefault);                                  /* inicializace bufferu */
    pvm_pkint(&mytid, 1, 1);                                       /* ulozeni identifikace */
    pvm_mcast(tids, pocet_procesu, 4);                             /* broadcast zpravy s kodem 4 */
                                                                   /*              vsem delnikum */
    
    /* cekani na odezvu od delniku */
    for(i=0; i<pocet_procesu; i++) {
        pvm_recv(-1, 5);                                           /* 5 je kod zpravy */
        pvm_upkint(&tid, 1, 1);                                    /* cislo delnika */
        printf("Delnik cislo %d k praci pripraven! \n", tid);
    }        

    /* pocatecni ukolovani delniku */
    odeslane_x = 0;
    for(i = 0; i < pocet_procesu && i < plocha_x; i++, odeslane_x++) {

        pvm_initsend(PvmDataDefault);	                           /* inicializace bufferu */

        pvm_pkint(&odeslane_x, 1, 1);
        pvm_pkint(&plocha_y  , 1, 1);
        pvm_pkint(&naboj_z   , 1, 1);

        pvm_pkdouble(&naboj_hodnota , 1, 1);   
        pvm_pkdouble(&permeabilita  , 1, 1);
        pvm_pkdouble(&permeabilita_r, 1, 1);
        
        pvm_send(tids[i], 6);		                               /* zaslani zpravy s kodem 6 */
        printf("Prace delnikovi %d pridelena \n", tids[i]);
    }
    
    celkovych_vysledku = plocha_x * plocha_y;
    /* Dokud je co odesilat, prijimame vysledky a zadavame volnym vlaknum dalsi praci */
    while(odeslane_x < plocha_x){
        pvm_recv(-1,7);
        pvm_upkint(&tid,1,1);
        pvm_upkint(&poslana_x,1,1);
        pvm_upkint(&poslana_y,1,1);
        pvm_upkdouble(&hodnota_intenzity,1,1);
        
        /* Pokud nema co vlakno delat, dostane novou praci */
        if(poslana_x == KONEC_PRACE && poslana_y == KONEC_PRACE){
             pvm_initsend(PvmDataDefault);	                       /* inicializace bufferu */

             pvm_pkint(&odeslane_x, 1, 1);
             pvm_pkint(&plocha_y  , 1, 1);
             pvm_pkint(&naboj_z   , 1, 1);

             pvm_pkdouble(&naboj_hodnota , 1, 1);   
             pvm_pkdouble(&permeabilita  , 1, 1);
             pvm_pkdouble(&permeabilita_r, 1, 1);
             
             pvm_send(tid, 6);		                           /* Zaslani zpravy s kodem 6 */
             printf("Prace delnikovi %d pridelena \n", tids[i]);            
             odeslane_x++;                                         /* Pricteme pocet odeslanych sloupcu */
        }
        /* Pokud dosel vysledek, ulozime ho */
        else{
             intenzita[poslana_x][poslana_y] = hodnota_intenzity;
             prijate_vysledky ++;                                  /* Pricteme pocet prijatych vysledku */
        }
    }    
    
    /* Dale musime jeste pockat na prijmuti zbylych vysledku */
    while(prijate_vysledky < celkovych_vysledku){
        pvm_recv(-1,7);
        pvm_upkint(&tid,1,1);
        pvm_upkint(&poslana_x,1,1);
        pvm_upkint(&poslana_y,1,1);
        pvm_upkdouble(&hodnota_intenzity,1,1);
        
        /* Pokud vlakno neoznamuje konec prace, ukladame hodnotu */
        if(poslana_x != KONEC_PRACE && poslana_y != KONEC_PRACE){
            intenzita[poslana_x][poslana_y] = hodnota_intenzity;
            prijate_vysledky ++;                                  /* Pricteme pocet prijatych vysledku */
        }                     
    }
    
    /* Vypis vysledku na obrazovku */
    printf("Vysledky vypoctu\n");
    for(i = 0; i < plocha_x; i++){
        for(j = 0; j < plocha_y; j++){
            printf("[%d, %d, %d] E = %E\n",i,j,(- naboj_z), intenzita[i][j]);
        }      
    }
    
    /* Uvolneni dynamicke pameti */
    for(i = 0; i < plocha_x; i++){
        free(intenzita[i]);
    }
    free(intenzita);
    
    pvm_exit();
    return 0;
}
