/* ========================================================================== */
/*                                                                            */
/*   kvadr_farmer.c                                                           */
/*   Jiri Frolik A06081 KIV/PPR 2007/08                                       */
/*                                                                            */
/*   Modul pro farmera rozdelujici praci workerum, kteri pocitaji             */
/*   hmotnost kvadru.                                                         */
/*                                                                            */
/* ========================================================================== */


//#include <stdio.h>
#include <pvm3.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "kvadr_zpracovani.h"
#include "kvadr_worker.h"


int mytid;                /* identifikator procesu sef */
int *tids;                /* identifikatory procesu delnici */
int *tidc;                /* pomocny ukazatel pro vytvareni delniku */
int tid;                  /* identifikacni cislo delnika */

double assigned_part;     /* pridelena cast vypoctu workerovi */
double assigned_total;    /* mnozstvi jiz pridelene prace */
KVADR kvadr;              /* struktura, ktera obsahuje hodnoty x, y, z */
int function_type;        /* typ funkce ro(x,y,z), se ktery, se pocita */

double result_weight;     /* dilci hmotnost vracena workerem */
double final_weight;      /* celkova vypoctena hmotnost */

int thread_count;         /* pocet workeru */
int thread_count_created; /* pocet uspesne vytvorenych delniku */
int thread_count_running; /* pocet bezicich workeru */

double dx;
double dy;
double dz;                /* prirustky po osach x, y, z */

int computing;            /* priznak, zda farmer jeste pocita 0=false, 1=true */

clock_t start_time;
clock_t stop_time;        /* casove udaje pro mereni casu vypoctu */
double final_time;        /* celkovy cas trvani vypoctu */

FILE *fw_log;             /* pointer na logovaci soubor */
int logging;              /* umozneno logovani (1 = true, 0 = false)*/
char *msg;                /* retezec, ktery se bude logovat */

/* Funkce otevre soubor LOG_FILE pro logovani. */
void start_log() {
  if ((fw_log = fopen(LOG_FILE, "w")) == NULL) {
    printf("Logovaci soubor %s se nepodarilo otevrit!\n", LOG_FILE);
    printf("Logovani nebude umozneno.\n");
    return;
  }
  printf("Logovaci soubor %s otevren.\n", LOG_FILE);
  logging = 1;             /* umozneno logovani */
}


/* Funkce uzavre logovaci soubor LOG_FILE. */
void stop_log() {
  if (fclose(fw_log) == EOF) {
    printf("Logovaci soubor %s se nepodarilo uzavrit!\n", LOG_FILE);
  }
  printf("Logovaci soubor %s uzavren.\n", LOG_FILE);
  logging = 0;             /* konec moznosti logovani */
}

/* Funkce zaloguje udalost danou retezcem msg do souboru LOG_FILE. */
void log_to_file(char *par_msg) {
  if (logging == 1) {   /* pokud byl otevren logovaci soubor - lze logovat */
    fprintf(fw_log, "%s", par_msg);
  }
}


/* Funkce inicializuje hodnoty globalnich promennych podle nactenych dat. */
void init_farmer(KVADR par_kvadr, double d_x, double d_y, double d_z,
      int t_count, double assign_part, int f_type) {
      
  int i;                  /* promenna cyklu */
      
  final_weight = 0.0;     /* zatim nulovy celkovy vysledek */
  assigned_total = 0.0;   /* jeste nebyla pridelena zadna prace */
  computing = 0;          /* false */
  thread_count = t_count;     /* pocet workeru celkem */
  thread_count_running = 0;   /* pocet bezicich workeru */
  thread_count_created = 0;   /* pocet vytvorenych workeru */
  dx = d_x;
  dy = d_y;
  dz = d_z;
  kvadr = par_kvadr;
  assigned_part = assign_part;
  function_type = f_type; /* nastaveni potrebnych promennych dle parametru */
  
  printf("farmer spusten\n");
  
  /* alokovani pameti pro logovaci retezec: */
  msg = (char *) malloc(MAX_STR_LEN * sizeof(char));

  start_time = clock();   /* zacatek mereni casu vypoctu */

  mytid = pvm_mytid();    /* prihlaseni do pvm */
  
  /* vytvoreni delniku */
  tids = (int *) malloc(thread_count * sizeof(int));  /* alokace pameti */
  
  tidc = tids;
  /* Dokud vytvarim workery: (dokud nebyli vsichni vytvoreni) */
  while (thread_count_created < thread_count) {
    /* Spusteni worker.exe jako delnika: */
    if (pvm_spawn("worker.exe", (char**)0, 0, "", 1, tidc)) {
   
      sprintf(msg, "Vytvoren worker %d.\n", *tidc);
      printf(msg);
      log_to_file(msg);                   /* vypis + zalogovani */
      
      thread_count_created++;
      tidc++;
      
    } else { 
      printf("Chyba pri vytvareni workera. Cislo chyby %d\n", *tidc); 
    } 
  }
  
  /* uvodni komunikace farmera s workery (registrace prac. sil) */  
  pvm_initsend(PvmDataDefault);   /* inicializace bufferu */
  pvm_pkint(&mytid, 1, 1);        /* ulozeni identifikace */
  pvm_mcast(tids, thread_count, IDLE);   /* broadcast workerum - vytvoreni */

  /* cekani na odezvu od workeru */
  for(i = 0; i < thread_count; i++) { 
  
    pvm_recv(-1, RESPONSE);              /* worker vrati odpoved */
    pvm_upkint(&tid, 1, 1);              /* cislo workera */  
    
    sprintf(msg, "Worker cislo %d pripraven!\n ", tid);
    printf(msg);
    log_to_file(msg);                    /* vypis + zalogovani */
    
  }
  
  
}




/* Funkce posle data danemu workerovi. */
void send_to_worker_at(int index, double from_x, double from_y, double from_z,
			double to_x, double to_y, double to_z, int function_type) {

  /* Poslu workerovi pridelenou cast kvadru ke zpracovani: */
  pvm_initsend(PvmDataDefault);   /* inicializace bufferu */
  
  pvm_pkdouble(&from_x, 1, 1);
  pvm_pkdouble(&from_y, 1, 1);
  pvm_pkdouble(&from_z, 1, 1);
  pvm_pkdouble(&to_x, 1, 1);
  pvm_pkdouble(&to_y, 1, 1);
  pvm_pkdouble(&to_z, 1, 1);
  pvm_pkdouble(&dx, 1, 1);
  pvm_pkdouble(&dy, 1, 1);
  pvm_pkdouble(&dz, 1, 1);
  pvm_pkint(&function_type, 1, 1);  /* poslu vsechna data*/
  
  pvm_send(index, WORK);            /* poslu zpravu, aby zacal pracovat */


}


/* Funkce prideli cast kvadru danemu workerovi k provedeni vypoctu. */
void assign_to_worker_at(int index) {

  double from_x = assigned_total * kvadr.x;
  double from_y = 0.0;
  double from_z = 0.0;    // pocita se po "tenkych ctvercich" podle osy x
  double to_x;
  double to_y = kvadr.y;
  double to_z = kvadr.z;  /* nastaveni vsech hodnot pro workera */

  /* dokud je jeste co pocitat - nebylo prideleno celych 100% casti: */
  if (assigned_total < ASSIGNED_DONE) {
    /* budu pocitat posledni kousek - dalsi uz presahuje kvadr */
    if ((assigned_total + assigned_part) >= ASSIGNED_DONE) {
 
      to_x = kvadr.x;     /* donastaveni to_x - podle toho, zda je posledni */
      
      /* odeslani dat: */
      send_to_worker_at(index, from_x, from_y, from_z, to_x, to_y, to_z,
              function_type);
      
      assigned_total = ASSIGNED_DONE;   /* posledni kousek prace prirazen */
    }
    else {
    
      to_x = (assigned_total + assigned_part) * kvadr.x;  /* dalsi hodnota x */
      
      /* odeslani dat: */
      send_to_worker_at(index, from_x, from_y, from_z, to_x, to_y, to_z,
              function_type);
              
      assigned_total += assigned_part;  /* pricteni casti pridelene prace */
    }
    
    thread_count_running++;
    
  }
  else {
    
    pvm_send(index, DIE);      /* poslu zpravu, aby worker ukoncil cinnost */
  
  }

}


/* Funkce spusti pracovni proces farmera. (spousti se po init_farmer()) */
void run_farmer() {

  int i;                          /* promenna cyklu */
  int is_work_left = 1;           /* je jeste co pocitat; 0=false, 1=true */
  
  for (i = 0; i < thread_count; i++) {
    assign_to_worker_at(tids[i]); /* prideleni prace danemu workerovi */
  }
  
  while (is_work_left == 1) {     /* dokud je co pocitat */
    
    if (pvm_nrecv(-1, RESULT) > 0) {      /* dostal jsem vysledek*/
            
      /* Vyhodnoceni zaslaneho vysledku */
      pvm_upkint(&tid, 1, 1);
      pvm_upkdouble(&result_weight, 1, 1);
      
      sprintf(msg, "Prijata hmotnost od delnika %d: %f\n", tid, result_weight);
      printf(msg);
      log_to_file(msg);           /* vypis + logovani */
      
      final_weight += result_weight; /* pricteni vysledku k celkove hmotnosti */
      thread_count_running--;
        
      /* Pokud je jeste nejaka prace, pridelim workerovi dalsi cast: */
      assign_to_worker_at(tid);            
    }
    
    /* Pokud neni co pocitat: */
    if ((thread_count_running <= 0) && (assigned_total >= ASSIGNED_DONE)) {
      is_work_left = 0;         /* neni prace - skonci cyklus */
    }
  
  } /* end of while */
  
  /* Ukonceni delniku */
  printf("Farmer: Ukoncuji delniky. Cekam na odezvu...\n");
  log_to_file("Farmer: Ukoncuji delniky. Cekam na odezvu...\n");  /* logovani */
  pvm_mcast(tids, thread_count, DIE);  /* broadcast zpravy s kodem KONEC */
  
  /* cekani na odezvu od delniku */
  for(i = 0; i < thread_count; i++) { 
  
    pvm_recv(-1, DEAD);                 /* zprava od workera - mrtvy */
    pvm_upkint(&tid, 1, 1);             /* id delnika */  

    sprintf(msg, "Farmer: Delnik cislo %d ukoncen\n", tid);
    printf(msg);
    log_to_file(msg);                   /* vypis + logovani */
  } 

  pvm_exit();           /* konec farmera */
  
  stop_time = clock();   /* konec mereni casu vypoctu */
  final_time = (double)(stop_time - start_time) / (double)CLOCKS_PER_SEC * 1000;

  sprintf(msg, "Celkova hmotnost kvadru je: %f\n", final_weight);
  printf(msg);
  log_to_file(msg);                     /* vypis + logovani */
  sprintf(msg, "Celkova doba vypoctu je %6.5f ms.\n", final_time);
  printf(msg);
  log_to_file(msg);                     /* vypis + logovani */
  
  stop_log();                           /* ukonceni logovani do souboru */
  
  free ((void *) msg);
  free ((void *) tids);

}

