/* ========================================================================== */
/*                                                                            */
/*   kvadr_hlavni.c                                                           */
/*   Jiri Frolik A06081 KIV/PPR 2007/08                                       */
/*                                                                            */
/*   Modul s funkcemi nacitani parametru pro vypocet hmotnosti kvadru.        */
/*   Navic obsahuje funkci main() pro spusteni farmera.                       */
/*                                                                            */
/* ========================================================================== */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* zahrnu modul farmera: */
#include "kvadr_farmer.c"

int thread_count;		       /* pocet workeru, co bude zpracovavat ulohu */
int function_type;		     /* jeden ze tri typu funkce pro vypocet hustoty */
double assigned_part;	     /* cast vypoctu pridelovana workerum (cislo < 1)*/

KVADR kvadr;               /* struktura kvadru */
double dx;
double dy;
double dz;                 /* prirustky po osach x, y, z */




/* Funkce zkontroluje nactene parametry, jsou-li zadany ve spravnem tvaru. */
int check_attributes() {
		
	/* Delkove parametry + pocet workeru musi byt kladne, assigned_part
     musi lezet v intervalu (0,1> a typ funkce musi byt jedna z hodnot 0,1,2. */
	if ((kvadr.x <= 0) || (kvadr.y <= 0) || (kvadr.z <= 0)
			|| (dx <= 0) || (dy <= 0) || (dz <= 0)
			|| (assigned_part <= 0) || (assigned_part > 1) || (thread_count <= 0)) {
		return 1;
	}
	if ((function_type < 0) && (function_type > 2)) {
		return 1;
	}
	return 0;
}



/* Funkce nacte pozadovane parametry vypoctu ze zadaneho souboru. */
int configure_from_file(char *filename) {
  
  FILE *fr;               /* pointer na konfiguracni soubor */
  
  if ((fr = fopen(filename, "r")) == NULL) {
    printf("Chyba pri cteni ze zadaneho konfiguracniho souboru %s!\n",filename);
    return 1;
  }
  
  /* vlastni nacteni parametru: */
  if (fscanf(fr, "%lf %lf %lf %lf %lf %lf %d %lf %d", &kvadr.x, &kvadr.y, &kvadr.z,
          &dx, &dy, &dz, &thread_count, &assigned_part, &function_type) != 9) {
    printf("Chybne zadane parametry v konfiguracnim souboru!\n");
    return 1;
  }
  
  if (fclose(fr) == EOF) {
    printf("Soubor %s se nepodarilo uzavrit!\n", filename);
    return 1;
  }

  kvadr.x0 = KVADR_X_0;
  kvadr.y0 = KVADR_Y_0;
  kvadr.z0 = KVADR_Z_0;   /* defaultni hodnoty: roh kvadru = pocatek souradnic*/

  return (check_attributes());    /* vrati vysledek kontroly nactenych hodnot */

}


/* Funkce nacte pozadovane hodnoty pro vypocet z parametru prikazove radky. */
int configure_from_args(int argc, char *argv[]) {

  /* postupne nactu vsechny parametry argv[]: */
  kvadr.x = atof(argv[1]);
  kvadr.y = atof(argv[2]);
  kvadr.z = atof(argv[3]);
  dx = atof(argv[4]);
  
  if (argc == 8) {        /* dy = dz = dx -> zadano pouze dx */
    dy = atof(argv[4]);
    dz = atof(argv[4]);
    thread_count = atoi(argv[5]);
    assigned_part = atof(argv[6]);
    function_type = atoi(argv[7]);
  } 
  else {                  /* hodnoty dx, dy a dz jsou ruzne - zadadny zvlast */
    dy = atof(argv[5]);
    dz = atof(argv[6]);
    thread_count = atoi(argv[7]);
    assigned_part = atof(argv[8]);
    function_type = atoi(argv[9]);
  }
  
  kvadr.x0 = KVADR_X_0;
  kvadr.y0 = KVADR_Y_0;
  kvadr.z0 = KVADR_Z_0;   /* defaultni hodnoty: roh kvadru = pocatek souradnic*/
  
  return (check_attributes());    /* vrati vysledek kontroly nactenych hodnot */

}


/* Funkce vypise napovedu k programu kvadr_worker. */
void print_info() {

  printf("\n--- Navod na spusteni programu: ---\n-\n");
	printf("- Program kvadr_hlavni se spousti bud bez parametru (bude zobrazena tato napoveda),\n");
	printf("- nebo s jednim parametrem (jmenem konfiguracniho souboru nachazejiciho se\n");
	printf("- v adresari 'config'), nebo je mozne zadat vsechny parametry primo, ve tvaru:\n-\n");
	printf("- farmer.exe <x> <y> <z> <dx> [<dy> <dz>] <poc_work> <prid_cast> <typ_func>\n-\n");
	printf("- Parametry dy a dz lze vynechat, pokud jsou stejne jako <dx>. Napriklad:\n");
	printf("- farmer.exe 5.5 4.4 3.3 0.01 20 0.01 0\n-\n");
	printf("- Typ funkce nabyva hodnot 0 / 1 / 2 pro vysledky funkce ro = 1 / x / x+y+z.\n-\n");
	printf("- Pozadovana struktura konfiguracniho souboru je uvedena v dokumentaci.\n");
}


/* Funkce vypise nactene parametry vypoctu. */
void print_pars() {
  
  printf("Nactene parametry vypoctu:\n");
  printf("x = %f, y = %f, z = %f\n", kvadr.x, kvadr.y, kvadr.z);
  printf("workeru = %d, cast_prace = %f, funkce = %d\n",
      thread_count, assigned_part, function_type);

}

/* Funkce zaloguje nactene parametry vypoctu. */
void log_pars() {

  char *message;    /* retezec k zapisu */
  
  message = (char *) malloc(MAX_STR_LEN * sizeof(char));
  
  log_to_file("Nactene parametry vypoctu:\n");
  sprintf(message, "x = %f, y = %f, z = %f\n", kvadr.x, kvadr.y, kvadr.z);
  log_to_file(message);
  sprintf(message, "workeru = %d, cast_prace = %f, funkce = %d\n",
      thread_count, assigned_part, function_type);
  log_to_file(message);
  
  free ((void *) message);
  
}


/* Hlavni funkce programu, nacte parametry potrebne k vypoctu
   a spusti farmera.*/
int main(int argc, char *argv[]) {
  
  char *config_name;        /* jmeno konfiguracniho souboru */
  
  /* rozhodovani se podle poctu parametru programu: */
  switch (argc) {
    case 1:         /* nezadan zadny parametr - vypise se napoveda */
      print_info();
      return 0;
    break;
    case 2:         /* k nacteni parametru se pouzije konfiguracni soubor */
      config_name = (char *) malloc((strlen(CONFIG_PATH)+strlen(argv[1])+2) 
                                  * sizeof(char));
      strcpy(config_name, CONFIG_PATH);
      if (configure_from_file(strcat(config_name, argv[1])) == 1) { 
        return 1;
      }
    break;
    case 8:
    case 10:        /* dve moznosti nacitani parametru primo z argv[] */
      if (configure_from_args(argc, argv) == 1) {
        printf("Chybne zadane parametry programu! Pro zobrazeni napovedy\n");
        printf("spustte program bez parametru.\n");
        return 1;
      }
    break;
    default:        /* spatne zadany pocet parametru programu */
      printf("Chybny pocet parametru programu! Pro zobrazeni napovedy\n");
      printf("spustte program bez parametru.\n\n");
      return 1;
    break;
  }
  
  start_log();    /* otevreni logovaciho souboru */
  
  print_pars();
  log_pars();     /* vypis a zalogovani nactenych hodnot */

  /* inicializace farmera a jeho nasledne spusteni: */
  init_farmer(kvadr, dx, dy, dz, thread_count, assigned_part, function_type);
  run_farmer();
  
  return 0;
  
}
