/* ========================================================================== */
/*                                                                            */
/*   kvadr_worker.c                                                           */
/*   Jiri Frolik A06081 KIV/PPR 2007/08                                       */
/*                                                                            */
/*   Modul workera vykonavajiciho dilci vypocty hmotnosti.                    */
/*                                                                            */
/* ========================================================================== */

#include <stdio.h>
#include <pvm3.h>
#include "kvadr_worker.h"
#include "kvadr_funkce.h"

int mytid;      /*  identifikator procesu worker  */
int chief_tid;  /*  identifikator procesu farmer  */

int current_state;  /* aktualni stav workera */
double part_weight; /* vysledek dilciho vypoctu workera */


/* Funkce, ktera spocte hodnotu ro(x,y,z) na zaklade vyberu druhu funkce ro. */
double compute_ro(double x, double y, double z, int function_type) {

  double ro;
		
	switch (function_type) {       /* vyber funkce */
		case 0: ro = ro_1(x, y, z); break;
		case 1: ro = ro_x(x, y, z); break;
		case 2: ro = ro_xyz(x, y, z); break;
		default: ro = 0; break;
	}
		
	return ro;

}



/* Funkce, ktera provede vypocet hmotnosti csati kvadru na zaklade
   predanych parametru a vrati vyslednou dilci hmotnost. */
double compute_part_weight(double from_x, double from_y, double from_z,
			double to_x, double to_y, double to_z,
			double dx, double dy, double dz,
			int function_type) {
  
  double iter_x, iter_y, iter_z;      // promenne cyklu
  double part_weight = 0;             // vysledek vypoctu dilci hmotnosti
  
    /* usek kvadru pres rozmer x: */
		for (iter_x = from_x; iter_x < to_x; iter_x += dx) {
			/* usek kvadru pres rozmer y: */
			for (iter_y = from_y; iter_y < to_y; iter_y += dy) {
				/* usek kvadru pres rozmer z: */
				for (iter_z = from_z; iter_z < to_z; iter_z += dz) {
					
					/* pricteni spoctene casti hmotnosti podle vzorce: */
					part_weight += dx * dy * dz * compute_ro(
											iter_x + dx/2,
											iter_y + dy/2,
											iter_z + dz/2, function_type); 
					
				}
			}
		}
  
  current_state = DONE;               /* momentalni vypocet skoncil */

  return part_weight;
}




/* Funkce prevezme data od farmera a provede vypocet hmotnosti zadane casti
   kvadru. Vysledek vrati. */
double init_and_compute() {

  double from_x;
  double from_y;
  double from_z;            /* hodnoty x y z, od kterych se pocita */
  double to_x;
  double to_y;
  double to_z;              /* hodnoty x y z, od kterych se pocita */
  double dx;
  double dy;
  double dz;                /* hodnoty prirustku dx, dy, dz */
  int function_type;

  double result;            /* vysledek vypoctu */
  
  /* prevzeti dat: */
  pvm_upkdouble(&from_x, 1, 1);
  pvm_upkdouble(&from_y, 1, 1);
  pvm_upkdouble(&from_z, 1, 1);
  pvm_upkdouble(&to_x, 1, 1);
  pvm_upkdouble(&to_y, 1, 1);
  pvm_upkdouble(&to_z, 1, 1);
  pvm_upkdouble(&dx, 1, 1);
  pvm_upkdouble(&dy, 1, 1);
  pvm_upkdouble(&dz, 1, 1);
  pvm_upkint(&function_type, 1, 1);
  
  /* provedeni vlastniho vypoctu: */
  result = compute_part_weight(from_x, from_y, from_z,
            to_x, to_y, to_z, dx, dy, dz, function_type);
  

  return result;
}



/* Hlavni funkce workera. */
int main(int argc, char **argv) {

  mytid = pvm_mytid();  /* prihlaseni do PVM */

  /* cekani na zpravu od farmera */
  pvm_recv(-1, IDLE);             /* zprava IDLE - worker vytvoren */
  pvm_upkint(&chief_tid, 1, 1);  /* zjisti cislo farmera*/  

  current_state = IDLE;       /* worker vytvoren */

  /* kontrola zda je farmer ten pravy - totozny s otcem */ 
  if (chief_tid == pvm_parent()) {
    /* odpoved farmerovi */  
    pvm_initsend(PvmDataDefault);/* inic. bufferu */
    pvm_pkint(&mytid, 1, 1);    /* ulozeni identifikace */
    pvm_send(chief_tid, RESPONSE);     /* vlastni odpoved farmerovi */
  }

  /* Dokud nebyl worker umrtven: */
  while (current_state != DEAD) {

		/* Pokud poslal farmer zpravu WORK: */
    if (pvm_nrecv(chief_tid, WORK) > 0) {
               
      part_weight = init_and_compute();   /* prevezme data a provede vypocet */

      /* Poslani vysledku */
      pvm_initsend(PvmDataDefault);  /* inic. bufferu */
      pvm_pkint(&mytid, 1, 1);
      pvm_pkdouble(&part_weight, 1, 1);
      pvm_send(chief_tid, RESULT);     /* poslani zpravy */   
      
    }
    
    /* Pokud farmer posle ukoncovaci zpravu: */
    if (pvm_nrecv(chief_tid, DIE) > 0) {
      current_state = DEAD;
    }    

	}  /* end of while */

  /* Zaslani zpravy farmerovi - worker uz je "mrtev". */
  pvm_pkint(&mytid, 1, 1);             /* ulozeni identifikace */
  pvm_send(chief_tid, DEAD);      /* poslu svuj stav - worker nezije */


  /* ukonceni cinnosti */
  pvm_exit();
  
  return 0;

}
