#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <pvm3.h>

#define KONEC_PRACE -1
#define PI 3.141592653

/**************************************************************/
/* PPR - Uloha c.9                                            */
/* Martin Podval <martin@m4rtin.com>                          */
/* Zadani: vypoctete intenzitu elektrickeho pole pro          */
/*         zadany naboj a vlastnosti prostredi                */
/* Farmar nacte vstupni hodnoty a podle poctu vlaken          */
/* rozdeli praci kazdemu z nich. To vypocte zadany sloupec    */
/* a ulozi jej. V skonceni vsech vypise vysledky              */
/* Pouzite kody zprav:                                        */
/*   4 ... broadcast (id sefa  -> delnici)                    */
/*   5 ... odpoved   (id delnika -> sef)                      */
/*   6 ... zadani ukolu (sef -> delnik),                      */
/*   7 ... vysledek     (delnik -> sef),                      */
/**************************************************************/
 
int main(int argc, char *argv[]) {
    int i;                                                          /* Pomocne promenne */
    int mytid;                                                      /* Moje ID */
    int chief_tid;  		                                        /* Identifikator procesu farmare */     
        
    int naboj_z;                                                    /* Z-tova souradnice */
    double naboj_hodnota;                                           /* Hodnota naboje */
    
    int pocitana_souradnice_x;                                      /* Sloupec, ktery pocitame */
    int velikost_sloupce;                                           /* Udava pocet prvku ve sloupci */
    
    double permeabilita;                                            /* Permeabilita vakua */        
    double permeabilita_r;                                          /* Relativni permeabilita */
    
    double vysledek;                                                /* Vysledna hodnota intenzity */
    double konstantni_cast;                                         /* Cast, kterou nemusime v cyklu znovu pocitat */
    double polomer;                                                 /* Vzdalenost pocitaneho bodu od naboje */
    
    /* Prihlaseni do PVM */
    mytid = pvm_mytid();

    /* Pockame na zpravu od farmare */
    pvm_recv(-1, 4);               	                                /* 4 je kod zpravy */
    pvm_upkint(&chief_tid, 1, 1);  	                                /* Zjisti cislo sefa */

    /* Provname farmare se skutecnym rodicem */
   if (chief_tid == pvm_parent()) {
       pvm_initsend(PvmDataDefault);	                            /* Inic. bufferu */
       pvm_pkint(&mytid, 1, 1);    	                                /* Ulozeni identifikace */
       pvm_send(chief_tid, 5);
   }
 
   while(1){
       pvm_recv (chief_tid, 6);   	
       
       /* Pockame na hodnoty od farmare */
       pvm_upkint(&pocitana_souradnice_x,1,1);
       pvm_upkint(&velikost_sloupce,1,1);
       pvm_upkint(&naboj_z,1,1); 

       pvm_upkdouble(&naboj_hodnota,1,1);
       pvm_upkdouble(&permeabilita,1,1);
       pvm_upkdouble(&permeabilita_r,1,1);
       
       /* Cast, ktera neni zavisla na pozici, je stale stejna */
       konstantni_cast = 4 * PI * permeabilita * permeabilita_r;
       
       /* Vypocteme vsechny hodnoty pro dany sloupec */
       for(i = 0; i < velikost_sloupce; i++){
           polomer = sqrt( ((float)(pocitana_souradnice_x*pocitana_souradnice_x) + (i*i)) + ((float)(naboj_z*naboj_z)));
           if(polomer>0)    vysledek = abs(naboj_hodnota) / (konstantni_cast * polomer * polomer);
           else             vysledek = 0;
           
           pvm_initsend(PvmDataDefault);
		   pvm_pkint(&mytid,1,1);			                        /* Ulozeni identifikace */
		   pvm_pkint(&pocitana_souradnice_x,1,1);				    /* Cislo sloupce */
		   pvm_pkint(&i,1,1);                            			/* Cislo na souradnici Y */
		   pvm_pkdouble(&vysledek,1,1);	                            /* Hodnota vysledku */
		   pvm_send(chief_tid, 7);			                        /* odeslani */ 
       }
       
       /* Po ukonceni vypoctu odesleme ukonceni vypoctu */
       pocitana_souradnice_x = KONEC_PRACE;
       i = KONEC_PRACE;
       vysledek = 0;
       
       pvm_initsend(PvmDataDefault);
       pvm_pkint(&mytid,1,1);			                        /* Ulozeni identifikace */
	   pvm_pkint(&pocitana_souradnice_x,1,1);				    /* Cislo sloupce */
	   pvm_pkint(&i,1,1);                            			/* Cislo na souradnici Y */
	   pvm_pkdouble(&vysledek,1,1);	                            /* Hodnota vysledku */
	   pvm_send(chief_tid, 7);			                        /* odeslani */ 
   }
   
   pvm_exit();    
   return 0;
}