// Module of monitor which can repeatedly make sum of
//   a given vector which contains items of the double type   
      
// Internal private class describing "piece of work"
class Cl_part_work {
  public VypData data;       // reference to data
  public int pocet;          // Pocet pocitanych bodu

  public Cl_part_work (VypData par_data, int par_pocet) {
    data = par_data;
    pocet = par_pocet;
  }
  public boolean empty () {  // returns true for empty work 
    if (pocet == 0)
      return true;
    else
      return false;
  } 
}


// public class - monitor
public class Cl_parop {

  // inner static class for worker threads
  private static class Cl_worker extends Thread {

    private Cl_parop monitor_ref;    // reference to monitor
                                     //    instance
    private int my_id;               // num. of worker
    private double loc_suma;         // Suma vypocitanych koeficientu

	private double talfa;            // Tangens alfa
    private double tbeta;            // Tangens beta
    private double koef;             // Vysledny koeficient
    private double hX = 1e-8;        // Krok ve smeru X
    private double hY = 1e-8;        // Krok ve smeru Y
    private double x;                // Souradnice
    private double y;                // Souradnice

    public Cl_worker (Cl_parop par_monitor_ref, int id) {
      monitor_ref = par_monitor_ref;
      my_id = id;
    }

    public void run () {             // worker activity
      while (!monitor_ref.stop_workers) {
        /* worker requests a piece of work */
        Cl_part_work part = monitor_ref.get_work(); // blocking

        /* sum operation */
        loc_suma = 0;
//-------------------------------------------------------------------
        for (int i = 0; i<part.pocet; i++) {
          // Nahdne vygenerovany bod (x, y) na oblasti 
          x = java.lang.Math.random() * 
                  monitor_ref.oblast.intervalX.length() + 
                           monitor_ref.oblast.intervalX.beg();
          y = java.lang.Math.random() * 
                  monitor_ref.oblast.intervalY.length() +
                           monitor_ref.oblast.intervalY.beg();

          // smernice tecny ve smeru X v bode (x,y)
          // pocitana z jednostrane diference
          talfa = (monitor_ref.B.fce(x + hX, y) - 
                           monitor_ref.B.fce(x, y)) / hX;

          // smernice tecny ve smeru Y v bode (x,y)
          // pocitana z jednostrane diference
          tbeta = (monitor_ref.B.fce(x, y + hY) - 
                           monitor_ref.B.fce(x, y)) / hY;

          koef  = java.lang.Math.sqrt(talfa * talfa + tbeta * tbeta + 1);

          loc_suma += koef;
		  //System.out.println (" [m: " + my_id + ", i: " + i + "]");
        }
        
//-------------------------------------------------------------------
        yield();
        /* return of result */
        monitor_ref.inc(loc_suma, part.pocet);    // blocking
        //System.out.println(my_id +"  " + loc_suma + "  " + part.pocet);
      }
      //System.out.println("worker " + my_id + " terminated");
    }
  }


  //  monitor's data members
  private VypData  data;
  private int      m;             // Celkovy pocet zpracovavanych bodu
  private double   glob_suma;     // Globalni suma koeficientu
  private int      glob_pocet;    // Pocet bodu

  private Obdelnik oblast;	// Pravouhla oblast v rovine XY
  private Plocha   B;		// Funkce plochy 

  private int n;               // num of workers
  private int barrier_cnt;     // counter of sync. barrier
  private int step;            // vector part length
  private double op_result;    // result of operation
  private int next_index;      // first index of first free part
  private boolean op_running;  // flag of running op.
  private boolean free_work;   // flag - there is still
                               //   a free part of vector
  private boolean result_ready;  // flag - set up by the worker
                               //   who finished the operation
  private boolean stop_workers;  // flag to finish workers
 
  private Cl_worker[] worker;  // references to worker threads


  // monitor's public methods - called from clients

  public Cl_parop (int n_proc, int par_step) { // constructor
    n = n_proc;
    step = par_step;
    free_work = false;
    op_running = false;
    stop_workers = false;
    barrier_cnt = 0;
    worker = new Cl_worker[n];
    for (int i = 0; i < n; i++) {
      worker[i] = new Cl_worker (this, i);
      worker[i].start();
    }
  }

  // operation
  public synchronized double op (int par_m, VypData par_data) {

    while (op_running) try {    // it blocks other clients
      wait ();
    } catch (InterruptedException e) {};

    m = par_m;					// Celkovy pocet bodu
    data = par_data;
    B = new Plocha(data.fce);	// Nastaveni cisla funkce
//	B.set_cislo(data.fce);
    oblast = new Obdelnik(data.oblast);
    glob_suma = 0;
    glob_pocet = 0;
    op_result = 0;
    next_index = 0;
    op_running = true;
    free_work = true;
    result_ready = false;

    notifyAll();                // it awakes workers

    while (op_running) try {    // it blocks the calling client
      wait ();
    } catch (InterruptedException e) {};

    op_result = glob_suma / glob_pocet;	// Prumerna hodnota
    return op_result;
  }

  // method to kill workers - use it only at the end of program !!!
  public double stop () {

    stop_workers = true; 
    return op(0, data);

  }
  
  public int get_n () {return n;} // returns num of workers

  public int get_step () {return step;}  // returns size of part

  // method to set size of part
  public synchronized void set_step (int par_step) {
    step = par_step;
  }


  // private methods - called from workers

  // method returns a descriptor of vector part to be processed
  private synchronized Cl_part_work get_work () {

    while (!op_running) try {   // it blocks workers beyond
      wait ();                  //   time of operation
    } catch (InterruptedException e) {};

    Cl_part_work part;

    int pocet = step; 
    if ((next_index + step) > m) {
      step = m - next_index + 1;
      free_work = false;
    }
    part = new Cl_part_work(data, pocet);
    next_index = next_index + step;

    return part;
  }

  // method updates global result by adding a local result
  private synchronized void inc (double par_suma, int par_pocet) {

    glob_suma += par_suma;	// Pricteni loc_sumy ke glob_sume
    glob_pocet += par_pocet;    // Z kolika bodu byla loc_suma spocitana

    if (!free_work) {
      barrier_cnt++;
      if (barrier_cnt == n) {
        result_ready = true;
        notifyAll();
      }
      while (!result_ready) try {  // sync. of workers before
        wait ();              // the end of operation
      } catch (InterruptedException e) {};
      barrier_cnt--;
      if (barrier_cnt == 0) {
        op_running = false;
        notifyAll();
      }
    }
  }

}
