package bin;
/** 
 * monitor - Farmer part from model 'farmer-worker' - creates workers (threads)
 * and set work  
 */
public class Farmer {
	private int k; // interval <0,k> for converting brightness
	private int p; // number of processes
	private int barrier_cnt;     // counter of sync. barrier
	private int step;            // vector part length
	private int min,max;    // minimum/maximum value in the array
	private int next_index;      // first index of first free part
	private boolean is_work;     // flag - there is still something to do
	private boolean stop_workers;  // flag to finish workers
	private int[] img; // data to be processed
	private WorkerMinMax[] workersMM; // workers that will find min&max values
	private Worker[] workers; // workers that will compute new brightness values
	
	public Farmer() {
		this.k = -1;
		this.p = -1;
		this.step = -1;
	}
	
	/** 
	 * inicialization - already starts first group of workers,
	 * but they will wait until they are asked to do some work
	 * (is_work is set to true)
	 */
	public void InitWorkers() {
		
	}
	
	/* k - for conversion from <min,max> to <0,k> */
	public void setK(int k) {
		this.k = k;
	}
	public int getK() {
		return k;
	}
	
	public void setStep(int s) {
		this.step = s;
	}
	
	public int getStep() {
		return step;
	}
	
	public int getMax() {
		return max;
	}
	
	public int getMin() {
		return min;
	}
	
	public void setP(int p) {
		this.p = p;
	}
	public int getP() {
		return p;
	}
	
	public void setImg(int[] img) {
		this.img = img;
	}
	
//	 method to stop workers - use it carefully - monitor can't than be used anyway !!!
	  public synchronized int stop () {
	    stop_workers = true;
	    this.notifyAll();
	    return 0;
	  }
	  
	  // method blocks workers until some work is ready
	  public synchronized boolean wait_for_work() {
	    while ((!is_work)&&(!stop_workers)) try {
	      this.wait ();
	    } catch (InterruptedException e) {};

	    if (stop_workers) {
	      return false;  // workers end their working loop
	    }
	    else {
	      return true;  // worker has something to do
	    }
	  } 
	
	  // returns a descriptor of vector part to be processed
	  public synchronized Part get_work () {
	    Part part;
	    int end_of_part = next_index + step;

	    if (end_of_part >= img.length) {
	      end_of_part = img.length;
	    }

	    part = new Part(img, next_index, end_of_part);
	    next_index = end_of_part;

	    return part;
	  }
	  
	  // method updates global result by adding a local result,
	  public synchronized void setMinMax (int res_min, int res_max) {
	    if(res_min < min)
	    	min = res_min;
	    if(res_max > max)
	    	max = res_max;
	    barrier_cnt++;
	    while (barrier_cnt < p) {  // first compute min&max
	        try {                   // sleep until everybody send its work
	            this.wait ();
	        } catch (InterruptedException e) {};
	    }

	    is_work = false;            // so there is no work left
	    	this.notifyAll();                // wake everybody to check status
	    this.stop();
	  }
	  
	  public synchronized void inc () {
	    barrier_cnt++;
	    while (barrier_cnt < p) {  // first compute min&max
	        try {                   // sleep until everybody send its work
	            this.wait ();
	        } catch (InterruptedException e) {};
	    }

	    is_work = false;            // so there is no work left
	    this.stop();
	  }	  

	public void convert(int[] img) {
		this.min = -1;
		this.max = -1;
		is_work = true; // if there is some work 
	    stop_workers = false; // after workers do their job, this is set to true
	    barrier_cnt = 0; // barrier to synchronize workers
	    this.img = img;
		next_index = 0;

	    /* first, it is useless to create more workers than all_amount/one_amount */
	    if(img.length/step+1 < p) {
	    	if(img.length%step == 0)
	    		p = img.length/step;
	    	else 
	    		p = img.length/step+1;
	    }
	    /* create and start first group of workers */
	    workersMM = new WorkerMinMax[p];
	    for (int i = 0; i < p; i++) {
	      workersMM[i] = new WorkerMinMax (this);
	      workersMM[i].start();
	    }
		
	    try {
	    	for (int i = 0; i < workersMM.length; i++) {
				this.workersMM[i].join();
			}
	    }catch(Exception e) {};
	    
	    /* now I know min&max, so I can recompute new values */
	    stop_workers = false;
	    barrier_cnt = 0;
	    next_index = 0;
	    is_work = true;
	    
	    workers = new Worker[p];
	    for (int i = 0; i < p; i++) {
	      workers[i] = new Worker(this);
	      workers[i].start();
	    }
	    try {
	    	for (int i = 0; i < workers.length; i++) {
				this.workers[i].join();
			}
	    }catch(Exception e) {};
	}
}
