package bin;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;

/**
 *  GUI 
 **/
public class Form extends JFrame {
	private static final long serialVersionUID = 1L;
	
	private int[] imgVals; // values that will be recomputed
	private LoadImage li; // to load&save image, convert to grayscale
	private JPanel jp; // to draw image on the screen
	private Farmer farmer; // to recompute image values by using model farmer-worker
	
	public Form() {
		li = new LoadImage();
		farmer = new Farmer();
		
		/* inicialization of GUI - create window, menu,... */
		this.setTitle("Image convertor");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setSize(800,580);
		jp = new JPanel();
		this.setLayout(new BorderLayout());
		this.add(jp);
		this.setJMenuBar(createMenuBar());
		this.setLocationRelativeTo(null);
		this.setVisible(true);
	}
	
	public int[] getImgVals() {
		return this.imgVals;
	}
	
	public void setK(int k) {
		farmer.setK(k);
	}
	
	public void setP(int p) {
		farmer.setP(p);
	}
	
	public void setA(int a) {
		farmer.setStep(a);
	}
	
	/* called when user chooses "convert image" in menu - iniciate farmer to convert image values */
	protected void convert() {
		if(farmer.getK() == -1) {
			JOptionPane.showMessageDialog(this,(Object)"k not set - implicit value of 150");
			farmer.setK(150);
		}
		if(farmer.getP() == -1) {
			JOptionPane.showMessageDialog(this,(Object)"p not set - implicit value of 5");
			farmer.setP(5);
		}
		if(farmer.getStep() == -1) {
			JOptionPane.showMessageDialog(this,(Object)"amount of work not set - implicit value of 1000");
			farmer.setStep(1000);
		}
		farmer.InitWorkers();
		farmer.convert(this.imgVals);
	
		this.li.setImage(this.imgVals); // rewrite original image values
		
		/* repaint image on the screen */
		Image i = li.getImage();
		if(i != null) {
			jp.removeAll();
			ImageIcon img = new ImageIcon(i);
			JLabel jl = new JLabel(img);
			jp.add(jl);
			this.validate();
			this.repaint();
		}
	}
	
	/* loads image values and draw image on the screen */
	protected void openFile(File f) {
		try {
			this.imgVals = li.Load(f);
		}catch(Exception e) {
			JOptionPane.showMessageDialog(this,(Object)"Missing or corrupted file!","Error",JOptionPane.ERROR_MESSAGE);
		}
		Image i = li.getImage();
		if(i != null) {
			this.jp.removeAll();
			ImageIcon img = new ImageIcon(li.getImage());
			JLabel jl = new JLabel(img);
			jp.add(jl);
			this.validate();
			this.repaint();
		}
	}
	
	/* save current image values */
	public void saveFile(File f) {
		try {
			li.Write(imgVals, f);
		}catch(Exception e) {
			System.out.println("Exception when saving the file");
		}
	}
	
	private JMenuBar createMenuBar() {
        JMenuBar menuBar;
        JMenu menu;
        JMenuItem menuItem;
        ImageIcon icon;
        
        menuBar = new JMenuBar();

        menu = new JMenu("File");
        menuBar.add(menu);

        icon = createIcon("openFile");
        menuItem = new JMenuItem("Open File", icon);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new OpenFileAct(this));
        menu.add(menuItem);
        
        icon = createIcon("convert");
        menuItem = new JMenuItem("Convert", icon);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ConvertAct(this));
        menuItem.setToolTipText("Recompute brightness values from interval <min,max> to <0,k>");
        menu.add(menuItem);
        
        icon = createIcon("saveFile");
        menuItem = new JMenuItem("Save File", icon);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new SaveFileAct(this));
        menu.add(menuItem);
        
        icon = createIcon("exit");
        menuItem = new JMenuItem("Exit",icon);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
        		KeyEvent.VK_ESCAPE, 0));
        menuItem.addActionListener(new ActionListener() {
	    	public void actionPerformed(ActionEvent arg0) {
	    		System.exit(0);
    	}});
        menu.add(menuItem);
       
        menu = new JMenu("Parametres");
        menuBar.add(menu);

        menuItem = new JMenuItem("No. of processes");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new SetPAct(this));
        menuItem.setToolTipText("how many processes will compute the values for this image");
        menu.add(menuItem);
        
        menuItem = new JMenuItem("Amount of work");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new SetAAct(this));
        menuItem.setToolTipText("Amount of work set to one process (model farmer-worker)");
        menu.add(menuItem);

        menuItem = new JMenuItem("k");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_K, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new SetKAct(this));
        menuItem.setToolTipText("maximum value of brightness (after conversion)");
        menu.add(menuItem);
        
        menu = new JMenu("Help");
         menuBar.add(menu);
        
        icon = createIcon("about");
        menuItem = new JMenuItem("About Image convertor",icon);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_F11, 0));
        menuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		JOptionPane.showMessageDialog(Form.this, ProgramInfo(),"Author",JOptionPane.INFORMATION_MESSAGE); 
        	}});
        menu.add(menuItem);
        
        icon = createIcon("author");
        menuItem = new JMenuItem("Author",icon);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        menuItem.addActionListener(new ActionListener() {
        	public void actionPerformed(ActionEvent e) {
        		JOptionPane.showMessageDialog(Form.this, AutorInfo(),"Author",JOptionPane.INFORMATION_MESSAGE); 
        	}});
        menu.add(menuItem);
        
        return menuBar;
    }
	
	/* load icon from file by using only its name */
	private ImageIcon createIcon(String name) {
		URLClassLoader urlClassLoader = (URLClassLoader) this.getClass().getClassLoader();
		URL url = urlClassLoader.findResource("icons/" + name+".gif");
		if(url == null)
		  return null;
		return new ImageIcon(url);
	}
	
	/* creates string with information about author */
	private String AutorInfo() {
		return "<html>Author: <font color=\"blue\">Martina Malkova</font><br> " + 
	    "Created: 25. 12. 2006<br>" +
	    "Version: 1.1" +
	    "</html>";
	}
	
	/* creates string with information about this program */
	private String ProgramInfo() {
		return "<html><font color=\"blue\">Purpose:</font><br>" +
				"Find min&max brightness in the image <br>" +
				"and convert it to interval 0-k. <br>" +
				"This is done by using p processors." +
	    "</html>";
	}
}

/* what happens when user chooses 'open file' in the menu */
class OpenFileAct extends AbstractAction {
	private static final long serialVersionUID = 1L;
	Form form;
	
	public OpenFileAct(Form f) {
		this.form = f;
	}

	public void actionPerformed(ActionEvent arg0) {
		JFileChooser fc = new JFileChooser();
		File f = new File(".");
		PictureFilter pf = new PictureFilter(); 
		fc.setAcceptAllFileFilterUsed(false);
		fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		fc.setFileFilter(pf);
		fc.setCurrentDirectory(f);
	    int returnVal = fc.showOpenDialog(form);
	    f = fc.getSelectedFile();
	    
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			form.openFile(f);
		}else
			JOptionPane.showMessageDialog(form, (Object)("Cannot open file: wrong format"),"Error",JOptionPane.ERROR_MESSAGE);
	}
}

/* what happens when user chooses 'save file' in the menu */
class SaveFileAct extends AbstractAction {
	private static final long serialVersionUID = 1L;
	Form form;
	
	public SaveFileAct(Form f) {
		this.form = f;
	}

	public void actionPerformed(ActionEvent arg0) {
		JFileChooser fc = new JFileChooser();
		PictureFilter pf = new PictureFilter(); 
		fc.setFileFilter(pf);
		
		File f = new File(".");
	    fc.setCurrentDirectory(f);
	    int returnVal = fc.showSaveDialog(form);
	    f = fc.getSelectedFile();
	    f = new File(checkFile(f.getName()));
	    if(form.getImgVals() == null)
			JOptionPane.showMessageDialog(form, (Object)("Cannot save file: no input"),"Error",JOptionPane.ERROR_MESSAGE);
	    else if (returnVal == JFileChooser.APPROVE_OPTION) {
			form.saveFile(f);
		}
		else
			JOptionPane.showMessageDialog(form, (Object)("Cannot save file: wrong format"),"Error",JOptionPane.ERROR_MESSAGE);
	}
	
	private String checkFile(String name) {
		if((!name.contains(".png"))&&(!name.contains(".jpg")&&(!name.contains(".bmp")))) {
			JOptionPane.showMessageDialog(form,(Object)"extension not set - implicit extension .jpg");
			return name+".jpg"; 
		}
		else
			return name;
	}
}

/* to let user choose only picture files */
class PictureFilter extends javax.swing.filechooser.FileFilter {
    public boolean accept(File file) {
        String filename = file.getName();
        return (filename.endsWith(".jpg")||filename.endsWith(".JPG")||(filename.endsWith(".png"))||filename.endsWith(".bmp")||(filename.indexOf('.')==-1));
    }
    public String getDescription() {
        return "picture files (*.jpg, *.png, *.bmp)";
    }
}

/* what happens when user chooses 'set k' in the menu */
class SetKAct extends AbstractAction {
	private static final long serialVersionUID = 1L;
	Form form;
	
	public SetKAct(Form form) {
		this.form = form;
	}
	
	public void actionPerformed(ActionEvent e) {
    	String text = JOptionPane.showInputDialog("k: ");
    	int k = 0;
    	if(text != null) {
    		try {
    			k = Integer.parseInt(text);
    		}catch(Exception ex) {
    			// wrong input format
    			JOptionPane.showMessageDialog(form,(Object)"k is supposed to be integer","Error",JOptionPane.ERROR_MESSAGE);
    		}
    		if((k > 0)&&(k<256))
    			form.setK(k);
    		else
    			JOptionPane.showMessageDialog(form,(Object)"k (brightness) is supposed to be between 0 and 255","Error",JOptionPane.ERROR_MESSAGE);
    	}
    }
}

/* what happens when user chooses 'set p' in the menu */
class SetPAct extends AbstractAction {
	private static final long serialVersionUID = 1L;
	Form form;
	
	public SetPAct(Form form) {
		this.form = form;
	}
	
	public void actionPerformed(ActionEvent e) {
    	String text = JOptionPane.showInputDialog("p: ");
    	int p = 0;
    	if(text != null) {
    		try {
    			p = Integer.parseInt(text);
    			
    		}catch(Exception ex) {
    			// wrong input format
    			JOptionPane.showMessageDialog(form,(Object)"p is supposed to be integer","Error",JOptionPane.ERROR_MESSAGE);
    		}
    		if(p > 0)
    			form.setP(p);
    		else
    			JOptionPane.showMessageDialog(form,(Object)"p is supposed to be greater than 0","Error",JOptionPane.ERROR_MESSAGE);
    	}
    }
}

/* what happens when user chooses 'set amount' in the menu */
class SetAAct extends AbstractAction {
	private static final long serialVersionUID = 1L;
	Form form;
	
	public SetAAct(Form form) {
		this.form = form;
	}
	
	public void actionPerformed(ActionEvent e) {
    	String text = JOptionPane.showInputDialog("amount: ");
    	int a = 0;
    	if(text != null) {
    		try {
    			a = Integer.parseInt(text);
    			
    		}catch(Exception ex) {
    			// wrong input format
    			JOptionPane.showMessageDialog(form,(Object)"amount is supposed to be integer","Error",JOptionPane.ERROR_MESSAGE);
    			return;
    		}
    		if(a > 0)
    			form.setA(a);
    		else
    			JOptionPane.showMessageDialog(form,(Object)"a is supposed to be greater than 0","Error",JOptionPane.ERROR_MESSAGE);
    	}
    }
}

/* what happens when user chooses 'convert' in the menu */
class ConvertAct extends AbstractAction {
	private static final long serialVersionUID = 1L;
	Form form;
	
	public ConvertAct(Form form) {
		this.form = form;
	}
	
	public void actionPerformed(ActionEvent e) {
		if(form.getImgVals() != null)
			form.convert();
    }
}