package bin;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;
/**
 * Loads arbitrary image and converts it to grayscale, writes images.
 * @author Martina Malkova
 */
public class LoadImage {
	private BufferedImage bufIm;
	private DirectColorModel cm;
	
	public LoadImage() {
	}
	
	public Image getImage() {
		if(bufIm != null)
			return Toolkit.getDefaultToolkit().createImage(bufIm.getSource());
		else 
			return null;
	}
	
	/**
	 *  set new values - it consists only from one value (blue) 
	 *  - we need to add it as rgb, where r=g=b 
	 **/
	public void setImage(int[] imgBright) {
		Byte b;
		
		for (int i = 0; i < imgBright.length; i++) {
			b = (byte)imgBright[i];
			bufIm.setRGB(i%bufIm.getWidth(), i/bufIm.getWidth(), b + (b << 8)+(b <<16)); // creating rgb integer where r=g=b
		}
		
	}
	
	/** loads image, converts to greyscale 
	 * @return field with grayscale values for each pixel */
	public int[] Load(File f) throws IOException{
		try {
		bufIm = ImageIO.read(f); // read image into memory
		}catch(IOException e) {
			throw e;
		}
		
		int[] brightVals = new int[bufIm.getWidth()*bufIm.getHeight()]; // for conversion, we will need only the brightness value, not whole rgb
		brightVals = bufIm.getRGB(0,0,bufIm.getWidth(),bufIm.getHeight(),null,0,bufIm.getWidth()); // read values into our field
		cm = new DirectColorModel(32,0x00ff0000,0x0000ff00,0x000000ff); // model to read separate colors (r,g,b) from integer representing rgb		/* convert rgb values to greyscale values */
		
		/* convert to grayscale */
		for (int i = 0; i < brightVals.length; i++) {
			brightVals[i] = GetGrayscale(brightVals[i], cm);
			bufIm.setRGB(i%bufIm.getWidth(), i/bufIm.getWidth(), brightVals[i]);
			brightVals[i] = cm.getBlue(brightVals[i]); 
		}
		/* set new values - to be able to draw image on the screen */
		
		return brightVals; // return brightness values - to be able to recompute them 
	}
	
	/**
	 * Writes image (already known height/width) - sets new values to the image 
	 * @param br - new grayscale values
	 * @param fileName - name of the file to write (e.g. "image.bmp")
	 * @param type - type of the file (e.g "bmp")
	 * @throws IOException 
	 */
	public void Write(int[] br, File f) throws IOException {
		try {
		ImageIO.write(bufIm, f.getName().substring(f.getName().indexOf('.')+1), f); // second parametar needs to be "png","bmp",..
		System.out.println(f.getName().substring(f.getName().indexOf('.')));
		}catch(IOException e) {
			throw e; // send to the upper level
		}
	}
	
	/* converts RGB to grayscale */
	 private int GetGrayscale(int rgb, DirectColorModel cm)
     { 
		 Byte r =(byte)(0.299 *cm.getRed(rgb)  + 0.587 * cm.getGreen(rgb) + 0.114 * cm.getBlue(rgb));
		 Byte g =(byte)(0.299 *cm.getRed(rgb)  + 0.587 * cm.getGreen(rgb) + 0.114 * cm.getBlue(rgb));
		 Byte b =(byte)(0.299 *cm.getRed(rgb)  + 0.587 * cm.getGreen(rgb) + 0.114 * cm.getBlue(rgb));
		 
		return r + (g << 8)+(b <<16);
     }
}
