package ppr.image;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

public class Bitmap {
	public static final int HEADER_LENGTH = 54;
	public static final int BYTES_PER_PIXEL = 3;
	
	private byte[] header;
	private FileInputStream bitmapFile;
	private long length;
	
	public Bitmap(String fileName)throws Exception{
		File f = new File(fileName);
		this.length = f.length() - Bitmap.HEADER_LENGTH;
		this.bitmapFile = new FileInputStream(f);
	}
	
	private void loadHeader()throws IOException{
		this.header = new byte[Bitmap.HEADER_LENGTH];
		this.bitmapFile.read(this.header);		
	}
	
	public boolean checkBitmap()throws IOException{
		this.loadHeader();
		
		if(this.header[0] != 0x42 || this.header[1] != 0x4D)
			return false;
		
		if(((this.header[28] << 8) | this.header[29]) != 0x1800)
			return false;
		
		if(((this.header[30] << 24) | (this.header[31] << 16) | (this.header[32] << 8) | this.header[33]) != 0x0)
			return false;
		
		return true;
	}
	
	public byte[][] loadData(int pixels)throws IOException{
		int bytes = pixels * Bitmap.BYTES_PER_PIXEL;
		int mod = (int)(this.length%bytes);
		int size = (int)(this.length/bytes);
		
		if(mod != 0) size++;
		
		if(this.header == null) this.loadHeader();
		
		byte[][] data = new byte[size][bytes];
		
		for(int i = 0; i < data.length; i++){
			this.bitmapFile.read(data[i]);
		}
		
		return data;
	}
	
	public void saveBitmap(String file,byte[][] data){
		try {
			FileOutputStream out = new FileOutputStream(file);
			
			out.write(this.header);
			
			for(int i = 0; i < data.length-1; i++){
				out.write(data[i]);
			}
			
			long len = data[data.length-1].length - data.length*data[data.length-1].length + this.length;
			out.write(data[data.length-1],0,(int)len);
			
			out.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void close(){
		try {
			this.bitmapFile.close();
		} catch (IOException e) {}
	}
}
