package ppr.threads.app;

import java.util.ArrayList;

/**
 * Trida pro spravu vypocetnich vlaken. 
 *
 */
public class ThreadManager extends Thread {
	private Parameters parameters = null; // parametry vypoctu
	private ArrayList<ThreadWeight> threads = null;  // vypocetni vlakna
	private long startTime = 0;  // zacatek vypoctu
	private long endTime = 0;  // konec vvpoctu
	
	private float countedWork = 0;  // spoctena prace
	private float totalWeight = 0;  // celkova vaha
		
	/**
	 * Vytvori instanci spravce.
	 * @param parameters parametry ulohy
	 */
	public ThreadManager(Parameters parameters) {
		this.parameters = parameters;		
	}
	
	/**
	 * 
	 */
	public void run() {
		this.createThreads();
		this.startThreads();
		
		int deadThred = 0;  // pocet zabitych vlaken
		
		startTime = System.currentTimeMillis(); // zacatek vypoctu
		
		while (deadThred < threads.size()) {   // dokud vlakna pocitaji
			for (ThreadWeight threadWeight : threads) {  // cykl pres vypocetni vlakna
				switch (threadWeight.getStateThread()) {  // kontrola stavu vlakna
					case COUNTED:  // vlakno dokoncilo vypocet
						Result result = threadWeight.getResult();  // vysledek vypoctu
						totalWeight += result.getValue();  // scitani mezivysledku
						// neni zde break, protoze lze vyuzit nasledujici cast kodu
					case NEW:   // vlakno je nove nebo je vysledek spocten
						if (countedWork < 100f)  // kontrola jestli je co pocitat
							setWork(threadWeight); // nastaveni pracovni davky
						else {
							threadWeight.setStateWork(StateWork.DIE);   // neni co pocitat --> ukonceni vlakna
							deadThred++;
						}
					break;
				}
			}
		}		
		endTime = System.currentTimeMillis();  // konec vypoctu
	}
	
	/**
	 * Vybranemu vypocetnimu vlaknu nastavi parametry pro vypocet.
	 * @param threadWeight vypocetni vlakno
	 */
	private void setWork(ThreadWeight threadWeight) {
		Work work = new Work(parameters.getRo(), parameters.getD());  // davka k vypoctu
		
		// krajni body vnitrniho kvadru
		Point3D startPoint = new Point3D();  
		Point3D endPoint = new Point3D();
				
		startPoint.setX(parameters.getX() * (countedWork / 100f));  //  vypocet prvni x-souradnice vnitrniho kvadru
		
		if ((countedWork + parameters.getPart()) <= 100f)  // kontrola jestli neprekrocime okraje ulohy 
			countedWork += parameters.getPart();
		else
			countedWork = 100f;
		
		endPoint.setX(parameters.getX() * (countedWork / 100f));	//  vypocet druhe x-souradnice vnitrniho kvadru

		// y a z se pocitaji az k okrajum.
		endPoint.setY(this.parameters.getY());
		endPoint.setZ(this.parameters.getZ());
		
		work.setStartPoint(startPoint);  
		work.setEndPoint(endPoint);
		
		threadWeight.setWork(work);  // nastaveni davky vlaknu
		threadWeight.setStateWork(StateWork.SET);   // nastaveni priznaku, ze je davka nastavena a vlakno muze pocitat
	}
	
	/**
	 * Podle parametru ulohy vytvori urceny pocet vlaken.
	 *
	 */
	private void createThreads() {
		this.threads = new ArrayList<ThreadWeight>();
		
		for (int i = 0; i < parameters.getCountThreads(); i++)
			threads.add(new ThreadWeight());
	}	
	
	/**
	 * Spusti vsechny vypocetni vlakna.
	 *
	 */
	private void startThreads() {
		for (ThreadWeight threadWeight : threads)
			threadWeight.start(); // spusteni vlaken
	}

	/**
	 * Vraci cas, kdy byl vypocet ukoncen.
	 * @return systemovy cas v ms  
	 */
	public long getEndTime() {
		return endTime;
	}

	/**
	 * Vraci cas, kdy byl vypocet zahajen.
	 * @return systemovy cas v ms  
	 */
	public long getStartTime() {
		return startTime;
	}

	/**
	 * Vraci celkovou hmotnost kvadru.
	 * @return hmotnost
	 */
	public float getTotalWeight() {
		return totalWeight;
	}
}