package ppr.threads.app;


/**
 * Vypocetni vlakno. Ze zadane davky (kvadru) spocita objem.  
 *
 */
public class ThreadWeight extends Thread {
	private StateWork stateThread = null; // stav vlakna 
	private Result result = null;	 // vysledek vypoctu
	private Work work = null;  // pracovni davka
	
	/**
	 * Vytvori instanci vypocetniho vlakna.
	 *
	 */
	public ThreadWeight() {
		this.stateThread = StateWork.NEW; // nastaveni stavu vlakna
	}
	
	@Override
	public void run() {
		while (this.getStateThread().compareTo(StateWork.DIE) != 0) {  // dokud je nejaka prace
			if (this.getStateThread().compareTo(StateWork.SET) == 0) {  // pracovni davka nastavena
				float weight = 0;
				
				//System.out.println(this.getName());
				
				this.result = new Result();  // instance objektu pro vysledek
				this.result.setTimeStart(System.currentTimeMillis());  // zacatek casu vypoctu
				
				for (float x = work.getStartPoint().getX(); x < work.getEndPoint().getX(); x += work.getD()) {		
					for (float y = work.getStartPoint().getY(); y < work.getEndPoint().getY(); y += work.getD()) {
						for (float z = work.getStartPoint().getZ(); z < work.getEndPoint().getZ(); z += work.getD()) {
							float mX = x;
							float mY = y;
							float mZ = z;
							
							float dX = work.getD();
							float dY = work.getD();
							float dZ = work.getD();
							
							// osetreni pripadu, pokud je soucasna pozice + integracni krok vetsi nez okraj zadaneho vypoctu
														
							if  ((x + work.getD()) >  work.getEndPoint().getX())								
								dX = mX = work.getEndPoint().getX() - x;
														
							if  ((y + work.getD()) >  work.getEndPoint().getY())								
								dY = mY = work.getEndPoint().getY() - y;
								 							
							if  ((z + work.getD()) >  work.getEndPoint().getZ())								 
								dZ = mZ = work.getEndPoint().getZ() - z;
												
							// vypocet vahy
							weight += dX * dY * dZ * work.getFncRo().getRo(mX + dX / 2f, mY + dY / 2f, mZ + dZ / 2f);							
						}	
					}
				}
					
				this.result.setValue(weight);  // ulozeni vysledku
				this.result.setTimeEnd(System.currentTimeMillis());  // konec vypoctu
				this.setStateWork(StateWork.COUNTED);  // nastaveni priznaku ze je vse spocteno
			}
			
			try {
				Thread.sleep(0); // uspani...
			} catch (InterruptedException e) {}
		}
	}
	
	/**
	 * Vraci stav vlakna.
	 * @return
	 */
	public StateWork getStateThread() {		
		return stateThread;
	}

	/**
	 * Nastavi stav vlakna. Ridici vlakno na konci vypoctu nastavi StateWork.DIE a vlakno skonci.
	 * @param state novy stav
	 */
	public void setStateWork(StateWork state) {
		synchronized (state) {
			this.stateThread = state;
		}
	}

	/**
	 * Vrati vysledek vypoctu.
	 * @return vysledek vypoctu
	 */
	public Result getResult() {
		return result;
	}
	
	/**
	 * Nastaveni pracovni davky.
	 * @param work 
	 */
	public void setWork(Work work) {
		this.work = work;
	}
}